!     Last change:  JD   23 Jan 2001    0:37 am
MODULE DEFN

! -- File DEFN.F90 contains definitions for defined types and global variables.


!****************************************************************************
! defined types        
!****************************************************************************
	
	type modelgrid
	  integer                         :: nrow,ncol
	  double precision                :: east_corner,north_corner,rotation
	  real                            :: cosang,sinang
	  real, dimension(:), pointer     :: delr,delc
	  integer                         :: specunit,specline
	  character (len=80)              :: specfile
	end type modelgrid


        integer, parameter                :: MAX_STRUCT_VARIO=5
        type geostructure
          character (len=10)              :: structname
          integer                         :: numvariogram,transform
          real                            :: nugget,mean,maxpowercov
          real                            :: variogram_contrib(MAX_STRUCT_VARIO)
          character (len=10)              :: variogram_name(MAX_STRUCT_VARIO)
        end type geostructure

        type variogram
          character (len=10)   :: varname
          integer              :: vartype
          real                 :: angle,a,anis
        end type variogram


        integer, parameter                :: MAX_OUT_PARAM=100, &
                                             MAX_IN_PARAM=20
        type mf2k_param_replace
          character (len=10)              :: replacetype
          integer                         :: numout,numin
          character (len=10)              :: outparamname(MAX_OUT_PARAM)
          integer                         :: outparamlay1(MAX_OUT_PARAM), &
                                             outparamlay2(MAX_OUT_PARAM)
          integer                         :: newparamlay1(MAX_IN_PARAM), &
                                             newparamlay2(MAX_IN_PARAM)
          character (len=3)               :: newparamprefix
          integer                         :: transformtype
          real                            :: minival,maxival,minpval,maxpval
          character (len=120)             :: minifile,maxifile
          real, dimension(:,:), pointer   :: miniarray,maxiarray
        end type mf2k_param_replace


!****************************************************************************
!global variables
!****************************************************************************

!variables for reading a file ------->

	integer, parameter              	:: NUM_WORD_DIM=100
	integer, dimension(NUM_WORD_DIM)        :: left_word,right_word
	character (len=300)             	:: cline


!variables for writing a message ------->
	
	integer                 :: imessage=0
	character (len=500)     :: amessage= ' '
	character (len=200)     :: initial_message=' '


!escape variables ------->

	integer                 :: escset=0
	character (len=5)       :: eschar = 'E ~e '


!variables in bore data manipulation ------->
	
	integer                         :: num_bore_coord, num_bore_list
	character (len=120)             :: bore_coord_file, bore_list_file
	integer, dimension(:), pointer			:: bore_coord_layer
	double precision, dimension(:), pointer         :: bore_coord_east, &
							   bore_coord_north
	character (len=10), dimension(:), pointer       :: bore_coord_id, &
                                                           bore_list_id,  &
                                                           bore_diff_id

!variables in pilot point manipulation ------->
	
	integer                         :: num_pilot_points
	character (len=120)             :: pilot_points_file=' '
	integer, dimension(:), pointer			:: pilot_point_zone
	double precision, dimension(:), pointer         :: pilot_point_east, &
							   pilot_point_north, &
                                                           pilot_point_val, &
                                                           pilot_point_val1
	character (len=10), dimension(:), pointer       :: pilot_point_id

!Variables used in characterisation of a gms 2D mesh

        integer                                 :: numnode_g
        integer                                 :: numelem_g
        character (len=200)                     :: meshfile_g
        integer, dimension(:), pointer          :: nodenum_g(:)
        double precision, dimension(:), pointer :: eastnode_g(:), northnode_g(:)
        integer, dimension(:), pointer          :: elemnum_g(:),elemindex_g(:)
        integer, dimension(:), pointer          :: elemnode_g(:,:)
        double precision, dimension(:), pointer :: eastelem_g(:), northelem_g(:)

!       Note: integers in elemindex indicate the order in a sorting list of each element number.
!             That is elemnode(elemindex(3)) is the element with third lowest element number.

!Variables used in characterisation of a FEFLOW mesh

        integer                                 :: numnode_f
        integer                                 :: numelem_f
        character (len=200)                     :: epfile_f
        double precision, dimension(:), pointer :: eastelem_f(:), northelem_f(:)
        double precision, dimension(:), pointer :: datelem_f(:)
        integer, dimension(:), pointer          :: zoneelem_f(:)

!variables in MODFLOW 2000 parameter replacement ------->

        integer                     :: numparamreplace, numzoneremove, nummultremove
        character (len=12), dimension(:), pointer        :: removezone, removemult
        character (len=120)                              :: repfile
        type (mf2k_param_replace), dimension(:), pointer :: replace


!variables recording data settings ------->

	integer				:: datespec
	character (len=3)               :: headerspec


END MODULE DEFN
!     Last change:  J    14 Jun 2002   10:07 pm
MODULE INTER

! -- Contains interface blocks for all subprograms.


!******************************************************************************
! generic subprograms
!******************************************************************************

interface char2num

	subroutine a2i(ifail,string,num)
	  integer, intent(out)          :: ifail
	  character (len=*), intent(in) :: string
	  integer, intent(out)          :: num
	end subroutine a2i
	subroutine a2r(ifail,string,num)
	  integer, intent(out)          :: ifail
	  character (len=*), intent(in) :: string
	  real, intent(out)             :: num
	end subroutine a2r
	subroutine a2d(ifail,string,num)
	  integer, intent(out)          :: ifail
	  character (len=*), intent(in) :: string
	  double precision, intent(out) :: num
	end subroutine a2d

end interface


interface num2char

	subroutine i2a(value,string,nchar)
	  integer, intent(in)           :: value
	  character (len=*), intent(out):: string
	  integer, intent(in), optional :: nchar
	end subroutine i2a
	subroutine r2a(value,string,nchar)
	  real, intent(in)              :: value
	  character (len=*), intent(out):: string
	  integer, intent(in), optional :: nchar
	end subroutine r2a
	subroutine d2a(value,string,nchar)
	  double precision, intent(in)  :: value
	  character (len=*), intent(out):: string
	  integer, intent(in), optional :: nchar
	end subroutine d2a

end interface


interface pos_test

	integer function pos_i_test(value,string)
	  integer, intent(in)           :: value
	  character (len=*), intent(in) :: string
	end function pos_i_test
	integer function pos_r_test(value,string)
	  real, intent(in)              :: value
	  character (len=*), intent(in) :: string
	end function pos_r_test
	integer function pos_d_test(value,string)
	  double precision, intent(in)  :: value
	  character (len=*), intent(in) :: string
	end function pos_d_test

end interface


interface nneg_test

	integer function nneg_i_test(value,string)
	  integer, intent(in)           :: value
	  character (len=*), intent(in) :: string
	end function nneg_i_test
	integer function nneg_r_test(value,string)
	  real, intent(in)              :: value
	  character (len=*), intent(in) :: string
	end function nneg_r_test
	integer function nneg_d_test(value,string)
	  double precision, intent(in)  :: value
	  character (len=*), intent(in) :: string
	end function nneg_d_test

end interface


interface key_read

	integer function int_key_read(value)
	  integer,intent(out)   :: value
	end function int_key_read
	integer function real_key_read(value)
	  real,intent(out)      :: value
	end function real_key_read
	integer function double_key_read(value)
	  double precision,intent(out)  :: value
	end function double_key_read

end interface



interface equals

       logical function equals_int(r1,r2)
          integer, intent(in)    :: r1
          integer, intent(in)    :: r2
       end function equals_int

       logical function equals_real(r1,r2)
          real, intent(in)      :: r1
          real, intent(in)      :: r2
       end function equals_real

       logical function equals_dbl(r1,r2)
          real (kind (1.0d0)), intent(in)      :: r1
          real (kind (1.0d0)), intent(in)      :: r2
       end function equals_dbl

end interface



!******************************************************************************
! other subprograms
!******************************************************************************

! dxf subprograms ------->

interface
	subroutine write_dxf_header(iunit,xlo,xhi,ylo,yhi)
	  integer, intent(in)             :: iunit
	  double precision, intent(in)    :: xlo,xhi,ylo,yhi
	end subroutine write_dxf_header

	subroutine write_dxf_polyhead(iunit)
	  integer, intent(in)     :: iunit
	end subroutine write_dxf_polyhead

	subroutine write_dxf_vertex(iunit,x,y)
	  integer, intent(in)             :: iunit
	  double precision, intent(in)    :: x,y
	end subroutine write_dxf_vertex

	subroutine write_dxf_polyfin(iunit)
	  integer, intent(in)     :: iunit
	end subroutine write_dxf_polyfin

	subroutine write_dxf_finish(iunit)
	  integer,intent(in)      :: iunit
	end subroutine write_dxf_finish

end interface

! utility subprograms ------->

interface

	subroutine casetrans(string,hi_or_lo)
	  character (len=*), intent(inout)        :: string
	  character (len=*), intent(in)           :: hi_or_lo
	end subroutine casetrans

	subroutine sub_error(subname)
	  character (len=*)               ::subname
	end subroutine sub_error

	integer function nextunit()
	end function nextunit

	subroutine close_files
	end subroutine close_files

	subroutine open_input_file(ifail,aprompt,infile,inunit,file_format,form_prompt, &
          fformat)
          integer, intent(out)                     :: ifail
          character (len=*), intent(in)            :: aprompt
          character (len=*), intent(out)           :: infile
          integer, intent(out)                     :: inunit
          character (len=*), intent(in), optional  :: file_format
          character (len=*), intent(in), optional  :: form_prompt
          character (len=*), intent(out), optional :: fformat
	end subroutine open_input_file

        subroutine open_output_file(ifail,aprompt,outfile,outunit,file_format)
          integer, intent(out)                       :: ifail
          character (len=*)                          :: aprompt,outfile
          integer, intent(out)                       :: outunit
          character (len=*), intent(in), optional    :: file_format
        end subroutine open_output_file

	subroutine wrtsig(ifail,val,word,nw,precis,tval,nopnt)
	  integer               :: ifail,nw,precis,nopnt
	  double precision      :: val,tval
	  character (len=*)     :: word
	end subroutine wrtsig

	subroutine readfig(specfile,coordfile,sampfile,pumpfile,pilotfile)
	  character (len=*), intent(out)                :: specfile
	  character (len=*), intent(out), optional      :: coordfile,sampfile,&
							   pumpfile,pilotfile
	end subroutine readfig

	subroutine read_settings(ifail,idate,iheader)
	  integer, intent(out)	:: ifail,idate,iheader
	end subroutine read_settings

	subroutine char_add(astring,achar)
          character (len=*), intent(inout)        :: astring
          character (len=*), intent(in)           :: achar
	end subroutine char_add

	subroutine int2alph(inum,alph,nsig)
	  integer, intent(in)			:: inum
	  character (len=*), intent(out)	:: alph
	  integer, optional, intent(in)		:: nsig
	end subroutine int2alph

end interface

! interpolation subprograms ------->

interface

	subroutine factor(gridspec,east,north,fac1,fac2,fac3,fac4, &
	icellno,jcellno)
	  use defn
	  type(modelgrid), intent(in)           :: gridspec
	  double precision, intent(in)          :: east,north
	  real, intent(out)                     :: fac1,fac2,fac3,fac4
	  integer, intent(out)                  :: icellno,jcellno
	end subroutine factor

	subroutine point_interp(ncol,nrow,thresh,fac1,fac2,fac3,fac4, &
	icellno,jcellno,bhead,rarray,imethod)
	  integer, intent(in)                   :: ncol,nrow
	  real, intent(in)                      :: thresh
	  real, intent(in)                      :: fac1,fac2,fac3,fac4
	  integer, intent(in)                   :: icellno,jcellno
	  real, intent(out)                     :: bhead
	  real, dimension(0:ncol+1,0:nrow+1), intent(in)  :: rarray
	  character (len=*), intent(in), optional         :: imethod
	end subroutine point_interp

end interface

! reading-a-file subprograms ------->

interface

	subroutine linesplit(ifail,num)
	  integer, intent(out)            :: ifail
	  integer, intent(in)		  :: num
	end subroutine linesplit

	integer function char2int(ifail,num)
	  integer, intent(in)             :: num
	  integer, intent(out)            :: ifail
	end function char2int

	real function char2real(ifail,num)
	  integer, intent(in)             :: num
	  integer, intent(out)            :: ifail
	end function char2real

	double precision function char2double(ifail,num)
	  integer, intent(in)             :: num
	  integer, intent(out)            :: ifail
	end function char2double

        subroutine getfile(ifail,cline,filename,ibeg,iend)
          integer, intent(out)            :: ifail
          integer, intent(in)             :: ibeg
          integer, intent(inout)          :: iend
          character (len=*), intent(in)   :: cline
          character (len=*), intent(out)  :: filename
       end subroutine getfile

       subroutine addquote(afile,aqfile)
          character (len=*), intent(in)   :: afile
          character (len=*), intent(out)  :: aqfile
       end subroutine addquote

end interface

!integer array subprograms ------->

interface

	subroutine read_integer_array(ifail,aprompt,array,pm_header,rows, &
	columns,defaultfile)
	  integer, intent(out)                    :: ifail
	  character (len=*), intent(inout)        :: aprompt
	  integer, intent(out),dimension(:,:)     :: array
	  character (len=*), intent(in), optional :: pm_header
	  integer, intent(in), optional           :: rows,columns
          character (len=*), optional             :: defaultfile
	end subroutine read_integer_array

	subroutine write_integer_array(ifail,aprompt,array,pm_header,rows, &
	columns)
	  integer, intent(out)			  :: ifail
	  character (len=*), intent(inout)	  :: aprompt
	  integer, intent(in), dimension(:,:)	  :: array
	  character (len=*), intent(in), optional :: pm_header
	  integer, intent(in), optional		  :: rows,columns
	end subroutine write_integer_array

end interface

! real array subprograms ------->

interface

	subroutine read_real_array(ifail,aprompt,array,pm_header,rows,columns)
	  integer, intent(out)                    :: ifail
	  character (len=*), intent(inout)        :: aprompt
	  real, intent(out),dimension(:,:)        :: array
	  character (len=*), intent(in), optional :: pm_header
	  integer, intent(in), optional           :: rows,columns
	end subroutine read_real_array

	subroutine write_real_array(ifail,aprompt,array,pm_header,rows,columns,&
	binary_header,atype,ntrans,kstp,kper,pertim,totim,text,ncol,nrow,ilay,istage, &
        realfile,aaformat)
	  integer, intent(out)                    :: ifail
	  character (len=*), intent(inout)        :: aprompt
	  real, intent(in),dimension(:,:)         :: array
	  character (len=*), intent(in), optional :: pm_header
	  integer, intent(in), optional           :: rows,columns
          character (len=*), optional             :: binary_header
          character (len=*), optional             :: atype
          integer, optional                       :: ntrans,kstp,kper
          real, optional                          :: pertim,totim
          character (len=16),optional             :: text
          integer, optional                       :: ncol,nrow,ilay
          integer, intent(in), optional           :: istage
          character (len=*), optional             :: realfile
          character (len=*), optional             :: aaformat
        end subroutine write_real_array

	subroutine read_surf_array(ifail,aprompt,array,gridspec)
	  use defn
	  integer, intent(out)			:: ifail
	  character (len=*), intent(in)		:: aprompt
	  real, intent(out), dimension(:,:)	:: array
	  type(modelgrid), intent(in)		:: gridspec
	end subroutine read_surf_array

	subroutine write_surf_array(ifail,aprompt,array,thresh,gridspec)
	  use defn
	  integer, intent(out)			:: ifail
	  character (len=*), intent(in)		:: aprompt
	  real, intent(inout), dimension(:,:)	:: array
	  real, intent(in)			:: thresh
	  type(modelgrid), intent(in)		:: gridspec
	end subroutine write_surf_array

	subroutine write_real_table_file(ifail,outunit,outfile,intarray,realarray)
          use defn
          integer, intent(out)                    :: ifail
          integer, intent(in)                     :: outunit
          character (len=*), intent(in)           :: outfile
          integer, dimension(:,:), intent(in)     :: intarray
          real, dimension(:,:), intent(in)        :: realarray
	end subroutine write_real_table_file

end interface

! message subprograms ------->

interface

	subroutine write_initial_message(leadspace,endspace)
	  character (len=*), intent(in), optional :: leadspace,endspace
	end subroutine write_initial_message

	subroutine write_message(increment,iunit,error,leadspace,endspace)
	  integer, intent(in), optional           ::increment,iunit
	  character (len=*), intent(in), optional ::error,leadspace,endspace
	end subroutine write_message

end interface

! grid subprograms ------->

interface

	subroutine spec_open(ifail,gridspec)
	  use defn
	  integer, intent(out)	  :: ifail
	  type (modelgrid)        :: gridspec
	end subroutine spec_open

	subroutine read_spec_dim(ifail,gridspec)
	  use defn
	  type (modelgrid)                :: gridspec
	  integer, intent(out)            :: ifail
	end subroutine read_spec_dim

	subroutine read_spec_data(ifail,gridspec)
	  use defn
	  type (modelgrid)        :: gridspec
	  integer, intent(out)    :: ifail
	end subroutine read_spec_data

	subroutine close_spec_file(gridspec,ok)
	  use defn
	  type (modelgrid)                :: gridspec
	  character (len=*),optional      :: ok
	end subroutine close_spec_file

	subroutine free_grid_mem(gridspec)
	  use defn
	  type (modelgrid)        :: gridspec
	end subroutine free_grid_mem

	subroutine rel_centre_coords(east,north,gridspec)
	  use defn
	  real, intent(out), dimension(:,:)       :: east,north
	  type(modelgrid), intent(in)             :: gridspec
	end subroutine rel_centre_coords

	subroutine grid2earth(east,north,gridspec)
	  use defn
	  real, intent(inout),dimension(:,:)      :: east,north
	  type(modelgrid), intent(in)             :: gridspec
	end subroutine grid2earth

	subroutine transform_to_earth(earth_east,earth_north,grid_east, &
	grid_north,gridspec)
	  use defn
	  real, intent(out)               :: earth_east, earth_north
	  real, intent(in)                :: grid_east, grid_north
	  type(modelgrid), intent(in)     :: gridspec
	end subroutine transform_to_earth

	subroutine rc2cell(icellno,irow,icol,gridspec)
	  use defn
	  integer, intent(out)            :: icellno
	  integer, intent(in)             :: irow,icol
	  type(modelgrid), intent(in)     :: gridspec
	end subroutine rc2cell

	subroutine cell2rc(icellno,irow,icol,gridspec)
          use defn
          integer, intent(in)             :: icellno
          integer, intent(out)            :: irow,icol
          type (modelgrid), intent(in)    :: gridspec
	end subroutine cell2rc

	subroutine corner(ind,e,n,icol,irow,gridspec,ecg,ncg)
	  use defn
	  integer, intent(in)                     :: ind
	  real, intent(out)                       :: e,n
	  integer, intent(in)                     :: icol,irow
	  type (modelgrid), intent(in)            :: gridspec
	  real, dimension(:,:), intent(in)        :: ecg,ncg
	end subroutine corner


        subroutine cell_coordinates(gridspec,east,north,icellrow,icellcol,roff,coff)
          use defn
          type(modelgrid), intent(in)             :: gridspec
          double precision, intent(in)            :: east,north
          integer, intent(out)                    :: icellrow,icellcol
          real, intent(out)                       :: roff,coff
        end subroutine cell_coordinates


     
end interface

! bore data manipulation subprograms ------->

interface

	subroutine read_bore_coord_file(ifail,aprompt)
	  integer, intent(out)            :: ifail
	  character (len=*), intent(in)   :: aprompt
	end subroutine read_bore_coord_file

	subroutine read_bore_list_file(ifail,aprompt,coord_check)
	  integer, intent(out)                    :: ifail
	  character (len=*), intent(in)           :: aprompt
	  character (len=*), intent(in), optional :: coord_check
	end subroutine read_bore_list_file

	subroutine read_bore_diff_file(ifail,aprompt,coord_check)
	  integer, intent(out)                    :: ifail
	  character (len=*), intent(in)           :: aprompt
	  character (len=*), intent(in), optional :: coord_check
	end subroutine read_bore_diff_file

	subroutine free_bore_mem
	end subroutine free_bore_mem

	subroutine read_rest_of_sample_line(ifail,cols,ndays,nsecs,value, &
	iline,sampfile)
          integer, intent(out)            :: ifail
          integer, intent(in)             :: cols
          integer, intent(out)            :: ndays,nsecs
          double precision, intent(out)   :: value
          integer, intent(in)             :: iline
          character (len=*), intent(in)   :: sampfile
	end subroutine read_rest_of_sample_line

	subroutine read_rest_of_pump_line(ifail,ibore,ndays,nsecs,pumped, &
	iline,pmpfile)
          integer, intent(out)                    :: ifail
          integer, intent(inout)                  :: ibore
          integer, intent(out), dimension (:)     :: ndays,nsecs
          double precision, intent(inout), dimension(:)  :: pumped
          integer, intent(in)                     :: iline
          character (len=*), intent(in)           :: pmpfile
	end subroutine read_rest_of_pump_line

	subroutine time_interp(ifail,nbore,ndays,nsecs,value,intday, &
	intsec,rnear,rconst,valinterp,extrap,direction)
          integer, intent(out)                    :: ifail
          integer, intent(in)                     :: nbore
          integer, intent(in), dimension(nbore)   :: ndays,nsecs
          double precision, intent(in), dimension(nbore)   :: value
          integer, intent(in)                     :: intday,intsec
	  real, intent(in)			  :: rnear,rconst
          double precision, intent(out)           :: valinterp
	  character (len=*), intent(in),optional  :: extrap
	  character (len=*), intent(in),optional  :: direction
	end subroutine time_interp

	subroutine get_num_ids(ifail,iunit,afile,numid,maxsamp,ignore_x)
	  integer, intent(out)                    :: ifail
          integer, intent(in)                     :: iunit
          character (len=*), intent(in)           :: afile
          integer, intent(out)                    :: numid,maxsamp
	  character (len=*), intent(in), optional :: ignore_x
	end subroutine get_num_ids

	subroutine get_ids_and_interval(ifail,iunit,afile,nid,aid,ndays1, &
                                nsecs1,ndays2,nsecs2, ignore_x)
          integer, intent(out)                    :: ifail
          integer, intent(in)                     :: iunit
          character (len=*), intent(in)           :: afile
          integer, intent(in)                     :: nid
          character (len=*), intent(out)          :: aid(nid)
          integer, intent(out)                    :: ndays1(nid),nsecs1(nid), &
                                                     ndays2(nid),nsecs2(nid)
	  character (len=*), intent(in), optional :: ignore_x
	end subroutine get_ids_and_interval

end interface

! geostat subprograms ------->

interface

	subroutine read_pilot_points_file(ifail,aprompt,accept_blank)
	  integer, intent(out)                    :: ifail
	  character (len=*), intent(in)           :: aprompt
          character (len=*), intent(in), optional :: accept_blank
	end subroutine read_pilot_points_file

	subroutine read_dual_pilot_points_file(ifail,aprompt,accept_blank)
	  integer, intent(out)                    :: ifail
	  character (len=*), intent(in)           :: aprompt
          character (len=*), intent(in), optional :: accept_blank
	end subroutine read_dual_pilot_points_file


	subroutine free_point_mem
	end subroutine free_point_mem

        subroutine read_structure_file_dim(ifail,structunit,numstruct,numvario,structfile)
          integer, intent(out)            :: ifail
          integer, intent(in)             :: structunit
          integer, intent(out)            :: numstruct,numvario
          character (len=*), intent(in)   :: structfile
        end subroutine read_structure_file_dim

        subroutine read_rest_of_structure_file(ifail,structunit,numstruct,numvario, &
          structfile,structure,vario)
          use defn
          integer, intent(out)            :: ifail
          integer, intent(in)             :: structunit,numstruct,numvario
          character (len=*), intent(in)   :: structfile
          type (geostructure), intent(out):: structure(numstruct)
          type (variogram), intent(out)   :: vario(numvario)
        end subroutine read_rest_of_structure_file


        subroutine read_parameter_replacement_file(ifail,aprompt,ncol,nrow)
	  integer, intent(out)            :: ifail
	  character (len=*), intent(in)   :: aprompt
          integer, intent(in)             :: ncol,nrow
        end subroutine read_parameter_replacement_file


        subroutine free_replace_mem()
        end subroutine free_replace_mem

end interface


! RSM subprograms ------->

interface

        subroutine read_gms_2d_mesh_file(ifail,aprompt,centroid)
          integer, intent(out)            :: ifail
          character (len=*), intent(in)   :: aprompt
          character (len=*), intent(in)   :: centroid
        end subroutine read_gms_2d_mesh_file

        subroutine read_gms_integer_mesh_data_file(ifail,aprompt,intdata)
          integer, intent(out)             :: ifail
          character (len=*), intent(inout) :: aprompt
          integer, intent(out)             :: intdata(:)
        end  subroutine read_gms_integer_mesh_data_file

        subroutine read_gms_real_mesh_data_file(ifail,aprompt,realdata)
          integer, intent(out)             :: ifail
          character (len=*), intent(inout) :: aprompt
          real, intent(out)                :: realdata(:)
        end  subroutine read_gms_real_mesh_data_file

end interface


! FEFLOW subprograms ------->

interface

        subroutine read_feflow_elem_prop_file(ifail,aprompt)
          integer, intent(out)            :: ifail
          character (len=*), intent(in)   :: aprompt
        end subroutine read_feflow_elem_prop_file

end interface



! date manipulation subprograms ------->

interface

	subroutine char2date(ifail,adate,dd,mm,yy)
          integer, intent(out)    	:: ifail
          character (len=*), intent(in) :: adate
          integer, intent(out) 		:: dd,mm,yy
	end subroutine char2date

	subroutine datestring(dd,mm,yy,hhh,mmm,sss,time,at,adate,atime)
	  integer, intent(in)             :: dd,mm,yy,hhh,mmm,sss
	  real, intent(in)                :: time
	  character (len=1), intent(in)   :: at
	  character (len=*), intent(out)  :: adate, atime
	end subroutine datestring

	logical function leap(year)
          integer, intent(in)     :: year
	end function leap

	integer function numdays(dr,mr,yr,d,m,y)
          integer, intent(in)     :: dr,mr,yr,d,m,y
	end function numdays

	integer function numsecs(h1,m1,s1,h2,m2,s2)
	  integer, intent(in)     :: h1,m1,s1,h2,m2,s2
	end function numsecs

	subroutine char2time(ifail,adate,hh,mm,ss)
          integer, intent(out)    	:: ifail
          character (len=*), intent(in) :: adate
          integer, intent(out) 		:: hh,mm,ss
	end subroutine char2time

	subroutine time2char(ifail,hh,mm,ss,atime)
	  integer, intent(out)            :: ifail
	  integer, intent(in)             :: hh,mm,ss
	  character (len=*), intent(out)  :: atime
	end subroutine time2char

	subroutine elapsdate(eltime,dayfactor,day1,mon1,year1,hour1,min1,sec1,&
	  day2,mon2,year2,hour2,min2,sec2)
	  real, intent(in)		:: eltime,dayfactor
	  integer, intent(in)		:: day1,mon1,year1,hour1,min1,sec1
	  integer, intent(out)		:: day2,mon2,year2,hour2,min2,sec2
	end subroutine elapsdate

	subroutine newdate(ndays,day1,mon1,year1,day2,mon2,year2)
	  integer, intent(in)		:: ndays,day1,mon1,year1
	  integer, intent(out)		:: day2,mon2,year2
	end subroutine newdate

	subroutine sectime(nsecs,sec,min,hour)
	  integer, intent(in)   :: nsecs
	  integer, intent(out)  :: sec,min,hour
	end subroutine sectime

end interface


! mapinfo interface programs ------->


interface

	subroutine write_mif_file(ifail,outunit,outfile,intarray,east,north, &
		        gridspec,izone,type)

          use defn
          integer, intent(out)                    :: ifail
          integer, intent(in)                     :: outunit
          character (len=*)                       :: outfile
          integer, dimension(:,:), intent(in)     :: intarray
          real, dimension(:,:), intent(in)        :: east,north
          type (modelgrid), intent(in)            :: gridspec
          integer, intent(in)                     :: izone
          character (len=1)                       :: type
	end subroutine write_mif_file


	subroutine write_real_mid_file(ifail,outunit,outfile,intarray,realarray)
          use defn
          integer, intent(out)                    :: ifail
          integer, intent(in)                     :: outunit
          character (len=*), intent(in)           :: outfile
          integer, dimension(:,:), intent(in)     :: intarray
          real, dimension(:,:), intent(in)        :: realarray
	end subroutine write_real_mid_file


	subroutine write_integer_mid_file(ifail,outunit,outfile,intarraywin, &
	  intarraydat)
          use defn
          integer, intent(out)                    :: ifail
          integer, intent(in)                     :: outunit
          character (len=*), intent(in)           :: outfile
          integer, dimension(:,:), intent(in)     :: intarraywin,intarraydat
	end subroutine write_integer_mid_file


	subroutine read_int_tab_file(ifail,tabunit,tabfile,intarray,gridspec)
	  use defn
	  integer, intent(out)			  :: ifail
	  integer, intent(in)			  :: tabunit
	  character (len=*), intent(in)		  :: tabfile
	  integer, dimension(:,:), intent(inout)  :: intarray
	  type(modelgrid), intent(in)		  :: gridspec
	end subroutine read_int_tab_file


	subroutine read_real_tab_file(ifail,tabunit,tabfile,realarray,gridspec)
	  use defn
	  integer, intent(out)			  :: ifail
	  integer, intent(in)			  :: tabunit
	  character (len=*), intent(in)		  :: tabfile
	  real, dimension(:,:), intent(inout)     :: realarray
	  type(modelgrid), intent(in)		  :: gridspec
	end subroutine read_real_tab_file

	subroutine read_real_tab_file_int(ifail,mode,tabunit,tabfile,realarray, &
                                          intarray,gridspec,insunit,insfile,aprefix)
	  use defn
	  integer, intent(out)			  :: ifail
          integer, intent(in)                     :: mode
	  integer, intent(in)			  :: tabunit
	  character (len=*), intent(in)		  :: tabfile
	  real, dimension(:,:), intent(inout)     :: realarray
          integer, dimension(:,:), intent(inout)  :: intarray
	  type(modelgrid), intent(in)		  :: gridspec
          integer, intent(in)                     :: insunit
          character (len=*), intent(in)           :: insfile
          character (len=*), intent(in)           :: aprefix
	end subroutine read_real_tab_file_int


end interface


END MODULE INTER
!     Last change:  JD   16 Dec 2000    2:00 am
program int2real

! -- Program INT2REAL builds or modifies a MODFLOW-compatible real array
!    based on an existing MODFLOW-compatible integer array.


	use defn
	use inter

	implicit none

	integer	:: ifail,ierr,ncol,nrow,itemp,numint,currint,irow,icol,newint, &
		   i,ihuge,corrunit,iline,j,inum,idate,iheader
	real	:: rtemp
	integer :: iloc(1)
	integer, allocatable, dimension(:,:)	:: intarray
	real, allocatable, dimension(:,:)	:: oldarray,newarray
	character (len=80)			:: aprompt,intfile,corrfile
	type (modelgrid) 			:: gridspec
	character (len=1)			:: action,aintreal
	character (len=15)			:: anum1,anum2

        integer, parameter :: MAXINT=50000
	integer		   :: intval(MAXINT),iwork(MAXINT)
	real		   :: realval(MAXINT)	

	open(unit=*,carriagecontrol='list')

	write(amessage,5)
5	format(' Program INT2REAL builds or modifies a MODFLOW-compatible ',&
	'real array based on an existing MODFLOW-compatible integer array.')
	call write_message(leadspace='yes',endspace='yes')

	call read_settings(ifail,idate,iheader)
	if(ifail.eq.1) then
	  write(amessage,7)
7	  format(' A settings file (settings.fig) was not found in the ', &
	  'current directory.')
	  call write_message
	  go to 9900
	else if(ifail.eq.2) then
	  write(amessage,8)
8	  format(' Error encountered while reading settings file settings.fig')
	  call write_message
	  go to 9900
	endif
	if((iheader.ne.0).or.(headerspec.eq.' ')) then
	  write(amessage,6)
6	  format(' Cannot read array header specification from settings file ', &
	  'settings.fig')
	  call write_message
	  go to 9900
	end if

	call readfig(gridspec%specfile)
10	call spec_open(ifail,gridspec)
	if((ifail.ne.0).or.(escset.eq.1)) then
	  write(6,*)
	  go to 9900
	end if
	call read_spec_dim(ifail,gridspec)
	if(ifail.ne.0) then
	  write(6,*)
	  go to 9900
	end if
	call close_spec_file(gridspec,ok='yes')

	ncol=gridspec%ncol
	nrow=gridspec%nrow
	allocate(newarray(ncol,nrow),intarray(ncol,nrow),stat=ierr)
	if(ierr.ne.0) then
	  write(amessage,50)
50	  format(' Cannot allocate sufficient memory to run INT2REAL.')
	  go to 9890
	end if

55	write(6,*)
	aprompt=' Enter name of integer array file: '
	call read_integer_array(ifail,aprompt,intarray,pm_header=headerspec, &
        rows=nrow,columns=ncol)
	if(ifail.ne.0) go to 9900
	if(escset.eq.1) then
	  escset=0
	  write(6,*)
	  deallocate(newarray,intarray,stat=ierr)
	  if(ierr.ne.0) then
	    write(amessage,57)
57	    format(' Memory management error: cannot continue execution.')
	    go to 9890
	  end if
!	  call free_grid_mem(gridspec)
	  go to 10
	end if
	intfile=aprompt

80	write(6,*)
90	write(6,110,advance='no')
110	format(' Modify an existing real array or create a new one?  [m/c]: ')
	read(5,'(a)') action
	if(action.eq.' ') go to 90
	if(index(eschar,action).ne.0) go to 55
	call casetrans(action,'lo')
	if((action.ne.'m').and.(action.ne.'c')) go to 90
	if(action.eq.'m') then
	  allocate(oldarray(ncol,nrow),stat=ierr)
	  if(ierr.ne.0) then
	    write(amessage,50)
	    go to 9890
	  end if
	end if
150	if(action.eq.'m') then
	  aprompt=' Enter name of file holding existing real array: '
	  call read_real_array(ifail,aprompt,oldarray,pm_header=headerspec, &
          rows=nrow,columns=ncol)
	  if(ifail.ne.0) go to 9900
	  if(escset.eq.1) then
	    escset=0
	    deallocate(oldarray,stat=ierr)
	    if(ierr.ne.0) then
	      write(amessage,57)
	      go to 9890
	    end if
	    go to 80
	  end if
	end if

! -- The different integers comprising the integer array are identified.

	numint=1
	currint=intarray(1,1)
	intval(1)=currint
	row_travel: do irow=1,nrow
	  col_travel: do icol=1,ncol
	    newint=intarray(icol,irow)
	    if(newint.eq.currint) cycle col_travel
	    currint=newint
	    prev_int: do i=1,numint
	      if(newint.eq.intval(i)) cycle col_travel
	    end do prev_int
	    numint=numint+1
	    if(numint.gt.MAXINT) then
	      call num2char(MAXINT,anum1)
	      write(amessage,220) trim(anum1),trim(intfile)
220	      format(' An integer array can hold a maximum of ',a, &
	      ' different integers for proper INT2REAL execution. The array ', &
	      ' contained in file ',a,' holds more than this. To increase ',&
	      'this limit, edit INT2REAL source code, increase parameter ', &
	      'MAXINT, and recompile.')
	      go to 9890
	    end if
	    intval(numint)=newint
	  end do col_travel
	end do row_travel

! -- The identified integers are next sorted.

        iwork=0
	ihuge=huge(intval(1))
	if(numint.lt.MAXINT) then
	  do i=numint+1,MAXINT
	    intval(i)=ihuge
	  end do
	end if
	do i=1,numint
	  iloc=minloc(intval)
	  iwork(i)=intval(iloc(1))
	  intval(iloc(1))=ihuge  
	end do
	intval=iwork

	write(6,*)
180	write(6,190,advance='no')
190	format(' Enter integer-real correspondence manually or using a file? ',&
	'[m/f]: ')
	read(5,'(a)') aintreal
	if(aintreal.eq.' ') go to 180
	if(index(eschar,aintreal).ne.0) then
	  if(action.eq.'m') then
	    write(6,*)
	    go to 150
	  else
	    go to 80
	  end if
	end if
	call casetrans(aintreal,'lo')
	if((aintreal.ne.'m').and.(aintreal.ne.'f')) go to 180
	if(aintreal.eq.'f') go to 600

! -- The integers are now presented to the user.

300 	write(6,*)
305	write(6,310)
310	format(' The following integers have been detected in the integer ',&
	'array:-')
	write(6,330)
330	format(' Enter corresponding real numbers.')
	if(action.eq.'m') then
	  write(6,350)
350	  format(' Press <ENTER> if integer effects no change to existing ', &
	  'real array.')
	end if
	i=0
	iwork=-1000
355	i=i+1
	if(i.gt.numint) go to 450
	call num2char(intval(i),anum1)
360	write(6,370,advance='no') trim(anum1)
370	format('   Enter real number corresponding to integer ',a,': ')
	itemp=key_read(rtemp)
	if(escset.eq.1) then
	  escset=0
	  write(6,*)
	  if(i.gt.2) then
	    i=i-2
	    go to 355
	  else if(i.eq.2) then
	    go to 305
	  else
	    go to 180
	  end if
	end if
	if(itemp.eq.-1) then
	  if(action.eq.'c') go to 360
	  go to 355
	else if(itemp.gt.0) then
	  write(6,380)
380	  format('   Data input error  - try again.')
	  go to 360
	end if
	realval(i)=rtemp
	iwork(i)=0
	go to 355
450	go to 1000

! -- An integer-real correspondence file is next read.

600	call open_input_file(ifail, &
	' Enter name of integer-real correspondence file: ',corrfile,corrunit)
	if(ifail.ne.0) go to 9900
	if(escset.ne.0) then
	  escset=0
	  write(6,*)
	  go to 180
	end if

	iline=0
	imessage=0
	inum=0
	iwork=-1000
	read_a_line: do
	  iline=iline+1
	  read(corrunit,'(a)',err=9000,end=800) cline
	  call linesplit(ifail,2)
	  if(ifail.lt.0) cycle read_a_line
	  inum=inum+1
	  if(ifail.gt.0) then
	    if(imessage.eq.0) then
	      write(amessage,620) trim(corrfile)
620	      format(' Errors in integer-real correspondence file ',a,' ---->')
	      call write_message(leadspace='yes')
	    end if
	    call num2char(iline,anum1)
	    write(amessage,640) trim(anum1)
640	    format(' Line ',a,': insufficient items.')
	    call write_message(increment=1)
	    cycle read_a_line
	  end if
	  itemp=char2int(ifail,1)
	  if(ifail.ne.0) then
	    if(imessage.eq.0) then
	      write(amessage,620) trim(corrfile)
	      call write_message(leadspace='yes')
	    end if
	    call num2char(iline,anum1)
	    write(amessage,650) trim(anum1)
650	    format(' Line ',a,': cannot read first item on line as an integer.')
	    call write_message(increment=1)
	  else
	    do i=1,numint
	      if(itemp.eq.intval(i)) then
	        if(iwork(i).eq.0) then
	          if(imessage.eq.0) then
	            write(amessage,620) trim(corrfile)
	            call write_message(leadspace='yes')
	          end if
	          call num2char(iline,anum1)
	          call num2char(itemp,anum2)
	          write(amessage,660) trim(anum1),trim(anum2)
660	          format(' Line ',a,': integer ',a,' cited previously.')
	          call write_message(increment=1)
	        end if
	        iwork(i)=0
	        go to 700
	      end if
	    end do
	    if(imessage.eq.0) then
	      write(amessage,620) trim(corrfile)
	      call write_message(leadspace='yes')
	    end if
	    call num2char(iline,anum1)
	    write(amessage,670) trim(anum1),trim(intfile)
670	    format(' Line ',a,': integer read as first item does not occur ',&
	    'in integer array read from file ',a)
	    call write_message(increment=1)
	  end if
700	  rtemp=char2real(ifail,2)
	  if(ifail.ne.0) then
	    if(imessage.eq.0) then
	      write(amessage,620) trim(corrfile)
	      call write_message(leadspace='yes')
	    end if
	    call num2char(iline,anum1)
	    write(amessage,720) trim(anum1)
720	    format(' Line ',a,': cannot read second item on line as a real ',&
	    'number.')
	    call write_message(increment=1)
	  else
	    realval(i)=rtemp
	  end if
	end do read_a_line
800	close (unit=corrunit,iostat=ierr)
	if(ierr.ne.0) then
	  write(amessage,810)
810	  format(' File management error: cannot continue execution.')
	  go to 9890
	end if
	if(imessage.ne.0) then
	  write(6,*)
	  go to 9900
	end if
	itemp=count(iwork.eq.0)
	if(itemp.eq.0) then
	  write(amessage,820) trim(corrfile)
820	  format(' No integer-real correspondence pairs read from file ',a)
	  go to 9890
	end if
	if(action.eq.'c') then
	  if(itemp.lt.numint) then
	    write(amessage,840) trim(corrfile),trim(intfile)
840	    format(' Integer-real corresondences were not supplied in file ',a,&
	    ' for all integers found in integer array file ',a)
	    call write_message(leadspace='yes')
	    write(amessage,850)
850	    format('   The following integer(s) were ommitted:-')
	    call write_message
	    cline=' '
	    j=1
	    do i=1,numint
	      if(iwork(i).eq.-1000) then
		call num2char(intval(i),anum1)
		anum1=adjustl(anum1)
	        cline(j+1:j+8)=anum1(1:8)
	        j=j+10
	        if(j.gt.72) then
		  write(6,'(a)') trim(cline)
	          j=1
		  cline=' '
	        end if
	      end if
	    end do
	    if(j.ne.1) write(6,'(a)') trim(cline)
	    write(6,*)
	    go to 9900
	  end if
	end if
	call num2char(inum,anum1)
	write(amessage,870) trim(anum1),trim(corrfile)
870	format(' - ',a,' integer-real pairs read from file ',a)
	call write_message

1000    if(action.eq.'m') newarray=oldarray
	do i=1,numint
	  if(iwork(i).eq.-1000) cycle
	  where(intarray.eq.intval(i)) newarray=realval(i)
	end do

! -- The output real array is next written.

        write(6,*)
        aprompt=' Enter name for output real array file: '
        call write_real_array(ifail,aprompt,newarray,pm_header=headerspec, &
        rows=nrow,columns=ncol)
        if(escset.eq.1) then
          escset=0
          write(6,*)
	  if(aintreal.eq.'m') then
	    i=numint-1
	    if(i.eq.0) then
	      go to 305
	    else
	      go to 355
	    end if
	  else
	    go to 600
	  end if
        end if
        write(6,*)
        go to 9900

9000	call num2char(iline,anum1)
	write(amessage,9010) trim(anum1),trim(corrfile)
9010	format(' Error reading line ',a,' of integer-real ',&
	'correspondence file ',a)
	go to 9890
9890	call write_message(leadspace='yes',endspace='yes')
9900	call close_files
	call free_grid_mem(gridspec)
	deallocate(intarray,newarray,oldarray,stat=ierr)

end program int2real

subroutine casetrans(string,hi_or_lo)

! -- Subroutine casetrans converts a string to upper or lower case.

! -- Arguments are as follows:-
!      string:	  contains the string whose case must be changed
!      hi_or_lo:  must be either 'lo' or 'hi' to indicate
!                 change of case direction.

! -- Revision history:-
!       June-November, 1995: version 1.

	use inter

	character (len=*), intent(inout)        :: string
	character (len=*), intent(in)           :: hi_or_lo
	character                               :: alo, ahi
	integer                                 :: inc,i

	if(hi_or_lo.eq.'lo') then
	  alo='A'; ahi='Z'; inc=iachar('a')-iachar('A')
	else if(hi_or_lo.eq.'hi') then
	  alo='a'; ahi='z'; inc=iachar('A')-iachar('a')
	else
	  call sub_error('CASETRANS')
	endif

	do i=1,len_trim(string)
	  if((string(i:i).ge.alo).and.(string(i:i).le.ahi)) &
	  string(i:i)=achar(iachar(string(i:i))+inc)
	end do

	return

end subroutine casetrans
!     Last change:  JD   16 Dec 2000    0:53 am
subroutine read_settings(ifail,idate,iheader)

! -- Subroutine read_settings reads a settings file located in the current
! -- directory.

! -- Arguments are as follows:-
!      ifail:    returned as zero unless settings file cannot be read
!      idate:    returned as zero unless date format is incorrect in settings
!                file
!      iheader:  returned as zero unless header specifier is incorrect


	use defn
	use inter

	integer, intent(out)			:: ifail,idate,iheader
	integer                                 :: iunit,ierr,iequals,i
	character (len=40)                      :: aline

	ifail=0
	idate=0
        iheader=0
	datespec=0
        headerspec=' '

	iunit=nextunit()
	open(unit=iunit,file='settings.fig',status='old',iostat=ierr)
	if(ierr.ne.0) then
	  ifail=1
	  return
	end if

	do
	  read(iunit,'(a)',err=100,end=200) cline
	  call casetrans(cline,'lo')
	  iequals=index(cline,'=')
	  if(iequals.le.1) cycle
	  aline=cline(1:iequals-1)
	  aline=adjustl(aline)
	  if(aline(1:4).eq.'date') then
	    aline=cline(iequals+1:)
	    aline=adjustl(aline)
	    if((aline(1:2).eq.'dd').and.(aline(4:5).eq.'mm')) then
	      datespec=1
	    else if((aline(1:2).eq.'mm').and.(aline(4:5).eq.'dd')) then
	      datespec=2
	    else
	      idate=1
	    end if
	  else if(aline(1:6).eq.'colrow') then
	    aline=cline(iequals+1:)
            do i=1,len_trim(aline)
              if(aline(i:i).eq.'''')aline(i:i)=' '
            end do
	    aline=adjustl(aline)
            call casetrans(aline,'lo')
            if(aline(1:3).eq.'yes')then
              headerspec='yes'
            else if(aline(1:2).eq.'no')then
              headerspec='no '
            else
              iheader=1
            end if
          end if
	end do
	go to 200

100	ifail=2
200     close(unit=iunit,iostat=ierr)
	return

end subroutine read_settings

!     Last change:  JD   21 Dec 2000    2:33 pm
subroutine readfig(specfile,coordfile,sampfile,pumpfile,pilotfile)

! -- Subroutine readfig reads a filename file.

! -- Arguments are as follows:-
!       specfile:  name of grid specification file
!       coordfile: name of bore coordinates file
!       sampfile:  name of bore sample file
!       pumpfile:  name of bore pumping file


	use defn
	use inter

	character (len=*),intent(out)           :: specfile
	character (len=*),intent(out),optional  :: coordfile,sampfile,pumpfile,pilotfile
	integer                                 :: iunit,ierr,iequals,nb,ifail
        character (len=100)                     :: filename

	specfile=' '
	if(present(coordfile)) coordfile=' '
	if(present(sampfile)) sampfile=' '
	if(present(pumpfile)) pumpfile=' '
        if(present(pilotfile)) pilotfile=' '

	iunit=nextunit()
	open(unit=iunit,file='files.fig',status='old',err=200)
	do
	  read(iunit,'(a)',err=100,end=200) cline
          cline=adjustl(cline)
	  iequals=index(cline,'=')
	  if(iequals.le.1) cycle
          nb=len_trim(cline)
          if(nb.eq.iequals) return
          call getfile(ifail,cline,filename,iequals+1,nb)
          if(ifail.ne.0) cycle
	  call casetrans(cline(1:iequals),'lo')
	  if(cline(1:10).eq.'grid_speci') then
	    specfile=filename
	  else if(cline(1:10).eq.'bore_coord') then
	    if(present(coordfile)) then
	      coordfile=filename
	    end if
	  else if(cline(1:10).eq.'bore_sampl') then
	    if(present(sampfile)) then
	      sampfile=filename
	    end if
	  else if(cline(1:10).eq.'bore_pumpi') then
	    if(present(pumpfile)) then
	      pumpfile=filename
	    end if
          else if(cline(1:12).eq.'pilot_points')then
            if(present(pilotfile))then
              pilotfile=filename
            end if
	  end if
100       continue
	end do

200     close(unit=iunit,iostat=ierr)
	return

end subroutine readfig

!*****************************************************************************
! subprograms for reading and parsing data (mainly from files) ------->
!*****************************************************************************

subroutine linesplit(ifail,num)

! -- subroutine linesplit splits a line into whitespace-delimited words

! -- Arguments are as follows:-
!       ifail:   returned as -1 if line is blank
!                returned as  1 if less than num segments
!       num:     number of words to be extracted

!    Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	integer, intent(out)            :: ifail
	integer, intent(in)             :: num
	integer                         :: nblc,j,i,nw
	character (len=3)               :: aspace

	ifail=0; nw=0; j=0
	aspace=' ,'//achar(9)   
	if(num.gt.NUM_WORD_DIM) call sub_error('LINESPLIT')
	nblc=len_trim(cline)
	if(nblc.eq.0) then
	  ifail=-1
	  return
	end if

5       if(nw.eq.num) return
	do i=j+1,nblc
	  if(index(aspace,cline(i:i)).eq.0) go to 20
	end do
	ifail=1
	return
20      nw=nw+1
	left_word(nw)=i
	do i=left_word(nw)+1,nblc
	  if(index(aspace,cline(i:i)).ne.0) go to 40
	end do
	right_word(nw)=nblc
	if(nw.lt.num) ifail=1
	return
40      right_word(nw)=i-1
	j=right_word(nw)
	go to 5

end subroutine linesplit


integer function char2int(ifail,num)

! -- Function char2int extracts an integer from a word demarcated by subroutine
!    linesplit.

! -- Arguments are as follows:-
!       ifail:    returned as zero unless an error condition arises
!       num:      the number of the word previously extracted by linesplit
!       returns   value of integer read from word

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn

	integer, intent(in)             :: num
	integer, intent(out)            :: ifail
	character (len=8)               :: afmt

	if(num.gt.NUM_WORD_DIM) call sub_error('CHAR2INT')
	if((right_word(num).lt.left_word(num)).or. &
	  (left_word(num).le.0)) call sub_error('CHAR2INT')

	ifail=0
	afmt='(i   )'
	write(afmt(3:5),'(i3)') right_word(num)-left_word(num)+1
	read(cline(left_word(num):right_word(num)),afmt,err=100) char2int
	return

100     ifail=1
	return

end function char2int


real function char2real(ifail,num)

! -- Function char2real extracts a real number from a word demarcated by
!    subroutine linesplit.

! -- Arguments are as follows:-
!       ifail:    returned as zero unless an error condition arises
!       num:      the number of the word previously extracted by linesplit
!       returns   value of real number read from word

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn

	integer, intent(in)             :: num
	integer, intent(out)            :: ifail
	integer                         :: ierr
	character (len=10)              :: afmt

	if(num.gt.NUM_WORD_DIM) call sub_error('CHAR2REAL')
	if((right_word(num).lt.left_word(num)).or. &
	  (left_word(num).le.0)) call sub_error('CHAR2REAL')

	ifail=0
	afmt='(f   .0)'
	write(afmt(3:5),'(i3)') right_word(num)-left_word(num)+1
	read(cline(left_word(num):right_word(num)),afmt, iostat=ierr) char2real
	if(ierr.ne.0) go to 110
	return

110     ifail=1
	return

end function char2real


double precision function char2double(ifail,num)

! -- Function char2double extracts a double precision number from a word
!    demarcated by subroutine linesplit.

! -- Arguments are as follows:-
!       ifail:    returned as zero unless an error condition arises
!       num:      the number of the word previously extracted by linesplit
!       returns   value of double precision number read from word

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn

	integer, intent(in)             :: num
	integer, intent(out)            :: ifail
	integer                         :: ierr
	character (len=10)              :: afmt

	if(num.gt.NUM_WORD_DIM) call sub_error('CHAR2DOUBLE')
	if((right_word(num).lt.left_word(num)).or. &
	  (left_word(num).le.0)) call sub_error('CHAR2DOUBLE')

	ifail=0
	afmt='(f   .0)'
	write(afmt(3:5),'(i3)') right_word(num)-left_word(num)+1
	read(cline(left_word(num):right_word(num)),afmt, iostat=ierr) char2double
	if(ierr.ne.0) go to 110
	return

110     ifail=1
	return

end function char2double
integer function nextunit()

! -- Function nextunit determines the lowest unit number available for
! -- opening.

! -- Revision history:-
!       June-November, 1995: version 1.

	logical::lopen

	do nextunit=10,100
	  inquire(unit=nextunit,opened=lopen)
	  if(.not.lopen) return
	end do
	write(6,10)
10      format(' *** No more unit numbers to open files ***')
	stop

end function nextunit
subroutine sub_error(subname)

! -- Subroutine sub_error names the subroutine causing a run-time error.

! -- Arguments are as follows:-
!       subname:  name of offending subroutine

! -- Revision history:-
!       June-November, 1995: version 1.

	character (len=*)               ::subname

	write(6,10) trim(subname)
10      format(/,' *** PROGRAMMING ERROR CALLING SUBROUTINE ',a,' ***')
	stop

end subroutine sub_error
!     Last change:  JD   26 Dec 2000    1:12 pm
subroutine spec_open(ifail,gridspec)

! -- Subroutine spec_open opens a grid specification file.

! -- Arguments are as follows:-
!       ifail:     returned as zero unless error condition arises.
!       gridspec:  defined variable containing grid specifications.

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	type (modelgrid)        :: gridspec
	integer, intent(out)    :: ifail
	integer                 :: ierr,jstep,nbb
	character (len=120)      :: atemp,aatemp,bbtemp

	ifail=0
	atemp=gridspec%specfile
	imessage=0
5       jstep=0
	do
	  jstep=jstep+1
	  if(jstep.ge.5) then
	    ifail=1
	    return
	  end if
10        if(atemp.ne.' ') then
	    write(6,20,advance='no') trim(atemp)
20          format(' Enter name of grid specification file [',a,']: ')
	  else
	    write(6,30,advance='no')
30          format(' Enter name of grid specification file: ')
	  endif
	  read(5,'(a)',err=10) aatemp
          nbb=len_trim(aatemp)
          if(aatemp.ne.' ')then
            call getfile(ifail,aatemp,bbtemp,1,nbb)
            if(ifail.ne.0) go to 10
          else
            bbtemp=' '
          end if
          gridspec%specfile=bbtemp
	  if(index(eschar,gridspec%specfile(1:2)).ne.0) then
	    escset=1
	    return
	  end if
	  if(gridspec%specfile.ne.' ') exit
	  if(atemp.ne.' ')then
	    gridspec%specfile=atemp
	    exit
	  end if
	end do

	gridspec%specunit=nextunit()
	open(unit=gridspec%specunit,file=gridspec%specfile,status='old',&
	iostat=ierr)
	if(ierr.ne.0) then
	  if(imessage.eq.5) then
	    ifail=1
	    return
	  end if
	  write(amessage,50) trim(gridspec%specfile)
50        format('cannot open file ',a,' - try again.')
	  call write_message(error='yes',increment=1)
	  if(gridspec%specfile.eq.atemp) atemp=' '
	  go to 5
	end if
	ifail=0
	return

end subroutine spec_open
!     Last change:  JD   11 Apr 2003    8:38 am
subroutine open_input_file(ifail,aprompt,infile,inunit,file_format,form_prompt,fformat)

! -- Subroutine open_input_file opens an input file.

! -- Arguments are as follows:-
!       ifail:        returned as non-zero if failure
!       aprompt:      user prompt for filename
!       infile:       name of input file
!       inunit:       unit number to communicate with file
!       file_format:  supplied as either "formatted" or "unformatted" to open
!                     formatted or unformatted file
!       form_prompt:  supplied as either "yes" or "no" to prompt for file format
!       fformat:      returns file format to main program if form_prompt is supplied

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	integer, intent(out)                       :: ifail
	character (len=*), intent(in)              :: aprompt
	character (len=*), intent(out)             :: infile
	integer, intent(out)                       :: inunit
	character (len=*), intent(in), optional    :: file_format
        character (len=*), intent(in), optional    :: form_prompt
        character (len=*), intent(out),optional    :: fformat
	integer                                    :: ierr,nbb,ifail1
	character (len=1)                          :: aformat,aa
	character (len=10)                         :: aaformat
        character (len=120)                        :: bfile
	logical                                    :: lopened
        character (len=20)                         :: bin_unform

! -- The correct unformatted word is read.

        include 'bin_unform.inc'

! -- The user is prompted for the name of the file to open.

	imessage=0
	ifail=0
10      write(6,'(a)',advance='no') aprompt(1:len_trim(aprompt)+1)
	read(5,'(a)') infile
	if(infile.eq.' ') go to 10
	infile=adjustl(infile)
	if(index(eschar,infile(1:2)).ne.0) then
	  escset=1
	  return
	end if
        nbb=len_trim(infile)
        call getfile(ifail1,infile,bfile,1,nbb)
        if(ifail1.ne.0) go to 10
        infile=bfile


! -- Is the file already open?

	inquire(file=infile,opened=lopened)
	if(lopened)then
	  write(amessage,30) trim(infile)
30        format(' File ',a,' is already open  - try again.')
	  call write_message(increment=1)
	  go to 10
	end if

! -- The file is opened (after the format is established).

	aformat='f'
        if(present(form_prompt))then
          if(form_prompt.eq.'yes')then
            if(.not.(present(fformat))) call sub_error('OPEN_INPUT_FILE')
31          write(6,32,advance='no')
32          format(' Is this a formatted or unformatted file? [f/u]: ')
            read(5,'(a)') aa
            call casetrans(aa,'lo')
            if(aa.eq.'e') go to 10
            if(aa.eq.'f')then
              aformat='f'
              fformat='f'
            else if(aa.eq.'u')then
              aformat='u'
              fformat='u'
            else
              go to 31
            end if
          end if
        else if(present(file_format)) then
	  if(file_format.eq.'formatted') then
	    aformat='f'
	  else if(file_format.eq.'unformatted')then
	    aformat='u'
	  else
	    call sub_error('OPEN_INPUT_FILE')
	  end if
	end if
	inunit=nextunit()
	if(aformat.eq.'f')then
	  open(unit=inunit,file=infile,status='old',iostat=ierr)
	else
	  open(unit=inunit,file=infile,status='old',form=bin_unform,  &
          iostat=ierr)
	end if

! -- If the file could not be opened a test is made as to whether the file
!    is formatted or unformatted when the opposite was expected.

	if(ierr.ne.0)then
	  inquire(file=infile,unformatted=aaformat,iostat=ierr)
	  if(ierr.ne.0)go to 40
	  call casetrans(aaformat,'lo')
	  if(aaformat.eq.' ') go to 40
	  if(aformat.eq.'f')then
	    if(aaformat.eq.'yes') then
	      if(imessage.gt.5) then
		ifail=1
		return
	      end if
	      write(amessage,35) trim(infile)
35            format(' File ',a,' is an unformatted file  - try again.')
	      call write_message(increment=1)
	      go to 10
	    else
	      go to 40
	    end if
	  else
	    if(aaformat.eq.'no') then
	      if(imessage.gt.5) then
		ifail=1
		return
	      end if
	      write(amessage,36) trim(infile)
36            format(' File ',a,' is a formatted file  - try again.')
	      call write_message(increment=1)
	      go to 10
	    else
	      go to 40
	    end if
	  end if
40        if(imessage.gt.5) then
	    ifail=1
	    return
	  end if
	  write(amessage,50) trim(infile)
50        format(' Cannot open file ',a,'  - try again.')
	  call write_message(increment=1)
	  go to 10
	end if

	return

end subroutine open_input_file
subroutine close_spec_file(gridspec,ok)

! -- Subroutine close_spec_file closes the grid specification file.

! -- Arguments are as follows:-
!       gridspec:     defined type holding grid specification information 
!                     (including unit number)
!       ok:           if supplied as "yes" a message is written to the screen

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	type (modelgrid)                :: gridspec
	logical                         :: lopen
	character (len=*),optional      :: ok

	inquire(unit=gridspec%specunit, opened=lopen)
	if(lopen) close(unit=gridspec%specunit)
	if(present(ok))then
	  if(ok.eq.'yes')then
	    write(amessage,10) trim(gridspec%specfile)
10          format('  - grid specifications read from file ',a)
	    call write_message
	  endif
	endif
	return

end subroutine close_spec_file
!     Last change:  JD    9 May 2003   11:43 am
subroutine read_integer_array(ifail,aprompt,array,pm_header,rows,columns,defaultfile)

! -- Subroutine read_integer_array reads an integer array from a file.

! -- Arguments are as follows:-
!       ifail:         returned as non-zero if error condition is encountered.
!       aprompt:       prompt text for integer array filename
!       array:         integer array read from file
!       pm_header:     if "yes" number of columns and rows expected as first
!                      line of input file.
!       rows,columns:  number of rows and columns in finite-difference grid
!       defaultfile:   a default name for the integer array file


	use defn
	use inter

	integer, intent(out)                    :: ifail
	character (len=*), intent(inout)        :: aprompt
	integer, intent(out),dimension(:,:)     :: array
	character (len=*), intent(in), optional :: pm_header
	integer, intent(in), optional           :: rows,columns
        character (len=*), optional             :: defaultfile
	character (len=120)                     :: afile,bfile
	character (len=4)                       :: extension
	logical                                 :: pm
	character (len=1)                       :: aformat
	integer                                 :: i,iunit,nncol,nnrow,ifail1,nbb,idef

	ifail=0
	pm=.false.
	if(present(pm_header)) then
	  if(pm_header.ne.'yes') go to 5
	  if((.not.(present(rows))).or.(.not.(present(columns)))) &
	  call sub_error('READ_INTEGER_ARRAY')
	  pm=.true.
	end if

        idef=0
        if(present(defaultfile))then
          if(defaultfile.ne.' ')then
            idef=1
            aprompt=trim(aprompt)//' ['//trim(defaultfile)//']:'
          end if
        end if

5       imessage=0
10      write(6,'(a)',advance='no') aprompt(1:len_trim(aprompt)+1)
	read(5,'(a)')afile
	if(afile.eq.' ') then
          if(idef.eq.0) then
            go to 10
          else
            afile=defaultfile
            go to 15
          end if
        end if
	afile=adjustl(afile)
	if(index(eschar,afile(1:2)).ne.0)then
	  escset=1
	  return
	end if
        nbb=len_trim(afile)
        call getfile(ifail1,afile,bfile,1,nbb)
        if(ifail1.ne.0) go to 10
        afile=bfile
15      continue
!       if(pm)then
!         aformat='f'
!       else
	  i=len_trim(afile)
	  extension=afile(max(i-3,1):i)
	  call casetrans(extension,'lo')
	  if(extension.eq.'.inu')then
	    aformat='u'
	  else if(extension.eq.'.inf')then
	    aformat='f'
	  else
30          write(6,40,advance='no')
40          format(' Is this a formatted or unformatted file? [f/u]: ')
	    read(5,'(a)') aformat
	    if((aformat.eq.'e').or.(aformat.eq.'E')) then
	      write(6,*)
	      go to 10
	    end if
	    if(aformat.eq.' ') go to 30
	    call casetrans(aformat,'lo')
	    if((aformat.ne.'f').and.(aformat.ne.'u')) go to 30
	  end if
!       end if

	iunit=nextunit()
	if(aformat.eq.'f') then
	  open(unit=iunit,file=afile,status='old',err=100)
	  if(pm) then
	    read(iunit,'(a)',end=150) cline
	    call linesplit(ifail,2)
	    if(ifail.ne.0) go to 200
	    nncol=char2int(ifail,1)
	    if(ifail.ne.0) go to 300
	    nnrow=char2int(ifail,2)
	    if(ifail.ne.0) go to 300
	    if((nncol.ne.columns).or.(nnrow.ne.rows)) go to 250
	    read(iunit,*,err=350,end=400) array
	  else
	    read(iunit,*,err=450,end=500) array
	  end if
	else
	  open(unit=iunit,file=afile,status='old',form='binary',err=120)
	  read(iunit,end=600)
	  read(iunit,err=550,end=600) array
	end if
	write(amessage,70) trim(afile)
70      format('  - integer array read from file ',a)
	call write_message
	aprompt=afile
	go to 999

100     if(imessage.eq.5) go to 995
	if(pm)then
	  write(amessage,110) trim(afile)
110       format('cannot open formatted integer array file ',&
	  a,' - try again.')
	else
	  write(amessage,111) trim(afile)
111       format('cannot open formatted integer array file ',a,' - try again.')
	end if
	call write_message(error='yes',increment=1,endspace='yes')
	go to 10

120     if(imessage.eq.5) go to 995
	write(amessage,130) trim(afile)
130     format('cannot open unformatted integer array file ',a,' - try again.')
	call write_message(error='yes',increment=1,endspace='yes')
	go to 10

150     write(amessage,160) trim(afile)
160     format('unexpected end to formatted integer array file ',a,'.')
	call write_message(error='yes',leadspace='yes',endspace='yes')
	go to 995

200     write(amessage,210) trim(afile)
210     format('column and row numbers are expected on first line of ',&
	'formatted integer array file ',a,'.')
	call write_message(error='yes',leadspace='yes',endspace='yes')
	go to 995

250     write(amessage,260) trim(afile)
260     format('the row and/or column number provided in header to ',&
	'integer array file ',a,' does not match that provided in ',&
	'grid specification file.')
	call write_message(error='yes',leadspace='yes',endspace='yes')
	go to 995

300     write(amessage,310) trim(afile)
310     format('cannot read row and/or column number from first line of ',&
	'formatted integer array file ',a,'.')
	call write_message(error='yes',leadspace='yes',endspace='yes')
	go to 995

350     write(amessage,360) trim(afile)
360     format(' Data error encountered while reading formatted integer ',&
	'array file ',a)
	call write_message(leadspace='yes',endspace='yes')
	go to 995

400     write(amessage,410) trim(afile)
410     format('unexpected end to formatted integer array file ',a,'.')
	call write_message(error='yes',leadspace='yes',endspace='yes')
	go to 995

450     write(amessage,460) trim(afile)
460     format(' Data error encountered while reading formatted integer ',&
	'array file ',a,'.')
	call write_message(leadspace='yes',endspace='yes')
	go to 995

500     write(amessage,510) trim(afile)
510     format('unexpected end to formatted integer array file ',a,'.')
	call write_message(error='yes',leadspace='yes',endspace='yes')
	go to 995

550     write(amessage,560) trim(afile)
560     format(' Error encountered while reading unformatted integer ',&
	'array file ',a,'.')
	call write_message(leadspace='yes',endspace='yes')
	go to 995

600     write(amessage,610) trim(afile)
610     format('unexpected end to unformatted integer array file ',a,'.')
	call write_message(error='yes',leadspace='yes',endspace='yes')
	go to 995

995     ifail=1
999     close(unit=iunit,iostat=i)
	return

end subroutine read_integer_array
subroutine free_grid_mem(gridspec)

! -- Subroutine free_grid_mem frees memory and deallocates pointers pertinent
!    to the storage of grid data.

! -- Arguments are as follows:-
!       gridspec:  defined variable holding grid specifications

! -- Revision history:-
!       June-November, 1995: version1.

	use defn
	use inter

	type (modelgrid)        :: gridspec
	integer                 ::ierr

	deallocate(gridspec%delr,gridspec%delc,stat=ierr)
	nullify (gridspec%delr,gridspec%delc)

end subroutine free_grid_mem
subroutine close_files

! -- Subroutine close_files closes all open files.

! -- Revision history:-
!       June-November, 1995: version 1.

	integer         :: i,ierr

	do i=10,100
	  close(unit=i,iostat=ierr)
	end do
	return

end subroutine close_files
subroutine read_spec_dim(ifail,gridspec)

! -- Subroutine read_spec_dim reads the grid row and column numbers from the
!    first line of a grid specification file.

! -- Arguments are as follows:-
!       ifail:     returned as zero unless an error condition arises
!       gridspec:  defined type variable holding grid specifications

!    Revision history:-
!       June-November, 1995: verson 1.

	use defn
	use inter

	type (modelgrid)                :: gridspec
	integer, intent(out)            :: ifail
	character (len=8)               :: anum

	gridspec%specline=0
	ifail=0
10      gridspec%specline=gridspec%specline+1
	read(gridspec%specunit,'(a)',end=100) cline
	call linesplit(ifail,2)
	if(ifail.lt.0) go to 10
	if(ifail.gt.0) then
	  call num2char(gridspec%specline,anum)
	  write(amessage,20) trim(anum),trim(gridspec%specfile)
20        format('insufficient items on line ',a,' of file ',a,'.')
	  call write_message(error='yes',leadspace='yes')
	  go to 200
	end if
	gridspec%nrow=char2int(ifail,1)
	if(ifail.ne.0) go to 130
	gridspec%ncol=char2int(ifail,2)
	if(ifail.ne.0) go to 130
	return

100     write(amessage,110) trim(gridspec%specfile)
110     format('grid dimensions missing from file ',a,'.')
	call write_message(error='yes',leadspace='yes')
	go to 200
130     call num2char(gridspec%specline,anum)
	write(amessage,140) trim(anum),trim(gridspec%specfile)
140     format('cannot read grid dimensions from line ',a,' of file ',a,'.')
	call write_message(error='yes',leadspace='yes')
	go to 200

200     ifail=1
	return

end subroutine read_spec_dim
!     Last change:  J    20 Dec 2004   12:46 pm
subroutine write_real_array(ifail,aprompt,array,pm_header,rows,columns,&
binary_header,atype,ntrans,kstp,kper,pertim,totim,text,ncol,nrow,ilay,istage, &
realfile,aaformat)

! -- Subroutine writ_real_array writes a real array to an output file.

! -- Arguments are as follows:-
!       ifail:         returned as non-zero if error condition arises
!       aprompt:       prompt requesting name of output file (contains output
!                      filename on exit)
!       array:         real array for output
!       pm_header:     if "yes" put number of columns and rows at start of file
!       rows,columns:  number of rows and columns in finite-difference grid
!       binary_header: if "yes" put MODFLOW/MT3D-type header at start of binary
!                      output file
!       atype:         determines whether MODFLOW or MT3D header
!       ntrans,kstp,kper,pertim,totim,text,ncol,nrow,ilay: used in header 
!                                                          (if required)
!       istage:        used if making a different call to this subroutine to open
!                      real array file and to write it
!       realfile:      name of real array file if two separate calls to this subroutine
!       aaformat:      format status if two separate calls to this subroutine

	use defn
	use inter

	integer, intent(out)                    :: ifail
	character (len=*), intent(inout)        :: aprompt
	real, intent(in),dimension(:,:)         :: array
	character (len=*), intent(in), optional :: pm_header
	integer, intent(in), optional           :: rows,columns
	character (len=*), optional             :: binary_header
	character (len=*), optional             :: atype
	integer, optional                       :: ntrans,kstp,kper
	real, optional                          :: pertim,totim
	character (len=16),optional             :: text
	integer, optional                       :: ncol,nrow,ilay
        integer, intent(in), optional           :: istage
        character (len=*), optional             :: realfile
        character (len=*), optional             :: aaformat
	character (len=120)                     :: afile,bfile
	character (len=4)                       :: extension
	logical                                 :: pm,lexist,lopened,mhead
	character (len=1)                       :: aformat,aa
	integer                                 :: i,iunit,nbb,ifail1,iistage,ierr
        integer                                 :: kkstp,kkper,iilay,nnrow,nncol
        real                                    :: ppertim,ttotim
        character (len=16)                      :: ttext

	ifail=0
	pm=.false.

	if(present(pm_header)) then
	  if(pm_header.ne.'yes') go to 5
	  if((.not.(present(rows))).or.(.not.(present(columns)))) &
	  call sub_error('WRITE_REAL_ARRAY')
	  pm=.true.
	end if

	mhead=.false.
	if(present(binary_header))then
	  if(binary_header.eq.'yes') then
	    if((.not.(present(atype))).or.(.not.(present(ntrans))).or. &
	       (.not.(present(kstp))).or.(.not.(present(kper))).or. &
	       (.not.(present(pertim))).or.(.not.(present(totim))).or. &
	       (.not.(present(text))).or.(.not.(present(ncol))).or. &
	       (.not.(present(nrow))).or.(.not.(present(ilay)))) &
	       call sub_error('WRITE_REAL_ARRAY')
	       mhead=.true.
	  else if(binary_header.ne.'no') then
	    call sub_error('WRITE_REAL_ARRAY')
	  end if
	end if

5       imessage=0

        if(present(istage)) then
          if((.not.(present(realfile))).or.(.not.(present(aaformat)))) &
          call sub_error('WRITE_REAL_ARRAY')
          iistage=istage
        else
          iistage=0
        end if
        if(iistage.eq.2) then
          afile=realfile
          aformat=aaformat
          go to 350
        end if

10      write(6,'(a)',advance='no') aprompt(1:len_trim(aprompt)+1)
	read(5,'(a)')afile
	afile=adjustl(afile)
	if(index(eschar,afile(1:2)).ne.0)then
	  escset=1
	  return
	end if
	if(afile.eq.' ') go to 10
        nbb=len_trim(afile)
        call getfile(ifail1,afile,bfile,1,nbb)
        if(ifail1.ne.0) go to 10
        afile=bfile
!       if(pm)then
!         aformat='f'
!       else
	  i=len_trim(afile)
	  extension=afile(max(i-3,1):i)
	  call casetrans(extension,'lo')
	  if(extension.eq.'.reu')then
	    aformat='u'
	  else if(extension.eq.'.ref')then
	    aformat='f'
	  else
20          if(aprompt(2:2).ne.' ') then
30            write(6,40,advance='no')
40            format(' Write a formatted or unformatted file? [f/u]: ')
	    else
	      write(6,50,advance='no')
50            format('   write a formatted or unformatted file? [f/u]: ')
	    end if
	    read(5,'(a)') aformat
	    if((aformat.eq.'e').or.(aformat.eq.'E')) then
	      write(6,*)
	      go to 10
	    end if
	    if(aformat.eq.' ') go to 20
	    call casetrans(aformat,'lo')
	    if((aformat.ne.'f').and.(aformat.ne.'u')) go to 20
	  end if
!       end if

	inquire (file=afile,opened=lopened)
	if(lopened) then
	  if(aprompt(2:2).ne.' ') then
	    write(6,51)
51          format(' file is already open  - try again')
	  else
	    write(6,52)
52          format('   file is already open  - try again')
	  end if
	  go to 10
	end if
!       inquire(file=afile,exist=lexist)
!       if(lexist) then
!55        if(aprompt(2:2).ne.' ') then
!           write(6,56,advance='no')
!56          format(' File already exists: overwrite it?  [y/n]: ')
!         else
!           write(6,57,advance='no')
!57          format('   file already exists: overwrite it?  [y/n]: ')
!         end if
!         read(5,'(a)') aa
!         if(aa.eq.' ') go to 55
!         call casetrans(aa,'lo')
!         if((index(eschar,aa).ne.0).or.(aa.eq.'n')) then
!           write(6,*)
!           go to 10
!         end if
!         if(aa.ne.'y') go to 55
!       end if

        if(iistage.eq.1)then
          realfile=afile
          aaformat=aformat
          return
        end if

350     continue

	iunit=nextunit()
	if(aformat.eq.'f') then
	  open(unit=iunit,file=afile,status='replace',err=100)
	  if(pm) then
	    write(iunit,60,err=700) columns, rows
60          format(1x,i7,1x,i7)
	  end if
	  do i=1,rows
	    write(iunit,65,err=700) array(:,i)
65          format(7(1pe14.6))
	  end do
	else
          open(unit=iunit,file=afile,form='binary',iostat=ierr)
          if(ierr.ne.0)then
            open(unit=iunit,file=afile,form='unformatted',err=100)
	  end if
	  if(mhead)then
	    if(atype.eq.'modflow')then
	      write(iunit,err=700) kstp,kper,pertim,totim,text,ncol,nrow,ilay
	    else if(atype.eq.'mt3d')then
	      write(iunit,err=700) ntrans,kstp,kper,totim,text,ncol,nrow,ilay
	    else
	      call sub_error('WRITE_REAL_ARRAY')
	    end if
	  else
            kkstp=1
            kkper=1
            ppertim=1.0
            ttotim=1.0
            ttext=' '
            iilay=1
            nncol=columns
            nnrow=rows
	    write(iunit,err=700)kkstp,kkper,ppertim,ttotim,ttext,nncol,nnrow,iilay
	  end if
	  write(iunit,err=700) array
	end if
	if(aprompt(2:2).ne.' ') then
	  if(aformat.eq.'f') then
	    write(amessage,70) trim(afile)
70          format('  - formatted real array written to file ',a)
	  else
	    write(amessage,71) trim(afile)
71          format('  - unformatted real array written to file ',a)
	  end if
	else
	  if(aformat.eq.'f')then
	    write(amessage,80) trim(afile)
80          format('    - formatted real array written to file ',a)
	  else
	    write(amessage,81) trim(afile)
81          format('    - unformatted real array written to file ',a)
	  end if
	end if
	call write_message
	aprompt=afile
	go to 999

100     write(amessage,110) trim(afile)
110     format('cannot open file ',a,' for output.')
	call write_message(error='yes',leadspace='yes')
	go to 995
700     write(amessage,710) trim(afile)
710     format(' Cannot write data to file ',a, &
	': file inaccessible or disk full.')
	call write_message(leadspace='yes')
	go to 995

995     ifail=1
999     close(unit=iunit,iostat=i)
	return

end subroutine write_real_array
!*****************************************************************************
! functions comprising the generic function KEY_READ
!*****************************************************************************

! -- Function key_read reads and checks a number from the keyboard.

! -- Arguments are as follows:-
!       value:   value of the number read
!       returns  0 unless an error condition arises

! -- Revision history:-
!       June-November, 1995: version 1.

integer function int_key_read(value)

	use defn

	integer, intent(out)    :: value
	integer                 :: ifail
	character (len=50)      :: atemp

	int_key_read=0
	read(5,'(a)') atemp
	if(atemp.ne.' ') atemp=adjustl(atemp)   ! lf90 bug
	if(atemp.eq.' ') then
	  int_key_read=-1
	else if(index(eschar,atemp(1:2)).ne.0) then
	  escset=1
	else
	  call a2i(ifail,atemp,value)
	  if(ifail.ne.0) int_key_read=1
	end if
	return

end function int_key_read


integer function real_key_read(value)

	use defn

	real, intent(out)       :: value
	integer                 :: ifail
	character (len=50)      :: atemp

	real_key_read=0
	read(5,'(a)') atemp
	if(atemp.ne.' ') atemp=adjustl(atemp)   ! lf90 bug
	if(atemp.eq.' ') then
	  real_key_read=-1
	else if(index(eschar,atemp(1:2)).ne.0) then
	  escset=1
	else
	  call a2r(ifail,atemp,value)
	  if(ifail.ne.0) real_key_read=1
	end if
	return

end function real_key_read


integer function double_key_read(value)

	use defn

	double precision, intent(out)   :: value
	integer                         :: ifail
	character (len=50)              :: atemp

	double_key_read=0
	read(5,'(a)') atemp
	if(atemp.ne.' ') atemp=adjustl(atemp)   ! lf90 bug
	if(atemp.eq.' ') then
	  double_key_read=-1
	else if(index(eschar,atemp(1:2)).ne.0) then
	  escset=1
	else
	  call a2d(ifail,atemp,value)
	  if(ifail.ne.0) double_key_read=1
	end if
	return

end function double_key_read

!     Last change:  JD   29 Jan 2001    2:11 pm

subroutine getfile(ifail,cline,filename,ibeg,iend)

! Subroutine getfile extracts a filename from a string.

! -- Arguments are as follows:-
!       ifail: returned as zero if filename successfully read
!       cline: a character string containing the file name
!       filename: the name of the file read from the string
!       ibeg: character position at which to begin search for filename
!       iend: on input  - character position at which to end search for filename
!             on output - character postion at which filename ends


        integer, intent(out)               :: ifail
        integer, intent(in)                :: ibeg
        integer, intent(inout)             :: iend
        character (len=*), intent(in)      :: cline
        character (len=*), intent(out)     :: filename

        integer                            :: i,j,k
        character (len=1)                  :: aa

        ifail=0
        do i=ibeg,iend
          aa=cline(i:i)
          if((aa.ne.' ').and.(aa.ne.',').and.(aa.ne.char(9)))go to 50
        end do
        ifail=1
        return

50      if((aa.eq.'"').or.(aa.eq.''''))then
          do j=i+1,iend
            if(cline(j:j).eq.aa) go to 60
          end do
          ifail=1
          return
60        iend=j
          if(i+1.gt.j-1)then
            ifail=1
            return
          else
            filename=cline(i+1:j-1)
          end if
        else
          do j=i+1,iend
            if((cline(j:j).eq.' ').or.(cline(j:j).eq.',').or.(cline(j:j).eq.char(9)))then
              k=j-1
              go to 100
            end if
          end do
          k=iend
100       filename=cline(i:k)
          if(cline(k:k).eq.'"')then
            ifail=1
            return
          else if(cline(k:k).eq.'''')then
            ifail=1
            return
          end if

          iend=k
        end if
        filename=adjustl(filename)
        return

end subroutine getfile


!*****************************************************************************
! subroutines comprising the generic subroutine NUM2CHAR ------->
!*****************************************************************************

! -- Subroutine num2char writes the character equivalent of a number.

! -- Arguments are as follows:-
!       value:   the number to be expressed in character form
!       string:  the number expressed in character form
!       nchar:   the maximum number of characters in which to express number

! -- Revision history:-
!       June-November, 1995: version 1.

subroutine i2a(value,string,nchar)

	use inter

	integer, intent(in)             :: value
	character (len=*), intent(out)  :: string
	integer, intent(in), optional   :: nchar
	character (len=12)              :: afmt
	integer                         :: llen

	string=' '
	afmt='(i    )'
	llen=min(30,len(string))
	if(present(nchar)) llen=min(llen,nchar)
	write(afmt(3:6),'(i4)') llen
	write(string(1:llen),afmt,err=100) value
	string=adjustl(string)
	if(string(1:1).eq.'*') go to 100
	return

100     string(1:llen)=repeat('#',llen)
	return

end subroutine i2a


subroutine d2a(value,string,nchar)

	use inter

	double precision, intent(in)    :: value
	character (len=*), intent(out)  :: string
	integer, intent(in), optional   :: nchar
	integer                         :: llen, ifail
	double precision                :: value_check
	character (len=32)              :: word

	string=' '
	llen=min(29,len(string))
	if(present(nchar)) llen=min(llen,nchar)
	call wrtsig(ifail,value,word,llen,1,value_check,0)
	if(ifail.lt.0) then
	  call sub_error('D2A')
	else if(ifail.gt.0) then
	  string(1:llen)=repeat('#',llen)
	else
	  string=adjustl(word)
	end if
	return

end subroutine d2a


subroutine r2a(value,string,nchar)

	use inter

	real,intent(in)                 :: value
	character (len=*), intent(out)  :: string
	integer, intent(in), optional   :: nchar
	integer                         :: llen,ifail
	double precision                :: dvalue,dvalue_check
	character (len=32)              :: word

	string=' '
	llen=min(29,len(string))
	if(present(nchar)) llen=min(llen,nchar)
	dvalue=value
	call wrtsig(ifail,dvalue,word,llen,0,dvalue_check,0)
	if(ifail.lt.0) then
	  call sub_error('R2A')
	else if(ifail.gt.0) then
	  string(1:llen)=repeat('#',llen)
	else
	  string=adjustl(word)
	end if
	return

end subroutine r2a

	
	SUBROUTINE WRTSIG(IFAIL,VAL,WORD,NW,PRECIS,TVAL,NOPNT)
! --
! -- SUBROUTINE WRTSIG WRITES A NUMBER INTO A CONFINED SPACE WITH MAXIMUM
! -- PRECISION
! --

! -- Revision history:-
!       July, 1993: version 1.
!       August 1994: modified for unix version (#ifdef's added)
!       August, 1995: #ifdefs commented out for inclusion in Groundwater
!                     Data Utilities

!       failure criteria:
!           ifail= 1 ...... number too large or small for single precision type
!           ifail= 2 ...... number too large or small for double precision type
!           ifail= 3 ...... field width too small to represent number
!           ifail=-1 ...... internal error type 1
!           ifail=-2 ...... internal error type 2
!           ifail=-3 ...... internal error type 3

	INTEGER PRECIS,LW,POS,INC,D,P,W,J,JJ,K,JEXP,N,JFAIL,NW, &
	EPOS,PP,NOPNT,KEXP,IFLAG,LEXP
	INTEGER IFAIL
	DOUBLE PRECISION VAL,TVAL
	CHARACTER*29 TWORD,TTWORD,FMT*14
	CHARACTER*(*) WORD

	LEXP=0
	IFLAG=0
	WORD=' '
	POS=1
	IF(VAL.LT.0.0D0)POS=0
!#ifdef USE_D_FORMAT
!        WRITE(TWORD,'(1PD23.15D3)') VAL
!#else
	WRITE(TWORD,'(1PE23.15E3)') VAL
!#endif
	READ(TWORD(20:23),'(I4)') JEXP
	EPOS=1
	IF(JEXP.LT.0)EPOS=0

	JFAIL=0
	IFAIL=0
	IF(PRECIS.EQ.0)THEN
	  LW=MIN(15,NW)
	ELSE
	  LW=MIN(23,NW)
	END IF

	N=0
	IF(NOPNT.EQ.1)N=N+1
	IF(POS.EQ.1)N=N+1
	IF(PRECIS.EQ.0)THEN
	  IF(ABS(JEXP).GT.38)THEN
	    IFAIL=1
	    RETURN
	  END IF
	  IF(POS.EQ.1) THEN
	    IF(LW.GE.13) THEN
	      WRITE(WORD,'(1PE13.7)',ERR=80) VAL
	      GO TO 200
	    END IF
	  ELSE
	    IF(LW.GE.14)THEN
	      WRITE(WORD,'(1PE14.7)',ERR=80) VAL
	      GO TO 200
	    END IF
	  END IF
	  IF(LW.GE.14-N) THEN
	    LW=14-N
	    GO TO 80
	  END IF
	ELSE
	  IF(ABS(JEXP).GT.275)THEN
	    IFAIL=2
	    RETURN
	  END IF
	  IF(POS.EQ.1) THEN
	    IF(LW.GE.22) THEN
!#ifdef USE_D_FORMAT
!              WRITE(WORD,'(1PD22.15D3)',ERR=80) VAL
!#else
	      WRITE(WORD,'(1PE22.15E3)',ERR=80) VAL
!#endif
	      GO TO 200
	    END IF
	  ELSE
	    IF(LW.GE.23) THEN
!#ifdef USE_D_FORMAT
!              WRITE(WORD,'(1PD23.15D3)',ERR=80) VAL
!#else
	      WRITE(WORD,'(1PE23.15E3)',ERR=80) VAL
!#endif
	      GO TO 200
	    END IF
	  END IF
	  IF(LW.GE.23-N)THEN
	    LW=23-N
	    GO TO 80
	  END IF
	END IF

	IF(NOPNT.EQ.1)THEN
	  IF((JEXP.EQ.LW-2+POS).OR.(JEXP.EQ.LW-3+POS))THEN
	    WRITE(FMT,15)LW+1
15          FORMAT('(F',I2,'.0)')
	    WRITE(WORD,FMT,ERR=19) VAL
	    IF(INDEX(WORD,'*').NE.0) GO TO 19
	    IF(WORD(1:1).EQ.' ') GO TO 19
	    WORD(LW+1:LW+1)=' '
	    GO TO 200
	  END IF
	END IF
19      D=MIN(LW-2+POS,LW-JEXP-3+POS)
20      IF(D.LT.0) GO TO 80
	WRITE(FMT,30) LW,D
30      FORMAT('(F',I2,'.',I2,')')
	WRITE(WORD,FMT,ERR=80) VAL
	IF(INDEX(WORD,'*').NE.0) THEN
	  D=D-1
	  GO TO 20
	END IF
	K=INDEX(WORD,'.')
	IF(K.EQ.0)THEN
	  IFAIL=-1
	  RETURN
	END IF
	IF((K.EQ.1).OR.((POS.EQ.0).AND.(K.EQ.2)))THEN
	  DO 70 J=1,3
	  IF(K+J.GT.LW) GO TO 75
	  IF(WORD(K+J:K+J).NE.'0') GO TO 200
70        CONTINUE
	  GO TO 80
75        IFAIL=3
	  RETURN
	END IF
	GO TO 200

80      WORD=' '
	IF(NOPNT.EQ.0)THEN
	  D=LW-7
	  IF(POS.EQ.1) D=D+1
	  IF(EPOS.EQ.1) D=D+1
	  IF(ABS(JEXP).LT.100) D=D+1
	  IF(ABS(JEXP).LT.10) D=D+1
	  IF((JEXP.GE.100).AND.(JEXP-(D-1).LT.100))THEN
	    P=1+(JEXP-99)
	    D=D+1
	    LEXP=99
	  ELSE IF((JEXP.GE.10).AND.(JEXP-(D-1).LT.10))THEN
	    P=1+(JEXP-9)
	    D=D+1
	    LEXP=9
	  ELSE IF((JEXP.EQ.-10).OR.(JEXP.EQ.-100)) THEN
	    IFLAG=1
	    D=D+1
	  ELSE
	    P=1
	  END IF
	  INC=0
85        IF(D.LE.0) GO TO 300
	  IF(IFLAG.EQ.0)THEN
	    WRITE(FMT,100,ERR=300) P,D+7,D-1
	  ELSE
	    WRITE(FMT,100,ERR=300) 0,D+8,D
	  END IF
	  WRITE(TWORD,FMT) VAL
	  IF(IFLAG.EQ.1) GO TO 87
	  READ(TWORD(D+4:D+7),'(I4)',ERR=500) KEXP
	  IF(((KEXP.EQ.10).AND.((JEXP.EQ.9).OR.(LEXP.EQ.9))).OR. &
	  ((KEXP.EQ.100).AND.((JEXP.EQ.99).OR.LEXP.EQ.99))) THEN
	    IF(INC.EQ.0)THEN
	      IF(LEXP.EQ.0)THEN
		IF(D-1.EQ.0) THEN
		  D=D-1
		ELSE
		  P=P+1
		END IF
	      ELSE IF(LEXP.EQ.9)THEN
		IF(JEXP-(D-2).LT.10) THEN
		  P=P+1
		ELSE
		  D=D-1
		END IF
	      ELSE IF(LEXP.EQ.99)THEN
		IF(JEXP-(D-2).LT.100)THEN
		  P=P+1
		ELSE
		  D=D-1
		END IF
	      END IF
	      INC=INC+1
	      GO TO 85
	    END IF
	  END IF
!#ifdef USE_D_FORMAT
!87        J=INDEX(TWORD,'D')
!#else
87        J=INDEX(TWORD,'E')
!#endif
	  GO TO 151
	END IF
	INC=0
	P=LW-2
	PP=JEXP-(P-1)
	IF(PP.GE.10)THEN
	  P=P-1
	  IF(PP.GE.100)P=P-1
	ELSE IF(PP.LT.0)THEN
	  P=P-1
	  IF(PP.LE.-10)THEN
	    P=P-1
	    IF(PP.LE.-100)P=P-1
	  END IF
	END IF
	IF(POS.EQ.0)P=P-1
90      CONTINUE
	D=P-1
	W=D+8
	WRITE(FMT,100) P,W,D
	IF(D.LT.0)THEN
	  IF(JFAIL.EQ.1) GO TO 300
	  JFAIL=1
	  P=P+1
	  GO TO 90
	END IF
!#ifdef USE_D_FORMAT
!100     FORMAT('(',I2,'pD',I2,'.',I2,'D3)')
!#else
100     FORMAT('(',I2,'pE',I2,'.',I2,'E3)')
!#endif
	WRITE(TWORD,FMT) VAL
!#ifdef USE_D_FORMAT
!        J=INDEX(TWORD,'D')
!#else
	J=INDEX(TWORD,'E')
!#endif
	IF(TWORD(J-1:J-1).NE.'.')THEN
	  IFAIL=-1
	  RETURN
	END IF
	N=1
	IF(TWORD(J+1:J+1).EQ.'-') N=N+1
	IF(TWORD(J+2:J+2).NE.'0') THEN
	  N=N+2
	  GO TO 120
	END IF
	IF(TWORD(J+3:J+3).NE.'0') N=N+1
120     N=N+1
	IF(J+N-2-POS.LT.LW)THEN
	  IF(INC.EQ.-1) GO TO 150
	  TTWORD=TWORD
	  P=P+1
	  INC=1
	  GO TO 90
	ELSE IF(J+N-2-POS.EQ.LW) THEN
	  GO TO 150
	ELSE
	  IF(INC.EQ.1)THEN
	    TWORD=TTWORD
	    GO TO 150
	  END IF
	  IF(JFAIL.EQ.1) GO TO 300
	  P=P-1
	  INC=-1
	  GO TO 90
	END IF

150     J=INDEX(TWORD,'.')
151     IF(POS.EQ.0)THEN
	  K=1
	ELSE
	 K=2
	END IF
	WORD(1:J-K)=TWORD(K:J-1)
	JJ=J
	J=J-K+1
	IF(PRECIS.EQ.0)THEN
	  WORD(J:J)='E'
	ELSE
	  WORD(J:J)='D'
	END IF
	JJ=JJ+2
	IF(NOPNT.EQ.0) JJ=JJ-1
	IF(TWORD(JJ:JJ).EQ.'-')THEN
	  J=J+1
	  WORD(J:J)='-'
	END IF
	IF(TWORD(JJ+1:JJ+1).NE.'0')THEN
	  J=J+2
	  WORD(J-1:J)=TWORD(JJ+1:JJ+2)
	  GO TO 180
	END IF
	IF(TWORD(JJ+2:JJ+2).NE.'0')THEN
	  J=J+1
	  WORD(J:J)=TWORD(JJ+2:JJ+2)
	END IF
180     J=J+1
	WORD(J:J)=TWORD(JJ+3:JJ+3)
	IF(IFLAG.EQ.1)THEN
	  IF(POS.EQ.1)THEN
	    JJ=1
	  ELSE
	    JJ=2
	  END IF
	  N=len_trim(WORD)
	  DO 190 J=JJ,N-1
190       WORD(J:J)=WORD(J+1:J+1)
	  WORD(N:N)=' '
	END IF

200     IF(len_trim(WORD).GT.LW)THEN
	  IFAIL=-2
	  RETURN
	END IF
	WRITE(FMT,30) LW,0
	READ(WORD,FMT,ERR=400) TVAL
	RETURN
300     IFAIL=3
	RETURN
400     IFAIL=-3
	RETURN
500     IFAIL=-2
	RETURN
	END

!*****************************************************************************
! subroutines comprising the generic subroutine CHAR2NUM ------->
!*****************************************************************************


! -- The subroutines comprising char2num convert a string to either an integer,
!    a real number, or a double precision number.

! -- Arguments are as follows:-
!      ifail:   indicates failure if returned as non-zero
!      string:  a character string containing a number
!      num:     an integer (for a2i), real (for a2r), or double precision (for
!               a2d) number extracted from the string.

! -- Revision history:-
!       June-November, 1995: version 1.


subroutine a2i(ifail,string,num)

	integer, intent(out)            :: ifail
	character (len=*), intent(in)   :: string
	integer, intent(out)            :: num
	character (len=10)              :: afmt

	ifail=0
	afmt='(i    )'
	write(afmt(3:6),'(i4)')len(string)
	read(string,afmt,err=10) num
	return

10      ifail=1
	return

end subroutine a2i


subroutine a2r(ifail,string,num)

	integer, intent(out)            :: ifail
	character (len=*), intent(in)   :: string
	real, intent(out)               :: num
	character (len=10)              :: afmt

	ifail=0
	afmt='(f    .0)'
	write(afmt(3:6),'(i4)')len(string)
	read(string,afmt,err=10) num
	return

10      ifail=1
	return

end subroutine a2r


subroutine a2d(ifail,string,num)

	integer, intent(out)            :: ifail
	character (len=*), intent(in)   :: string
	double precision, intent(out)   :: num
	character (len=10)              :: afmt

	ifail=0
	afmt='(f    .0)'
	write(afmt(3:6),'(i4)')len(string)
	read(string,afmt,err=10) num
	return

10      ifail=1
	return

end subroutine a2d

!     Last change:  J    20 Dec 2004    1:04 pm
subroutine read_real_array(ifail,aprompt,array,pm_header,rows,columns)

! -- Subroutine read_real_array reads a real array from a file.

! -- Arguments are as follows:-
!       ifail:         returned as non-zero if error condition is encountered.
!       aprompt:       prompt text for real array filename
!       array:         real array read from file
!       pm_header:     if "yes" number of columns and rows expected as first
!                      line of input file.
!       rows,columns:  number of rows and columns in finite-difference grid

! -- Revision history:-
!       June-November, 1995:- version 1.

	use defn
	use inter

	integer, intent(out)                    :: ifail
	character (len=*), intent(inout)        :: aprompt
	real, intent(out),dimension(:,:)        :: array
	character (len=*), intent(in), optional :: pm_header
	integer, intent(in), optional           :: rows,columns
	character (len=120)                     :: afile,bfile
	character (len=4)                       :: extension
	logical                                 :: pm
	character (len=1)                       :: aformat
	integer                                 :: i,iunit,nncol,nnrow,nbb,ifail1
        integer                                 :: kstp,kper,ilay
        real                                    :: pertim,totim
        character (len=16)                      :: text

	ifail=0
	pm=.false.
	if(present(pm_header)) then
	  if(pm_header.ne.'yes') go to 5
	  if((.not.(present(rows))).or.(.not.(present(columns)))) &
	  call sub_error('READ_REAL_ARRAY')
	  pm=.true.
	end if

5       imessage=0
10      write(6,'(a)',advance='no') aprompt(1:len_trim(aprompt)+1)
	read(5,'(a)')afile
	afile=adjustl(afile)
	if(index(eschar,afile(1:2)).ne.0)then
	  escset=1
	  return
	end if
	if(afile.eq.' ') go to 10
        nbb=len_trim(afile)
        call getfile(ifail1,afile,bfile,1,nbb)
        if(ifail1.ne.0) go to 10
        afile=bfile
!       if(pm)then
!         aformat='f'
!       else
	  i=len_trim(afile)
	  extension=afile(max(i-3,1):i)
	  call casetrans(extension,'lo')
	  if(extension.eq.'.reu')then
	    aformat='u'
	  else if(extension.eq.'.ref')then
	    aformat='f'
	  else
20          if(aprompt(2:2).ne.' ') then
30            write(6,40,advance='no')
40            format(' Is this a formatted or unformatted file? [f/u]: ')
	    else
	      write(6,50,advance='no')
50            format('   is this a formatted or unformatted file? [f/u]: ')
	    end if
	    read(5,'(a)') aformat
	    if((aformat.eq.'e').or.(aformat.eq.'E')) then
	      write(6,*)
	      go to 10
	    end if
	    if(aformat.eq.' ') go to 20
	    call casetrans(aformat,'lo')
	    if((aformat.ne.'f').and.(aformat.ne.'u')) go to 20
	  end if
!       end if

	iunit=nextunit()
	if(aformat.eq.'f') then
	  open(unit=iunit,file=afile,status='old',err=100)
	  if(pm) then
	    read(iunit,'(a)',end=150) cline
	    call linesplit(ifail,2)
	    if(ifail.ne.0) go to 200
	    nncol=char2int(ifail,1)
	    if(ifail.ne.0) go to 300
	    nnrow=char2int(ifail,2)
	    if(ifail.ne.0) go to 300
	    if((nncol.ne.columns).or.(nnrow.ne.rows)) go to 250
	    read(iunit,*,err=350,end=400) array
	  else
	    read(iunit,*,err=450,end=500) array
	  end if
	else
	  open(unit=iunit,file=afile,status='old',form='binary',err=120)
	  read(iunit,end=600) kstp,kper,pertim,totim,text,nncol,nnrow,ilay
	  read(iunit,err=550,end=600) array
	end if
	if(aprompt(2:2).ne.' ') then
	  write(amessage,70) trim(afile)
70        format('  - real array read from file ',a)
	else
	  write(amessage,80) trim(afile)
80        format('    - real array read from file ',a)
	end if
	call write_message
	aprompt=afile
	go to 999

100     if(imessage.eq.5) go to 995
	if(pm)then
	  write(amessage,110) trim(afile)
110       format('cannot open formatted real array file ',&
	  a,' - try again.')
	else
	  write(amessage,111) trim(afile)
111       format('cannot open formatted real array file ',a,' - try again.')
	end if
	call write_message(error='yes',increment=1,endspace='yes')
	go to 10

120     if(imessage.eq.5) go to 995
	write(amessage,130) trim(afile)
130     format('cannot open unformatted real array file ',a,' - try again.')
	call write_message(error='yes',increment=1,endspace='yes')
	go to 10

150     write(amessage,160) trim(afile)
160     format('unexpected end to formatted real array file ',a,'.')
	call write_message(error='yes',leadspace='yes',endspace='yes')
	go to 995

200     write(amessage,210) trim(afile)
210     format('column and row numbers are expected on first line of ',&
	'real array file ',a,'.')
	call write_message(error='yes',leadspace='yes',endspace='yes')
	go to 995

250     write(amessage,260) trim(afile)
260     format('the row and/or column number provided in header to ',&
	'real array file ',a,' does not match that provided in ',&
	'grid specification file.')
	call write_message(error='yes',leadspace='yes',endspace='yes')
	go to 995

300     write(amessage,310) trim(afile)
310     format('cannot read row and/or column number from first line of ',&
	'formatted real array file ',a,'.')
	call write_message(error='yes',leadspace='yes',endspace='yes')
	go to 995

350     write(amessage,360) trim(afile)
360     format(' Data error encountered while reading formatted real ',&
	'array file ',a)
	call write_message(leadspace='yes',endspace='yes')
	go to 995

400     write(amessage,410) trim(afile)
410     format('unexpected end to formatted real array file ',a,'.')
	call write_message(error='yes',leadspace='yes',endspace='yes')
	go to 995

450     write(amessage,460) trim(afile)
460     format(' Data error encountered while reading formatted real ',&
	'array file ',a,'.')
	call write_message(leadspace='yes',endspace='yes')
	go to 995

500     write(amessage,510) trim(afile)
510     format('unexpected end to formatted real array file ',a,'.')
	call write_message(error='yes',leadspace='yes',endspace='yes')
	go to 995

550     write(amessage,560) trim(afile)
560     format(' Error encountered while reading unformatted real ',&
	'array file ',a,'.')
	call write_message(leadspace='yes',endspace='yes')
	go to 995

600     write(amessage,610) trim(afile)
610     format('unexpected end to unformatted real array file ',a,'.')
	call write_message(error='yes',leadspace='yes',endspace='yes')
	go to 995

995     ifail=1
999     close(unit=iunit,iostat=i)
	return

end subroutine read_real_array
subroutine write_message(increment,iunit,error,leadspace,endspace)

! -- Subroutine write_message formats and writes a message.

! -- Arguments are as follows:-
!       increment:  the increment to the message counter
!       iunit:      the unit number to which the message is written
!       error:      if "yes" precede message with "Error"
!       leadspace   if "yes" precede message with blank line
!       endspace    if "yes" follow message by blank line

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	integer, intent(in), optional           ::increment,iunit
	integer                                 ::jend,i,nblc,junit,leadblank
	integer                                 ::itake,j
	character (len=*), intent(in), optional ::error,leadspace,endspace
	character (len=20) ablank

	ablank=' '
	itake=0
	j=0
	if(present(increment)) imessage=imessage+increment
	if(present(iunit))then
	  junit=iunit
	else
	  junit=6
	end if
	if(present(leadspace))then
	  if(leadspace.eq.'yes') write(junit,*)
	endif
	if(present(error))then
	  if(index(error,'yes').ne.0)then
	    nblc=len_trim(amessage)
	    amessage=adjustr(amessage(1:nblc+8))
	    if(nblc+8.lt.len(amessage)) amessage(nblc+9:)=' '
	    amessage(1:8)=' Error: '
	  end if
	end if

	do i=1,20
	  if(amessage(i:i).ne.' ')exit
20      end do
	leadblank=i-1
	nblc=len_trim(amessage)
5       jend=j+78-itake
	if(jend.ge.nblc) go to 100
	do i=jend,j+1,-1
	if(amessage(i:i).eq.' ') then
	  if(itake.eq.0) then
	     write(junit,'(a)',err=200) amessage(j+1:i)
	     itake=2+leadblank
	  else
	     write(junit,'(a)',err=200) ablank(1:leadblank+2)//amessage(j+1:i)
	  end if
	  j=i
	  go to 5
	end if
	end do
	if(itake.eq.0)then
	  write(junit,'(a)',err=200) amessage(j+1:jend)
	  itake=2+leadblank
	else
	  write(junit,'(a)',err=200) ablank(1:leadblank+2)//amessage(j+1:jend)
	end if
	j=jend
	go to 5
100     jend=nblc
	if(itake.eq.0)then
	write(junit,'(a)',err=200) amessage(j+1:jend)
	  else
	write(junit,'(a)',err=200) ablank(1:leadblank+2)//amessage(j+1:jend)
	  end if
	if(present(endspace))then
	  if(endspace.eq.'yes') write(junit,*)
	end if
	return

200     call exit(100)

end subroutine write_message
