!     Last change:  JD   23 Jan 2001    0:37 am
MODULE DEFN

! -- File DEFN.F90 contains definitions for defined types and global variables.


!****************************************************************************
! defined types        
!****************************************************************************
	
	type modelgrid
	  integer                         :: nrow,ncol
	  double precision                :: east_corner,north_corner,rotation
	  real                            :: cosang,sinang
	  real, dimension(:), pointer     :: delr,delc
	  integer                         :: specunit,specline
	  character (len=80)              :: specfile
	end type modelgrid


        integer, parameter                :: MAX_STRUCT_VARIO=5
        type geostructure
          character (len=10)              :: structname
          integer                         :: numvariogram,transform
          real                            :: nugget,mean,maxpowercov
          real                            :: variogram_contrib(MAX_STRUCT_VARIO)
          character (len=10)              :: variogram_name(MAX_STRUCT_VARIO)
        end type geostructure

        type variogram
          character (len=10)   :: varname
          integer              :: vartype
          real                 :: angle,a,anis
        end type variogram


        integer, parameter                :: MAX_OUT_PARAM=100, &
                                             MAX_IN_PARAM=20
        type mf2k_param_replace
          character (len=10)              :: replacetype
          integer                         :: numout,numin
          character (len=10)              :: outparamname(MAX_OUT_PARAM)
          integer                         :: outparamlay1(MAX_OUT_PARAM), &
                                             outparamlay2(MAX_OUT_PARAM)
          integer                         :: newparamlay1(MAX_IN_PARAM), &
                                             newparamlay2(MAX_IN_PARAM)
          character (len=3)               :: newparamprefix
          integer                         :: transformtype
          real                            :: minival,maxival,minpval,maxpval
          character (len=120)             :: minifile,maxifile
          real, dimension(:,:), pointer   :: miniarray,maxiarray
        end type mf2k_param_replace


!****************************************************************************
!global variables
!****************************************************************************

!variables for reading a file ------->

	integer, parameter              	:: NUM_WORD_DIM=100
	integer, dimension(NUM_WORD_DIM)        :: left_word,right_word
	character (len=300)             	:: cline


!variables for writing a message ------->
	
	integer                 :: imessage=0
	character (len=500)     :: amessage= ' '
	character (len=200)     :: initial_message=' '


!escape variables ------->

	integer                 :: escset=0
	character (len=5)       :: eschar = 'E ~e '


!variables in bore data manipulation ------->
	
	integer                         :: num_bore_coord, num_bore_list
	character (len=120)             :: bore_coord_file, bore_list_file
	integer, dimension(:), pointer			:: bore_coord_layer
	double precision, dimension(:), pointer         :: bore_coord_east, &
							   bore_coord_north
	character (len=10), dimension(:), pointer       :: bore_coord_id, &
                                                           bore_list_id,  &
                                                           bore_diff_id

!variables in pilot point manipulation ------->
	
	integer                         :: num_pilot_points
	character (len=120)             :: pilot_points_file=' '
	integer, dimension(:), pointer			:: pilot_point_zone
	double precision, dimension(:), pointer         :: pilot_point_east, &
							   pilot_point_north, &
                                                           pilot_point_val, &
                                                           pilot_point_val1
	character (len=10), dimension(:), pointer       :: pilot_point_id

!Variables used in characterisation of a gms 2D mesh

        integer                                 :: numnode_g
        integer                                 :: numelem_g
        character (len=200)                     :: meshfile_g
        integer, dimension(:), pointer          :: nodenum_g(:)
        double precision, dimension(:), pointer :: eastnode_g(:), northnode_g(:)
        integer, dimension(:), pointer          :: elemnum_g(:),elemindex_g(:)
        integer, dimension(:), pointer          :: elemnode_g(:,:)
        double precision, dimension(:), pointer :: eastelem_g(:), northelem_g(:)

!       Note: integers in elemindex indicate the order in a sorting list of each element number.
!             That is elemnode(elemindex(3)) is the element with third lowest element number.

!Variables used in characterisation of a FEFLOW mesh

        integer                                 :: numnode_f
        integer                                 :: numelem_f
        character (len=200)                     :: epfile_f
        double precision, dimension(:), pointer :: eastelem_f(:), northelem_f(:)
        double precision, dimension(:), pointer :: datelem_f(:)
        integer, dimension(:), pointer          :: zoneelem_f(:)

!variables in MODFLOW 2000 parameter replacement ------->

        integer                     :: numparamreplace, numzoneremove, nummultremove
        character (len=12), dimension(:), pointer        :: removezone, removemult
        character (len=120)                              :: repfile
        type (mf2k_param_replace), dimension(:), pointer :: replace


!variables recording data settings ------->

	integer				:: datespec
	character (len=3)               :: headerspec


END MODULE DEFN
!     Last change:  J    14 Jun 2002   10:07 pm
MODULE INTER

! -- Contains interface blocks for all subprograms.


!******************************************************************************
! generic subprograms
!******************************************************************************

interface char2num

	subroutine a2i(ifail,string,num)
	  integer, intent(out)          :: ifail
	  character (len=*), intent(in) :: string
	  integer, intent(out)          :: num
	end subroutine a2i
	subroutine a2r(ifail,string,num)
	  integer, intent(out)          :: ifail
	  character (len=*), intent(in) :: string
	  real, intent(out)             :: num
	end subroutine a2r
	subroutine a2d(ifail,string,num)
	  integer, intent(out)          :: ifail
	  character (len=*), intent(in) :: string
	  double precision, intent(out) :: num
	end subroutine a2d

end interface


interface num2char

	subroutine i2a(value,string,nchar)
	  integer, intent(in)           :: value
	  character (len=*), intent(out):: string
	  integer, intent(in), optional :: nchar
	end subroutine i2a
	subroutine r2a(value,string,nchar)
	  real, intent(in)              :: value
	  character (len=*), intent(out):: string
	  integer, intent(in), optional :: nchar
	end subroutine r2a
	subroutine d2a(value,string,nchar)
	  double precision, intent(in)  :: value
	  character (len=*), intent(out):: string
	  integer, intent(in), optional :: nchar
	end subroutine d2a

end interface


interface pos_test

	integer function pos_i_test(value,string)
	  integer, intent(in)           :: value
	  character (len=*), intent(in) :: string
	end function pos_i_test
	integer function pos_r_test(value,string)
	  real, intent(in)              :: value
	  character (len=*), intent(in) :: string
	end function pos_r_test
	integer function pos_d_test(value,string)
	  double precision, intent(in)  :: value
	  character (len=*), intent(in) :: string
	end function pos_d_test

end interface


interface nneg_test

	integer function nneg_i_test(value,string)
	  integer, intent(in)           :: value
	  character (len=*), intent(in) :: string
	end function nneg_i_test
	integer function nneg_r_test(value,string)
	  real, intent(in)              :: value
	  character (len=*), intent(in) :: string
	end function nneg_r_test
	integer function nneg_d_test(value,string)
	  double precision, intent(in)  :: value
	  character (len=*), intent(in) :: string
	end function nneg_d_test

end interface


interface key_read

	integer function int_key_read(value)
	  integer,intent(out)   :: value
	end function int_key_read
	integer function real_key_read(value)
	  real,intent(out)      :: value
	end function real_key_read
	integer function double_key_read(value)
	  double precision,intent(out)  :: value
	end function double_key_read

end interface



interface equals

       logical function equals_int(r1,r2)
          integer, intent(in)    :: r1
          integer, intent(in)    :: r2
       end function equals_int

       logical function equals_real(r1,r2)
          real, intent(in)      :: r1
          real, intent(in)      :: r2
       end function equals_real

       logical function equals_dbl(r1,r2)
          real (kind (1.0d0)), intent(in)      :: r1
          real (kind (1.0d0)), intent(in)      :: r2
       end function equals_dbl

end interface



!******************************************************************************
! other subprograms
!******************************************************************************

! dxf subprograms ------->

interface
	subroutine write_dxf_header(iunit,xlo,xhi,ylo,yhi)
	  integer, intent(in)             :: iunit
	  double precision, intent(in)    :: xlo,xhi,ylo,yhi
	end subroutine write_dxf_header

	subroutine write_dxf_polyhead(iunit)
	  integer, intent(in)     :: iunit
	end subroutine write_dxf_polyhead

	subroutine write_dxf_vertex(iunit,x,y)
	  integer, intent(in)             :: iunit
	  double precision, intent(in)    :: x,y
	end subroutine write_dxf_vertex

	subroutine write_dxf_polyfin(iunit)
	  integer, intent(in)     :: iunit
	end subroutine write_dxf_polyfin

	subroutine write_dxf_finish(iunit)
	  integer,intent(in)      :: iunit
	end subroutine write_dxf_finish

end interface

! utility subprograms ------->

interface

	subroutine casetrans(string,hi_or_lo)
	  character (len=*), intent(inout)        :: string
	  character (len=*), intent(in)           :: hi_or_lo
	end subroutine casetrans

	subroutine sub_error(subname)
	  character (len=*)               ::subname
	end subroutine sub_error

	integer function nextunit()
	end function nextunit

	subroutine close_files
	end subroutine close_files

	subroutine open_input_file(ifail,aprompt,infile,inunit,file_format,form_prompt, &
          fformat)
          integer, intent(out)                     :: ifail
          character (len=*), intent(in)            :: aprompt
          character (len=*), intent(out)           :: infile
          integer, intent(out)                     :: inunit
          character (len=*), intent(in), optional  :: file_format
          character (len=*), intent(in), optional  :: form_prompt
          character (len=*), intent(out), optional :: fformat
	end subroutine open_input_file

        subroutine open_output_file(ifail,aprompt,outfile,outunit,file_format)
          integer, intent(out)                       :: ifail
          character (len=*)                          :: aprompt,outfile
          integer, intent(out)                       :: outunit
          character (len=*), intent(in), optional    :: file_format
        end subroutine open_output_file

	subroutine wrtsig(ifail,val,word,nw,precis,tval,nopnt)
	  integer               :: ifail,nw,precis,nopnt
	  double precision      :: val,tval
	  character (len=*)     :: word
	end subroutine wrtsig

	subroutine readfig(specfile,coordfile,sampfile,pumpfile,pilotfile)
	  character (len=*), intent(out)                :: specfile
	  character (len=*), intent(out), optional      :: coordfile,sampfile,&
							   pumpfile,pilotfile
	end subroutine readfig

	subroutine read_settings(ifail,idate,iheader)
	  integer, intent(out)	:: ifail,idate,iheader
	end subroutine read_settings

	subroutine char_add(astring,achar)
          character (len=*), intent(inout)        :: astring
          character (len=*), intent(in)           :: achar
	end subroutine char_add

	subroutine int2alph(inum,alph,nsig)
	  integer, intent(in)			:: inum
	  character (len=*), intent(out)	:: alph
	  integer, optional, intent(in)		:: nsig
	end subroutine int2alph

end interface

! interpolation subprograms ------->

interface

	subroutine factor(gridspec,east,north,fac1,fac2,fac3,fac4, &
	icellno,jcellno)
	  use defn
	  type(modelgrid), intent(in)           :: gridspec
	  double precision, intent(in)          :: east,north
	  real, intent(out)                     :: fac1,fac2,fac3,fac4
	  integer, intent(out)                  :: icellno,jcellno
	end subroutine factor

	subroutine point_interp(ncol,nrow,thresh,fac1,fac2,fac3,fac4, &
	icellno,jcellno,bhead,rarray,imethod)
	  integer, intent(in)                   :: ncol,nrow
	  real, intent(in)                      :: thresh
	  real, intent(in)                      :: fac1,fac2,fac3,fac4
	  integer, intent(in)                   :: icellno,jcellno
	  real, intent(out)                     :: bhead
	  real, dimension(0:ncol+1,0:nrow+1), intent(in)  :: rarray
	  character (len=*), intent(in), optional         :: imethod
	end subroutine point_interp

end interface

! reading-a-file subprograms ------->

interface

	subroutine linesplit(ifail,num)
	  integer, intent(out)            :: ifail
	  integer, intent(in)		  :: num
	end subroutine linesplit

	integer function char2int(ifail,num)
	  integer, intent(in)             :: num
	  integer, intent(out)            :: ifail
	end function char2int

	real function char2real(ifail,num)
	  integer, intent(in)             :: num
	  integer, intent(out)            :: ifail
	end function char2real

	double precision function char2double(ifail,num)
	  integer, intent(in)             :: num
	  integer, intent(out)            :: ifail
	end function char2double

        subroutine getfile(ifail,cline,filename,ibeg,iend)
          integer, intent(out)            :: ifail
          integer, intent(in)             :: ibeg
          integer, intent(inout)          :: iend
          character (len=*), intent(in)   :: cline
          character (len=*), intent(out)  :: filename
       end subroutine getfile

       subroutine addquote(afile,aqfile)
          character (len=*), intent(in)   :: afile
          character (len=*), intent(out)  :: aqfile
       end subroutine addquote

end interface

!integer array subprograms ------->

interface

	subroutine read_integer_array(ifail,aprompt,array,pm_header,rows, &
	columns,defaultfile)
	  integer, intent(out)                    :: ifail
	  character (len=*), intent(inout)        :: aprompt
	  integer, intent(out),dimension(:,:)     :: array
	  character (len=*), intent(in), optional :: pm_header
	  integer, intent(in), optional           :: rows,columns
          character (len=*), optional             :: defaultfile
	end subroutine read_integer_array

	subroutine write_integer_array(ifail,aprompt,array,pm_header,rows, &
	columns)
	  integer, intent(out)			  :: ifail
	  character (len=*), intent(inout)	  :: aprompt
	  integer, intent(in), dimension(:,:)	  :: array
	  character (len=*), intent(in), optional :: pm_header
	  integer, intent(in), optional		  :: rows,columns
	end subroutine write_integer_array

end interface

! real array subprograms ------->

interface

	subroutine read_real_array(ifail,aprompt,array,pm_header,rows,columns)
	  integer, intent(out)                    :: ifail
	  character (len=*), intent(inout)        :: aprompt
	  real, intent(out),dimension(:,:)        :: array
	  character (len=*), intent(in), optional :: pm_header
	  integer, intent(in), optional           :: rows,columns
	end subroutine read_real_array

	subroutine write_real_array(ifail,aprompt,array,pm_header,rows,columns,&
	binary_header,atype,ntrans,kstp,kper,pertim,totim,text,ncol,nrow,ilay,istage, &
        realfile,aaformat)
	  integer, intent(out)                    :: ifail
	  character (len=*), intent(inout)        :: aprompt
	  real, intent(in),dimension(:,:)         :: array
	  character (len=*), intent(in), optional :: pm_header
	  integer, intent(in), optional           :: rows,columns
          character (len=*), optional             :: binary_header
          character (len=*), optional             :: atype
          integer, optional                       :: ntrans,kstp,kper
          real, optional                          :: pertim,totim
          character (len=16),optional             :: text
          integer, optional                       :: ncol,nrow,ilay
          integer, intent(in), optional           :: istage
          character (len=*), optional             :: realfile
          character (len=*), optional             :: aaformat
        end subroutine write_real_array

	subroutine read_surf_array(ifail,aprompt,array,gridspec)
	  use defn
	  integer, intent(out)			:: ifail
	  character (len=*), intent(in)		:: aprompt
	  real, intent(out), dimension(:,:)	:: array
	  type(modelgrid), intent(in)		:: gridspec
	end subroutine read_surf_array

	subroutine write_surf_array(ifail,aprompt,array,thresh,gridspec)
	  use defn
	  integer, intent(out)			:: ifail
	  character (len=*), intent(in)		:: aprompt
	  real, intent(inout), dimension(:,:)	:: array
	  real, intent(in)			:: thresh
	  type(modelgrid), intent(in)		:: gridspec
	end subroutine write_surf_array

	subroutine write_real_table_file(ifail,outunit,outfile,intarray,realarray)
          use defn
          integer, intent(out)                    :: ifail
          integer, intent(in)                     :: outunit
          character (len=*), intent(in)           :: outfile
          integer, dimension(:,:), intent(in)     :: intarray
          real, dimension(:,:), intent(in)        :: realarray
	end subroutine write_real_table_file

end interface

! message subprograms ------->

interface

	subroutine write_initial_message(leadspace,endspace)
	  character (len=*), intent(in), optional :: leadspace,endspace
	end subroutine write_initial_message

	subroutine write_message(increment,iunit,error,leadspace,endspace)
	  integer, intent(in), optional           ::increment,iunit
	  character (len=*), intent(in), optional ::error,leadspace,endspace
	end subroutine write_message

end interface

! grid subprograms ------->

interface

	subroutine spec_open(ifail,gridspec)
	  use defn
	  integer, intent(out)	  :: ifail
	  type (modelgrid)        :: gridspec
	end subroutine spec_open

	subroutine read_spec_dim(ifail,gridspec)
	  use defn
	  type (modelgrid)                :: gridspec
	  integer, intent(out)            :: ifail
	end subroutine read_spec_dim

	subroutine read_spec_data(ifail,gridspec)
	  use defn
	  type (modelgrid)        :: gridspec
	  integer, intent(out)    :: ifail
	end subroutine read_spec_data

	subroutine close_spec_file(gridspec,ok)
	  use defn
	  type (modelgrid)                :: gridspec
	  character (len=*),optional      :: ok
	end subroutine close_spec_file

	subroutine free_grid_mem(gridspec)
	  use defn
	  type (modelgrid)        :: gridspec
	end subroutine free_grid_mem

	subroutine rel_centre_coords(east,north,gridspec)
	  use defn
	  real, intent(out), dimension(:,:)       :: east,north
	  type(modelgrid), intent(in)             :: gridspec
	end subroutine rel_centre_coords

	subroutine grid2earth(east,north,gridspec)
	  use defn
	  real, intent(inout),dimension(:,:)      :: east,north
	  type(modelgrid), intent(in)             :: gridspec
	end subroutine grid2earth

	subroutine transform_to_earth(earth_east,earth_north,grid_east, &
	grid_north,gridspec)
	  use defn
	  real, intent(out)               :: earth_east, earth_north
	  real, intent(in)                :: grid_east, grid_north
	  type(modelgrid), intent(in)     :: gridspec
	end subroutine transform_to_earth

	subroutine rc2cell(icellno,irow,icol,gridspec)
	  use defn
	  integer, intent(out)            :: icellno
	  integer, intent(in)             :: irow,icol
	  type(modelgrid), intent(in)     :: gridspec
	end subroutine rc2cell

	subroutine cell2rc(icellno,irow,icol,gridspec)
          use defn
          integer, intent(in)             :: icellno
          integer, intent(out)            :: irow,icol
          type (modelgrid), intent(in)    :: gridspec
	end subroutine cell2rc

	subroutine corner(ind,e,n,icol,irow,gridspec,ecg,ncg)
	  use defn
	  integer, intent(in)                     :: ind
	  real, intent(out)                       :: e,n
	  integer, intent(in)                     :: icol,irow
	  type (modelgrid), intent(in)            :: gridspec
	  real, dimension(:,:), intent(in)        :: ecg,ncg
	end subroutine corner


        subroutine cell_coordinates(gridspec,east,north,icellrow,icellcol,roff,coff)
          use defn
          type(modelgrid), intent(in)             :: gridspec
          double precision, intent(in)            :: east,north
          integer, intent(out)                    :: icellrow,icellcol
          real, intent(out)                       :: roff,coff
        end subroutine cell_coordinates


     
end interface

! bore data manipulation subprograms ------->

interface

	subroutine read_bore_coord_file(ifail,aprompt)
	  integer, intent(out)            :: ifail
	  character (len=*), intent(in)   :: aprompt
	end subroutine read_bore_coord_file

	subroutine read_bore_list_file(ifail,aprompt,coord_check)
	  integer, intent(out)                    :: ifail
	  character (len=*), intent(in)           :: aprompt
	  character (len=*), intent(in), optional :: coord_check
	end subroutine read_bore_list_file

	subroutine read_bore_diff_file(ifail,aprompt,coord_check)
	  integer, intent(out)                    :: ifail
	  character (len=*), intent(in)           :: aprompt
	  character (len=*), intent(in), optional :: coord_check
	end subroutine read_bore_diff_file

	subroutine free_bore_mem
	end subroutine free_bore_mem

	subroutine read_rest_of_sample_line(ifail,cols,ndays,nsecs,value, &
	iline,sampfile)
          integer, intent(out)            :: ifail
          integer, intent(in)             :: cols
          integer, intent(out)            :: ndays,nsecs
          double precision, intent(out)   :: value
          integer, intent(in)             :: iline
          character (len=*), intent(in)   :: sampfile
	end subroutine read_rest_of_sample_line

	subroutine read_rest_of_pump_line(ifail,ibore,ndays,nsecs,pumped, &
	iline,pmpfile)
          integer, intent(out)                    :: ifail
          integer, intent(inout)                  :: ibore
          integer, intent(out), dimension (:)     :: ndays,nsecs
          double precision, intent(inout), dimension(:)  :: pumped
          integer, intent(in)                     :: iline
          character (len=*), intent(in)           :: pmpfile
	end subroutine read_rest_of_pump_line

	subroutine time_interp(ifail,nbore,ndays,nsecs,value,intday, &
	intsec,rnear,rconst,valinterp,extrap,direction)
          integer, intent(out)                    :: ifail
          integer, intent(in)                     :: nbore
          integer, intent(in), dimension(nbore)   :: ndays,nsecs
          double precision, intent(in), dimension(nbore)   :: value
          integer, intent(in)                     :: intday,intsec
	  real, intent(in)			  :: rnear,rconst
          double precision, intent(out)           :: valinterp
	  character (len=*), intent(in),optional  :: extrap
	  character (len=*), intent(in),optional  :: direction
	end subroutine time_interp

	subroutine get_num_ids(ifail,iunit,afile,numid,maxsamp,ignore_x)
	  integer, intent(out)                    :: ifail
          integer, intent(in)                     :: iunit
          character (len=*), intent(in)           :: afile
          integer, intent(out)                    :: numid,maxsamp
	  character (len=*), intent(in), optional :: ignore_x
	end subroutine get_num_ids

	subroutine get_ids_and_interval(ifail,iunit,afile,nid,aid,ndays1, &
                                nsecs1,ndays2,nsecs2, ignore_x)
          integer, intent(out)                    :: ifail
          integer, intent(in)                     :: iunit
          character (len=*), intent(in)           :: afile
          integer, intent(in)                     :: nid
          character (len=*), intent(out)          :: aid(nid)
          integer, intent(out)                    :: ndays1(nid),nsecs1(nid), &
                                                     ndays2(nid),nsecs2(nid)
	  character (len=*), intent(in), optional :: ignore_x
	end subroutine get_ids_and_interval

end interface

! geostat subprograms ------->

interface

	subroutine read_pilot_points_file(ifail,aprompt,accept_blank)
	  integer, intent(out)                    :: ifail
	  character (len=*), intent(in)           :: aprompt
          character (len=*), intent(in), optional :: accept_blank
	end subroutine read_pilot_points_file

	subroutine read_dual_pilot_points_file(ifail,aprompt,accept_blank)
	  integer, intent(out)                    :: ifail
	  character (len=*), intent(in)           :: aprompt
          character (len=*), intent(in), optional :: accept_blank
	end subroutine read_dual_pilot_points_file


	subroutine free_point_mem
	end subroutine free_point_mem

        subroutine read_structure_file_dim(ifail,structunit,numstruct,numvario,structfile)
          integer, intent(out)            :: ifail
          integer, intent(in)             :: structunit
          integer, intent(out)            :: numstruct,numvario
          character (len=*), intent(in)   :: structfile
        end subroutine read_structure_file_dim

        subroutine read_rest_of_structure_file(ifail,structunit,numstruct,numvario, &
          structfile,structure,vario)
          use defn
          integer, intent(out)            :: ifail
          integer, intent(in)             :: structunit,numstruct,numvario
          character (len=*), intent(in)   :: structfile
          type (geostructure), intent(out):: structure(numstruct)
          type (variogram), intent(out)   :: vario(numvario)
        end subroutine read_rest_of_structure_file


        subroutine read_parameter_replacement_file(ifail,aprompt,ncol,nrow)
	  integer, intent(out)            :: ifail
	  character (len=*), intent(in)   :: aprompt
          integer, intent(in)             :: ncol,nrow
        end subroutine read_parameter_replacement_file


        subroutine free_replace_mem()
        end subroutine free_replace_mem

end interface


! RSM subprograms ------->

interface

        subroutine read_gms_2d_mesh_file(ifail,aprompt,centroid)
          integer, intent(out)            :: ifail
          character (len=*), intent(in)   :: aprompt
          character (len=*), intent(in)   :: centroid
        end subroutine read_gms_2d_mesh_file

        subroutine read_gms_integer_mesh_data_file(ifail,aprompt,intdata)
          integer, intent(out)             :: ifail
          character (len=*), intent(inout) :: aprompt
          integer, intent(out)             :: intdata(:)
        end  subroutine read_gms_integer_mesh_data_file

        subroutine read_gms_real_mesh_data_file(ifail,aprompt,realdata)
          integer, intent(out)             :: ifail
          character (len=*), intent(inout) :: aprompt
          real, intent(out)                :: realdata(:)
        end  subroutine read_gms_real_mesh_data_file

end interface


! FEFLOW subprograms ------->

interface

        subroutine read_feflow_elem_prop_file(ifail,aprompt)
          integer, intent(out)            :: ifail
          character (len=*), intent(in)   :: aprompt
        end subroutine read_feflow_elem_prop_file

end interface



! date manipulation subprograms ------->

interface

	subroutine char2date(ifail,adate,dd,mm,yy)
          integer, intent(out)    	:: ifail
          character (len=*), intent(in) :: adate
          integer, intent(out) 		:: dd,mm,yy
	end subroutine char2date

	subroutine datestring(dd,mm,yy,hhh,mmm,sss,time,at,adate,atime)
	  integer, intent(in)             :: dd,mm,yy,hhh,mmm,sss
	  real, intent(in)                :: time
	  character (len=1), intent(in)   :: at
	  character (len=*), intent(out)  :: adate, atime
	end subroutine datestring

	logical function leap(year)
          integer, intent(in)     :: year
	end function leap

	integer function numdays(dr,mr,yr,d,m,y)
          integer, intent(in)     :: dr,mr,yr,d,m,y
	end function numdays

	integer function numsecs(h1,m1,s1,h2,m2,s2)
	  integer, intent(in)     :: h1,m1,s1,h2,m2,s2
	end function numsecs

	subroutine char2time(ifail,adate,hh,mm,ss)
          integer, intent(out)    	:: ifail
          character (len=*), intent(in) :: adate
          integer, intent(out) 		:: hh,mm,ss
	end subroutine char2time

	subroutine time2char(ifail,hh,mm,ss,atime)
	  integer, intent(out)            :: ifail
	  integer, intent(in)             :: hh,mm,ss
	  character (len=*), intent(out)  :: atime
	end subroutine time2char

	subroutine elapsdate(eltime,dayfactor,day1,mon1,year1,hour1,min1,sec1,&
	  day2,mon2,year2,hour2,min2,sec2)
	  real, intent(in)		:: eltime,dayfactor
	  integer, intent(in)		:: day1,mon1,year1,hour1,min1,sec1
	  integer, intent(out)		:: day2,mon2,year2,hour2,min2,sec2
	end subroutine elapsdate

	subroutine newdate(ndays,day1,mon1,year1,day2,mon2,year2)
	  integer, intent(in)		:: ndays,day1,mon1,year1
	  integer, intent(out)		:: day2,mon2,year2
	end subroutine newdate

	subroutine sectime(nsecs,sec,min,hour)
	  integer, intent(in)   :: nsecs
	  integer, intent(out)  :: sec,min,hour
	end subroutine sectime

end interface


! mapinfo interface programs ------->


interface

	subroutine write_mif_file(ifail,outunit,outfile,intarray,east,north, &
		        gridspec,izone,type)

          use defn
          integer, intent(out)                    :: ifail
          integer, intent(in)                     :: outunit
          character (len=*)                       :: outfile
          integer, dimension(:,:), intent(in)     :: intarray
          real, dimension(:,:), intent(in)        :: east,north
          type (modelgrid), intent(in)            :: gridspec
          integer, intent(in)                     :: izone
          character (len=1)                       :: type
	end subroutine write_mif_file


	subroutine write_real_mid_file(ifail,outunit,outfile,intarray,realarray)
          use defn
          integer, intent(out)                    :: ifail
          integer, intent(in)                     :: outunit
          character (len=*), intent(in)           :: outfile
          integer, dimension(:,:), intent(in)     :: intarray
          real, dimension(:,:), intent(in)        :: realarray
	end subroutine write_real_mid_file


	subroutine write_integer_mid_file(ifail,outunit,outfile,intarraywin, &
	  intarraydat)
          use defn
          integer, intent(out)                    :: ifail
          integer, intent(in)                     :: outunit
          character (len=*), intent(in)           :: outfile
          integer, dimension(:,:), intent(in)     :: intarraywin,intarraydat
	end subroutine write_integer_mid_file


	subroutine read_int_tab_file(ifail,tabunit,tabfile,intarray,gridspec)
	  use defn
	  integer, intent(out)			  :: ifail
	  integer, intent(in)			  :: tabunit
	  character (len=*), intent(in)		  :: tabfile
	  integer, dimension(:,:), intent(inout)  :: intarray
	  type(modelgrid), intent(in)		  :: gridspec
	end subroutine read_int_tab_file


	subroutine read_real_tab_file(ifail,tabunit,tabfile,realarray,gridspec)
	  use defn
	  integer, intent(out)			  :: ifail
	  integer, intent(in)			  :: tabunit
	  character (len=*), intent(in)		  :: tabfile
	  real, dimension(:,:), intent(inout)     :: realarray
	  type(modelgrid), intent(in)		  :: gridspec
	end subroutine read_real_tab_file

	subroutine read_real_tab_file_int(ifail,mode,tabunit,tabfile,realarray, &
                                          intarray,gridspec,insunit,insfile,aprefix)
	  use defn
	  integer, intent(out)			  :: ifail
          integer, intent(in)                     :: mode
	  integer, intent(in)			  :: tabunit
	  character (len=*), intent(in)		  :: tabfile
	  real, dimension(:,:), intent(inout)     :: realarray
          integer, dimension(:,:), intent(inout)  :: intarray
	  type(modelgrid), intent(in)		  :: gridspec
          integer, intent(in)                     :: insunit
          character (len=*), intent(in)           :: insfile
          character (len=*), intent(in)           :: aprefix
	end subroutine read_real_tab_file_int


end interface


END MODULE INTER
!     Last change:  J     2 May 2002    8:49 pm
program mod2obs

! -- Program MOD2OBS writes a bore sample file of model-generated heads over
!    time, interpolated to the sites and times of samples supplied in an 
!    existing bore sample file.

	use defn
	use inter

	implicit none

	logical      :: active
	integer      :: ifail,ierr,ncol,nrow,i,j,modunit,dds,mms,yys,hhhs,&
	    		mmms,ssss,outunit,iarray,mcol,mrow,kstp,kper,&
			ntrans,ilay,cols,ndays,nsecs,k, &
			icol,irow,idate,sampunit,ilt0,iheader,   &
                        ndaystart,nsecstart,nlay,iline,numsamp1,jindex, &
                        nindex,day,mon,year,hour,sec,min,iflag,kk,kmin,ii,nind1
	real	     :: thresh,gt_thresh,day_convert,pertim,totim,  &
	                r_day_convert,bore_time,rtemp1,rtemp2,rlim,rrlim
	integer, allocatable, dimension(:)         :: layer,icellno,jcellno, &
                                                      numsamp,index_bore,iorder
	real, allocatable, dimension(:,:,:)        :: rarray1,rarray2
	real, allocatable, dimension(:)	           :: fac1,fac2,fac3,fac4
	integer, allocatable, dimension(:)         :: bore_days(:),bore_secs(:)
	real, allocatable, dimension(:)            :: bore_val(:)
	real, allocatable, dimension (:)           :: arrtime1(:),arrtime2(:)
	double precision                           :: dtemp
	double precision, allocatable, dimension(:):: east,north
	type (modelgrid) gridspec
	character (len=10)  :: abore, aboreold, alay, alim
	character (len=80)  :: modfile,outfile,sampfile
	character (len=1)   :: af,at
	character (len=15)  :: adate,atime,anum1
	character (len=16)  :: text
	character (len=20)  :: aval

!	open(unit=*,action='read',carriagecontrol='list')

	write(amessage,5)
5       format(' Program MOD2OBS writes a bore sample file of model-generated ',&
	'heads over time, interpolated to the sites and times ',   &
        'supplied in an existing bore sample file.')
	call write_message(leadspace='yes',endspace='yes')

	include 'unformat.inc'
	
        call read_settings(ifail,idate,iheader)
        if(ifail.eq.1) then
          write(amessage,7)
7         format(' A settings file (settings.fig) was not found in the ', &
          'current directory.')
          call write_message
          go to 9900
        else if(ifail.eq.2) then
          write(amessage,8)
8         format(' Error encountered while reading settings file settings.fig')
          call write_message
          go to 9900
        endif
        if((idate.ne.0).or.(datespec.eq.0)) then
          write(amessage,9)
9         format(' Cannot read date format from settings file ', &
          'settings.fig')
          call write_message
          go to 9900
        end if

!	open(unit=75,file='debug.dat')		!debug

	call readfig(gridspec%specfile,bore_coord_file,sampfile)
10      call spec_open(ifail,gridspec)
	if(ifail.ne.0) go to 9900
	if(escset.eq.1) go to 9900
	call read_spec_dim(ifail,gridspec)
	if(ifail.ne.0) go to 9900
	call read_spec_data(ifail,gridspec)
	if(ifail.ne.0) go to 9900
	call close_spec_file(gridspec,ok='yes')

	ncol=gridspec%ncol
	nrow=gridspec%nrow

30      call read_bore_coord_file(ifail, &
	' Enter name of bore coordinates file: ')
	if(ifail.ne.0) go to 9900
	if(escset.ne.0) then
	  escset=0
	  write(6,*)
	  go to 10
	end if
	do i=1,num_bore_coord
	  if(bore_coord_layer(i).eq.-999) go to 80
	end do
	go to 100
80	write(amessage,90) trim(bore_coord_file)
90	format(' The following bores were not provided with layer numbers in ',&
	'bore coordinates file ',a,' ----->')
	call write_message(leadspace='yes')
	j=2
	imessage=0
	write_bores: do i=1,num_bore_coord
	  if(bore_coord_layer(i).eq.-999) then
	    write(amessage(j:),'(a)') trim(bore_coord_id(i))
	    j=j+11
	    if(j.ge.71) then
	      call write_message(increment=1)
	      if(imessage.gt.12) goto 9900
	      j=2
	    end if
	  end if
	end do write_bores
	if(j.ne.2) call write_message
	go to 9900

100     call read_bore_list_file(ifail, &
       ' Enter name of bore listing file: ')
	if(ifail.ne.0) go to 9900
	if(escset.ne.0) then
	  escset=0
	  write(6,*)
	  go to 30
	end if

	if(num_bore_list.gt.1) then
	  do i=1,num_bore_list-1
	    do j=i+1,num_bore_list
	      if(bore_list_id(i).eq.bore_list_id(j))then
	        write(amessage,110) trim(bore_list_file)
110		format(' Execution of program MOD2OBS cannot proceed as ',&
		'there are multiple occurrences of the same bore in bore ',&
		'listing file ',a)
		go to 9890
	      end if
	    end do
	  end do
	end if

	allocate(east(num_bore_list), north(num_bore_list), &
	layer(num_bore_list),&
	fac1(num_bore_list), fac2(num_bore_list), fac3(num_bore_list), &
	fac4(num_bore_list), icellno(num_bore_list), jcellno(num_bore_list), &
	numsamp(num_bore_list), index_bore(num_bore_list), &
        iorder(num_bore_list),stat=ierr)
	if(ierr.ne.0) then
	  write(amessage,130)
130	  format(' Insufficient memory available to continue execution.')
	  go to 9890
	end if
        index_bore=0

	imessage=0
	list: do i=1,num_bore_list
	  do j=1,num_bore_coord
	    if(bore_coord_id(j).eq.bore_list_id(i)) then
	      east(i)=bore_coord_east(j)
	      north(i)=bore_coord_north(j)
	      layer(i)=bore_coord_layer(j)
	      cycle list
	    end if
	  end do
	  write(amessage,140) trim(bore_list_id(i)),trim(bore_list_file),&
	  trim(bore_coord_file)
140	  format(' No coordinates for bore ',a,' from bore listing ',&
	  'file ',a,' are provided in bore coordinates file ',a)
	  if(imessage.eq.0) write(6,*)
	  call write_message(increment=1)
	end do list
	if(imessage.ne.0) go to 9900

	do i=1,num_bore_list
	  call factor(gridspec,east(i),north(i),fac1(i),fac2(i),fac3(i), &
	  fac4(i),icellno(i),jcellno(i))
	end do
	if(all(icellno.eq.-999)) then
	  write(amessage,145) trim(bore_list_file),trim(gridspec%specfile)
145	  format(' None of the bores cited in bore listing file ',a, &
	  ' are within the bounds of the finite difference grid as defined ',&
	  'in grid specification file ',a)
	  go to 9890
	end if

600	call open_named_input_file(ifail,          &
	' Enter name of bore sample file: ',sampfile,sampunit)
	if(ifail.ne.0) go to 9900
	if(escset.ne.0) then
	  escset=0
	  write(6,*)
	  deallocate(east,north,layer,fac1,fac2,fac3,fac4,icellno,jcellno,  &
          numsamp,index_bore,iorder,stat=ierr)
	  if(ierr.ne.0) go to 9300
	  go to 100
	end if

150	write(6,*)
	call open_input_file(ifail, &
	' Enter name of unformatted model-generated file: ',modfile,modunit, &
	file_format='unformatted')
	if(ifail.ne.0) go to 9900
	if(escset.ne.0)then
	  escset=0
	  write(6,*)
	  close(unit=sampunit)
	  go to 600
	end if

170	write(6,180,advance='no')
180	format(' Is this a MODFLOW or MT3D file?  [f/t]: ')
	read(5,'(a)') af
	if(af.eq.' ') go to 170
	call casetrans(af,'lo')
	if(index(eschar,af).ne.0) then
	  close(unit=modunit)
	  go to 150
	end if
	if((af.ne.'f').and.(af.ne.'t')) go to 170

200	write(6,210,advance='no')
210	format(' Enter inactive threshold value for numbers in this file: ')
	i=key_read(thresh)
	if(escset.ne.0)then
	  escset=0
	  write(6,*)
	  go to 170
	else if(i.eq.-1) then
	  go to 200
	else if(i.ne.0) then
	  write(6,220)
220	  format(' Illegal input  - try again.')
	  go to 200
	end if
	if(thresh.gt.1.0e37) then
	  write(amessage,230)
230	  format(' Threshold must be less than 1.0E37  - try again.')
	  call write_message
	  go to 200
	end if

250	write(6,260,advance='no')
260	format(' Enter time units used by model (yr/day/hr/min/sec) [y/d/h/m/s]: ')
	read(5,'(a)') at
	if(at.eq.' ') go to 250
	if(index(eschar,at).ne.0) then
	  write(6,*)
	  go to 200
	end if
	call casetrans(at,'lo')
	if(at.eq.'s') then
	  day_convert=1.0/86400.0
	else if(at.eq.'m') then
	  day_convert=1.0/1440.0
	else if(at.eq.'h') then
	  day_convert=1.0/24.0
	else if(at.eq.'d') then
	  day_convert=1.0
	else if(at.eq.'y') then
	  day_convert=365.25
	else
	  go to 250
	end if
	r_day_convert=1.0/day_convert

300	write(6,*)
310	if(datespec.eq.1) then
	  write(6,320,advance='no')
320	  format(' Enter simulation starting date [dd/mm/yyyy]: ')
	else
	  write(6,321,advance='no')
321	  format(' Enter simulation starting date [mm/dd/yyyy]: ')
	end if
	read(5,'(a)') adate
	if(adate.eq.' ') go to 310
	adate=adjustl(adate)
	if(index(eschar,adate(1:2)).ne.0) then
	  write(6,*)
	  go to 250
	end if
	call char2date(ifail,adate,dds,mms,yys)
	if(ifail.ne.0)then
	  write(6,340)
340	  format(' Illegal date  - try again.')
	  go to 310
	end if

360	write(6,370,advance='no')
370	format(' Enter simulation starting time [hh:mm:ss]: ')
	read(5,'(a)') atime
	if(atime.eq.' ') go to 360
	atime=adjustl(atime)
	if(index(eschar,atime(1:2)).ne.0) go to 300
	call char2time(ifail,atime,hhhs,mmms,ssss)
	if(ifail.ne.0)then
	  write(6,380)
380	  format(' Illegal time  - try again.')
	  go to 360
	end if
	ndaystart=numdays(1,1,1970,dds,mms,yys)
	nsecstart=numsecs(0,0,0,hhhs,mmms,ssss)
390	write(6,*)
395	write(6,396,advance='no')
396	format(' How many layers in the model? ')
	read(5,'(a)') alay
	if(alay.eq.' ') go to 395
	alay=adjustl(alay)
	if(index(eschar,alay(1:2)).ne.0) then
	  write(6,*)
	  go to 360
	end if
	call char2num(ifail,alay,nlay)
	if(ifail.ne.0) go to 395
	if(nlay.le.0) go to 395

440	write(amessage,445)
445	format(' If a sample time does not lie between model output ',  &
        'times, or if there is only one model output time, value at ', &
	'the sample time can equal that at nearest model output time:-')
	call write_message(leadspace='yes')
465	write(6,467,advance='no')
467	format('   Enter extrapolation limit in days (fractional if ', &
        'necessary): ')
	read(5,'(a)') alim
	if(alim.eq.' ') go to 465
	alim=adjustl(alim)
	if(index(eschar,alim(1:2)).ne.0) then
	  go to 390
	end if
	call char2num(ifail,alim,rlim)
	if(ifail.ne.0) go to 465
	if(rlim.lt.0.0) go to 465
	rrlim=rlim
	rlim=rlim*r_day_convert

400	write(6,*)
410	call open_output_file(ifail, &
	' Enter name for bore sample output file: ',outfile,outunit)
	if(ifail.ne.0) go to 9900
	if(escset.ne.0) then
	  escset=0
	  go to 440
	end if

	deallocate(bore_coord_id,bore_coord_east,bore_coord_north, &
	bore_coord_layer,stat=ierr)
	if(ierr.ne.0) go to 9300
	nullify(bore_coord_id,bore_coord_east,bore_coord_north, &
	bore_coord_layer)	
	allocate(rarray1(0:ncol+1,0:nrow+1,nlay),                  &
        rarray2(0:ncol+1,0:nrow+1,nlay),arrtime1(nlay),arrtime2(nlay),stat=ierr)
	if(ierr.ne.0) then
	  write(amessage,480)
480	  format(' Insufficient available memory to continue MOD2OBS ',&
	  'execution: try using fewer bores, a smaller bore sample file, or a ',&
	  'smaller model grid.')
	  go to 9890
	end if

	aboreold=' '
	active=.false.
	iline=0
	imessage=0
	numsamp=0
	jindex=0
        read_sample: do
          iline=iline+1
          if(iline.eq.2000) write(6,590)
590       format(/,' Working.....')
          read(sampunit,'(a)',err=9600,end=700) cline
          cols=5
          call linesplit(ifail,5)
          if(ifail.lt.0) cycle read_sample
          if(ifail.gt.0) then
            call linesplit(ifail,4)
            if(ifail.ne.0) go to 9150
            cols=4
          end if
          if(right_word(1)-left_word(1).gt.9) go to 9500
          abore=cline(left_word(1):right_word(1))
          call casetrans(abore,'hi')
	  if(abore.ne.aboreold)then
	    aboreold=abore
	    if(active) numsamp(i)=numsamp1
	    do i=1,num_bore_list
	      if(abore.eq.bore_list_id(i)) go to 620
	    end do
	    active=.false.
	    cycle read_sample
620	    numsamp1=0
	    active=.true.
	    index_bore(i)=jindex+1
	  else
	    if(.not.active) cycle read_sample
	  end if
          call read_rest_of_sample_line(ifail,cols,ndays,nsecs,dtemp, &
          iline,sampfile)
          if(ifail.ne.0) go to 9900
	  if(dtemp.lt.-1.0e38) cycle read_sample
          if(ndays.lt.ndaystart-rrlim-1) cycle read_sample
	  jindex=jindex+1
	  numsamp1=numsamp1+1
	end do read_sample

700	if(active) numsamp(i)=numsamp1
	nindex=jindex
	allocate(bore_days(nindex),bore_secs(nindex),bore_val(nindex), &
        stat=ierr)
	if(ierr.ne.0) then
	  write(amessage,480)
	  go to 9890
	end if
	bore_val=-1.1e38
	nind1=nindex

! -- Now we re-read the bore sample file to read in data.

	rewind(unit=sampunit,iostat=ierr)
	if(ierr.ne.0)then
	  write(amessage,720) trim(sampfile)
720	  format(' Error re-winding bore sample file ',a)
	  go to 9890
	end if

	aboreold=' '
	jindex=0
	iline=0
	read_sample_1: do
	  iline=iline+1
	  read(sampunit,'(a)',err=9600,end=800) cline
	  cols=5
	  call linesplit(ifail,5)
	  if(ifail.lt.0) cycle read_sample_1
	  if(ifail.gt.0)then
	    call linesplit(ifail,4)
	    if(ifail.ne.0) go to 9150
	    cols=4
	  end if
	  abore=cline(left_word(1):right_word(1))
	  call casetrans(abore,'hi')
	  if(abore.ne.aboreold)then
	    aboreold=abore
	    do i=1,num_bore_list
	      if(abore.eq.bore_list_id(i)) go to 820
	    end do
	    active=.false.
	    cycle read_sample_1
820	    active=.true.
	  else
	    if(.not.active) cycle read_sample_1
	  end if
	  call read_rest_of_sample_line(ifail,cols,ndays,nsecs,dtemp,iline, &
	  sampfile)
	  if(ifail.ne.0) go to 9900
	  if(dtemp.lt.-1.0e38) cycle read_sample_1
	  if(ndays.lt.ndaystart-rrlim-1) cycle read_sample_1
	  jindex=jindex+1
	  bore_days(jindex)=ndays
	  bore_secs(jindex)=nsecs
	end do read_sample_1
800	continue

!	do i=1,nindex					!debug
!	  write(75,*) bore_days(i),bore_secs(i)		!debug
!	end do						!debug
!	write(75,*)					!debug
!	do i=1,num_bore_list				!debug
!	  write(75,*) index_bore(i), numsamp(i)		!debug
!	end do						!debug

	rarray2=0.0
        gt_thresh=thresh+2*spacing(thresh)
        rarray2(0,:,:)=gt_thresh
        rarray2(ncol+1,:,:)=gt_thresh
        rarray2(:,0,:)=gt_thresh
        rarray2(:,nrow+1,:)=gt_thresh

	iarray=0
	arrtime1=0.0
	arrtime2=0.0
	read_an_array: do
	  iarray=iarray+1
	  if(af.eq.'f')then
	    mrow=0
	    mcol=0
	    read(modunit,err=9100,end=1000) kstp,kper,pertim,totim, &
	    text,mcol,mrow,ilay
	  else
	    mrow=0
	    mcol=0
	    read(modunit,err=9100,end=1000) ntrans,kstp,kper,&
	    totim,text,mcol,mrow,ilay
	  end if
	  if((mcol.le.0).or.(mrow.le.0).or.(ilay.le.0)) go to 9100
	  if((mrow.ne.nrow).or.(mcol.ne.ncol)) then
	    call num2char(iarray,anum1)
	    write(amessage,450) trim(anum1),trim(modfile),trim(gridspec%specfile)
450	    format(' Number of rows and columns read from header to array ',&
	    'number ',a,' from model output file ',a,' does not agree with ',&
	    'grid specifications as read from grid specification file ',a,   &
            '. Alternatively, unformatted file protocol might be a problem - ', &
            'try using the alternative version of this program.')
	    go to 9890
	  end if
	  if(ilay.gt.nlay)then
	    call num2char(iarray,anum1)
	    write(amessage,460) trim(anum1),trim(modfile)
460	    format(' Number of layers read from header to array number ', &
	    a,' from model output file ',a,' exceeds number of model ', &
	    'layers as supplied by user.')
	    go to 9890
	  end if

	  rarray1(:,:,ilay)=rarray2(:,:,ilay)
	  read(modunit,err=9200,end=9250)   &
          ((rarray2(icol,irow,ilay),icol=1,ncol),irow=1,nrow)
	  arrtime1(ilay)=arrtime2(ilay)
	  arrtime2(ilay)=totim
	  if(arrtime1(ilay).eq.0.0) then
	    interp: do i=1,num_bore_list
	      if(layer(i).ne.ilay) cycle interp
	      if(numsamp(i).eq.0) cycle interp
	      jindex=index_bore(i)
	      do j=jindex,jindex+numsamp(i)-1
	        bore_time=(bore_days(j)-ndaystart)*r_day_convert +       &
	        (bore_secs(j)-nsecstart)/86400.0*r_day_convert
	        if((bore_time.ge.arrtime2(ilay)-rlim).and.               &
	           (bore_time.le.arrtime2(ilay)+rlim))then
	           call point_interp(ncol,nrow,thresh,fac1(i),fac2(i),  &
	           fac3(i),fac4(i),icellno(i),jcellno(i),bore_val(j),   &
	           rarray2(0,0,ilay))
	        end if
	      end do
	    end do interp
	    cycle read_an_array
	  end if
	  interpolate: do i=1,num_bore_list
	    if(layer(i).ne.ilay) cycle interpolate
	    if(numsamp(i).eq.0) cycle interpolate
	    jindex=index_bore(i)
	    do j=jindex,jindex+numsamp(i)-1
!             if(bore_val(j).gt.-1.0e38) cycle 
	      bore_time=(bore_days(j)-ndaystart)*r_day_convert +       &
              (bore_secs(j)-nsecstart)/86400.0*r_day_convert
	      if(bore_time.gt.arrtime2(ilay)+rlim)then
	        continue
	      else if(bore_time.lt.arrtime1(ilay)-rlim)then
	        continue
	      else if(bore_time.lt.arrtime1(ilay))then
	        continue
!	        call point_interp(ncol,nrow,thresh,fac1(i),fac2(i),  &
!                fac3(i),fac4(i),icellno(i),jcellno(i),bore_val(j),   &
!	        rarray1(0,0,ilay))
	      else if(bore_time.ge.arrtime2(ilay))then
	        call point_interp(ncol,nrow,thresh,fac1(i),fac2(i),  &
                fac3(i),fac4(i),icellno(i),jcellno(i),bore_val(j),   &
	        rarray2(0,0,ilay))
	      else
	        call point_interp(ncol,nrow,thresh,fac1(i),fac2(i),  &
                fac3(i),fac4(i),icellno(i),jcellno(i),rtemp1,        &
	        rarray1(0,0,ilay))
	        call point_interp(ncol,nrow,thresh,fac1(i),fac2(i),  &
                fac3(i),fac4(i),icellno(i),jcellno(i),rtemp2,        &
	        rarray2(0,0,ilay))
	        bore_val(j)=rtemp1+(rtemp2-rtemp1)*(bore_time-arrtime1(ilay)) &
	        /(arrtime2(ilay)-arrtime1(ilay))
	      end if
	    end do
	  end do interpolate

	end do read_an_array

!note: this will not work if there is only one array per layer. There
!have to be many arrays per layers corresponding to different times.

1000	if(iarray.eq.1) then
	  write(amessage,1010) trim(modfile)
1010	  format(' No arrays were found in model-generated unformatted ',&
	  'file ',a)
	  go to 9890
	end if
	write(6,*)
	close(unit=modunit)
	iarray=iarray-1
	call num2char(iarray,anum1)
	write(amessage,1050) trim(anum1),trim(modfile)
1050	format(' - ',a,' arrays read from file ',a)
	call write_message

	iorder=index_bore
	do ii=1,num_bore_list

	  kmin=nind1+10
	  ilt0=0
          do kk=1,num_bore_list
	    if(numsamp(kk).eq.0) cycle
	    if(iorder(kk).lt.0) cycle
	    ilt0=ilt0+1
	    if(iorder(kk).lt.kmin)then
	      kmin=iorder(kk)
	      i=kk
	    end if
	  end do
	  if(ilt0.eq.0) go to 1150
!	  write(75,*) ilt0				!debug
	  iorder(i)=-1

	  if(numsamp(i).eq.0) cycle
	  jindex=index_bore(i)
	  do j=jindex,jindex+numsamp(i)-1
	  if(bore_val(j).lt.-1.0e38)then
	    cycle
!	    aval='cannot_interpolate'
	  else if(bore_val(j).gt.7.0e37)then
	    aval='not_in_grid'
	  else if (bore_val(j).gt.4.0e37)then
	    aval='dry_or_inactive'
	  else
	    write(aval,'(1pg14.7)') bore_val(j)
	  end if
	  aval=adjustl(aval)
	  call newdate(bore_days(j),1,1,1970,day,mon,year)
	  hour=bore_secs(j)/3600
	  sec=bore_secs(j)-hour*3600
	  min=sec/60
	  sec=sec-min*60
	  if(datespec.eq.1)then
	    write(outunit,1100,err=9400) trim(bore_list_id(i)), &
	    day,mon,year,hour,min,sec,trim(aval)
	  else
	    write(outunit,1100,err=9400) trim(bore_list_id(i)), &
	    mon,day,year,hour,min,sec,trim(aval)
	  end if
1100	  format(1x,a,t14,i2.2,'/',i2.2,'/',i4.4,t28,i2.2,':',i2.2,':',i2.2,&
	  t40,a)
	  end do
	end do
1150	close(unit=outunit)
	write(amessage,1200) trim(outfile)
1200	format(' - bore sample file ',a,' written ok.')
	call write_message

	iflag=0
	if(any(numsamp.eq.0))then
	  iflag=1
	  go to 1249
	end if
	do i=1,num_bore_list
	  if(numsamp(i).ne.0) then
	    do j=index_bore(i),index_bore(i)+numsamp(i)-1
	      if(bore_val(j).gt.-1e38) goto 1230
	    end do
	    go to 1240
1230	  continue
	  end if
	end do
	go to 1249
1240	iflag=1
1249	continue
	if(iflag.eq.0) go to 1300

	write(amessage,1250) trim(bore_list_file),trim(outfile)
1250	format(' The following bores appearing in the bore listing ', &
	'file ',a,' will not appear in the bore sample file ',a,':-')
	call write_message(leadspace='yes')
	amessage=' '
        j=4
        imessage=0
        write_bores_1: do i=1,num_bore_list
          if(numsamp(i).eq.0) then
            write(amessage(j:),'(a)') trim(bore_list_id(i))
            j=j+11
            if(j.ge.69) then
              call write_message(increment=1)
              if(imessage.gt.12) goto 9900
              j=4
            end if
          end if
        end do write_bores_1
	write_bores_2: do i=1,num_bore_list
	  if(numsamp(i).ne.0)then
	    do k=index_bore(i),index_bore(i)+numsamp(i)-1
	      if(bore_val(k).gt.-1.0e38) go to 1290
	    end do
            write(amessage(j:),'(a)') trim(bore_list_id(i))
            j=j+11
            if(j.ge.69) then
              call write_message(increment=1)
              if(imessage.gt.12) goto 9900
              j=4
            end if
1290	    continue
	  endif
	end do write_bores_2
        if(j.ne.4) call write_message

1300	continue
	go to 9900

9100	call num2char(iarray,anum1)
	write(amessage,9110) trim(anum1),trim(modfile)
9110	format(' Error reading header to array number ',a,&
	' from model-generated file ',a)
	go to 9890
9150    call num2char(iline,anum1)
        write(amessage,9160) trim(anum1),trim(sampfile)
9160    format(' Error on line ',a,' of bore sample file ',a,': ',&
        'insufficient entries.')
        go to 9890
9200	call num2char(iarray,anum1)
	write(amessage,9210) trim(anum1),trim(modfile)
9210	format(' Error reading array number ',a,' from model ',&
	'output file ',a)
	go to 9890
9250	call num2char(iarray,anum1)
	write(amessage,9260) trim(anum1),trim(modfile)
9260	format(' Unexpected end of file encountered while reading array ',&
	'number ',a,' from model output file ',a)
	go to 9890
9300	write(amessage,9310)
9310	format(' Memory management error: cannot continue execution.')
	go to 9890
9400	write(amessage,9410) trim(outfile)
9410	format(' Error writing data to bore sample file ',a,&
	': file inaccessible or disk full.')
	go to 9890
9500    call num2char(iline,anum1)
        write(amessage,9510) trim(anum1),trim(sampfile)
9510    format(' Error on line ',a,' of bore sample file ',a,': bore ',&
        'identifier greater than 10 characters in length.')
        go to 9890
9600    call num2char(iline,anum1)
        write(amessage,9610) trim(anum1),trim(sampfile)
9610    format(' Error reading line ',a,' of bore sample file ',a)
        go to 9890


9890	call write_message(leadspace='yes')
9900	call close_files
	call free_bore_mem
	call free_grid_mem(gridspec)
	deallocate(rarray1,rarray2,east,north,fac1,fac2,fac3,fac4,icellno, &
	jcellno,layer,numsamp,index_bore,arrtime1,arrtime2, &
	bore_days,bore_secs,bore_val,iorder,stat=ierr)
	write(6,*)

end program mod2obs


subroutine free_bore_mem

! -- Subroutine free_bore_mem frees memory and deallocates pointers pertinent
!    to the storage of bore data.

! -- Revision history:-
!      June-November, 1995: version 1.

	use defn
	use inter

	integer                 :: ierr

	deallocate(bore_coord_id,bore_coord_east,bore_coord_north, &
	bore_list_id,bore_coord_layer,bore_diff_id,stat=ierr)
	nullify (bore_coord_id,bore_coord_east,bore_coord_north,bore_list_id,&
	bore_coord_layer,bore_diff_id)

end subroutine free_bore_mem
subroutine free_grid_mem(gridspec)

! -- Subroutine free_grid_mem frees memory and deallocates pointers pertinent
!    to the storage of grid data.

! -- Arguments are as follows:-
!       gridspec:  defined variable holding grid specifications

! -- Revision history:-
!       June-November, 1995: version1.

	use defn
	use inter

	type (modelgrid)        :: gridspec
	integer                 ::ierr

	deallocate(gridspec%delr,gridspec%delc,stat=ierr)
	nullify (gridspec%delr,gridspec%delc)

end subroutine free_grid_mem
subroutine close_files

! -- Subroutine close_files closes all open files.

! -- Revision history:-
!       June-November, 1995: version 1.

	integer         :: i,ierr

	do i=10,100
	  close(unit=i,iostat=ierr)
	end do
	return

end subroutine close_files
subroutine newdate(ndays,day1,mon1,year1,day2,mon2,year2)

! -- Subroutine NEWDATE evaluates the date after NDAYS days have elapsed from
!    a provided date. NDAYS may be negative.

! -- Arguments are as follows:-
!       ndays:            elapsed number of days
!       day1,mon1,year1:  days, month and year of first date
!       day2,mon2,year2:  days, month and year of second date

! -- Revision history:-
!       June-November, 1995: version 1.

	use inter
	implicit none

	integer, intent(in)     :: ndays,day1,mon1,year1
	integer, intent(out)    :: day2,mon2,year2

	integer  :: yearref,newdays,idays,iyear,jdays,i
	integer, dimension(12) :: monthdays

	data monthdays /31,28,31,30,31,30,31,31,30,31,30,31/

! -- First a reference date is chosen. This is the beginning of the first
! -- year. Alternatively the reference date is the beginning of a year prior
! -- to the likely calculated date if NDAYS is negative.

	if(ndays.ge.0) then
	  yearref=year1
	else
	  yearref=year1-abs(ndays)/365-1
	end if
	newdays=numdays(31,12,yearref-1,day1,mon1,year1)
	newdays=ndays+newdays
	if(newdays.lt.0) call sub_error('NEWDATE')

! -- Next days are counted, starting at the new reference date.

	idays=0
	iyear=yearref
	do
	  jdays=idays+365
	  if(leap(iyear)) jdays=jdays+1
	  if(jdays.ge.newdays) go to 20
	  iyear=iyear+1
	  idays=jdays
	end do
	call sub_error('NEWDATE')
20      year2=iyear

	do i=1,12
	  jdays=idays+monthdays(i)
	  if((i.eq.2).and.(leap(year2))) jdays=jdays+1
	  if(jdays.ge.newdays) go to 40
	  idays=jdays
	end do
	call sub_error('NEWDATE')
40      mon2=i
	day2=newdays-idays
	if((day2.le.0).or.(mon2.le.0).or.(year2.le.0)) call sub_error('NEWDATE')

	return

end subroutine newdate  
!     Last change:  JD   17 Dec 2000    0:04 am
subroutine point_interp(ncol,nrow,thresh,&
	fac1,fac2,fac3,fac4,icellno,jcellno,bhead,rarray,imethod)

! -- Subroutine INTERP multiplies the heads at grid cell centres by the
!    factors determined in subroutine FACTOR to carry out the interpolation
!    to bore locations. If a bore is near the edge of the grid, or borders
!    one or more dry or inactive cells, dummy heads are assigned to the
!    cell centres or pseudo cell centres surrounding the bore.

! -- Arguments are as follows:-
!       ncol, nrow:     number of columns and rows in finite-difference grid
!       thresh:         abs. value threshold above which cells are inactive
!       fac1,fac2,fac3,fac4:  interpolation factors from grid to point
!       icellno:        index of grid cell containing point
!       jcellno:        index of cell centre to top right of point
!       bhead:          interpolated value at point
!       rarray:         array from which interpolation takes place
!       imethod:        denotes whether to interpolate past last row/column
!                       of active cell centres


	integer, intent(in)                     :: ncol,nrow
	real, intent(in)                        :: thresh
	real, intent(in)                        :: fac1,fac2,fac3,fac4
	integer, intent(in)                     :: icellno,jcellno
	real, intent(out)                       :: bhead
	real, dimension(0:ncol+1,0:nrow+1),intent(in)  :: rarray
	character (len=*), intent(in), optional :: imethod

	integer :: i,j,irow,icol,jrow,jcol,k
	integer :: iact(2,2)
	real :: head(2,2)

	if(icellno.eq.-999) then
	  bhead=7.1e37
	  return
	end if
	irow=(icellno-1)/ncol+1
	icol=icellno-(irow-1)*ncol
	jrow=(jcellno-1)/(ncol+1)
	jcol=jcellno-jrow*(ncol+1)-1
	if(abs(rarray(icol,irow)).gt.thresh) then
	  bhead=5.1e37
	  return
	end if

	head(1,1)=rarray(jcol,jrow)
	head(1,2)=rarray(jcol+1,jrow)
	head(2,1)=rarray(jcol,jrow+1)
	head(2,2)=rarray(jcol+1,jrow+1)
	do k=1,2
	do j=1,2
	iact(j,k)=1
	end do
	end do
	if(abs(head(1,1)).gt.thresh) iact(1,1)=0
	if(abs(head(1,2)).gt.thresh) iact(1,2)=0
	if(abs(head(2,1)).gt.thresh) iact(2,1)=0
	if(abs(head(2,2)).gt.thresh) iact(2,2)=0
	if(imethod.eq.'inside')then
	  do k=1,2
	  do j=1,2
	    if(iact(j,k).ne.1) then
	      bhead=4.1e37
	      return
	    end if
	  end do
	  end do
	end if

	if(iact(1,1).eq.0)then
	  if(iact(1,2).eq.0)then
	    if(iact(2,1).eq.0)then
	      head(1,1)=head(2,2)
	      head(1,2)=head(2,2)
	      head(2,1)=head(2,2)
	    else if(iact(2,2).eq.0)then
	      head(1,1)=head(2,1)
	      head(1,2)=head(2,1)
	      head(2,2)=head(2,1)
	    else
	      head(1,1)=head(2,1)
	      head(1,2)=head(2,2)
	    end if
	  else if(iact(2,1).eq.0)then
	    if(iact(2,2).eq.0)then
	      head(1,1)=head(1,2)
	      head(2,1)=head(1,2)
	      head(2,2)=head(1,2)
	    else
	      head(1,1)=head(1,2)
	      head(2,1)=head(2,2)
	    end if
	  else if(iact(2,2).eq.0)then
	    if(icol.eq.jcol+1)then
	      head(1,1)=head(1,2)
	      head(2,1)=head(1,2)
	      head(2,2)=head(1,2)
	    else
	      head(1,1)=head(2,1)
	      head(1,2)=head(2,1)
	      head(2,2)=head(2,1)
	   end if
	  else
	    if(fac2+fac3.le.1.0e-6)then
	      head(1,1)=(head(1,2)+head(2,1))/2.0
	    else
	      head(1,1)=(head(1,2)*fac2+head(2,1)*fac3)/(fac2+fac3)
	    end if
	  end if
	else if(iact(1,2).eq.0)then
	  if(iact(2,2).eq.0)then
	    if(iact(2,1).eq.0)then
	      head(1,2)=head(1,1)
	      head(2,1)=head(1,1)
	      head(2,2)=head(1,1)
	    else
	      head(1,2)=head(1,1)
	      head(2,2)=head(2,1)
	    end if
	  else if(iact(2,1).eq.0)then
	    if(icol.eq.jcol)then
	      head(1,2)=head(1,1)
	      head(2,1)=head(1,1)
	      head(2,2)=head(1,1)
	    else
	      head(1,1)=head(2,2)
	      head(1,2)=head(2,2)
	      head(2,1)=head(2,2)
	    end if
	  else
	    if(fac1+fac4.le.1.0e-6)then
	      head(1,2)=(head(1,1)+head(2,2))/2.0
	    else
	      head(1,2)=(head(1,1)*fac1+head(2,2)*fac4)/(fac1+fac4)
	    end if
	  end if
	else if(iact(2,1).eq.0)then
	  if(iact(2,2).eq.0)then
	    head(2,1)=head(1,1)
	    head(2,2)=head(1,2)
	  else
	    if(fac1+fac4.le.1.0e-6)then
	      head(2,1)=(head(1,1)+head(2,2))/2.0
	    else
	      head(2,1)=(head(1,1)*fac1+head(2,2)*fac4)/(fac1+fac4)
	    end if
	  end if
	else if(iact(2,2).eq.0)then
	  if(fac2+fac3.le.1.0e-6)then
	    head(2,2)=(head(1,2)+head(2,1))/2.0
	  else
	    head(2,2)=(head(1,2)*fac2+head(2,1)*fac3)/(fac2+fac3)
	  end if
	end if

	bhead=head(1,1)*fac1+head(1,2)*fac2+head(2,1)*fac3+ &
	head(2,2)*fac4

	return
end subroutine point_interp
!*****************************************************************************
! subroutines comprising the generic subroutine NUM2CHAR ------->
!*****************************************************************************

! -- Subroutine num2char writes the character equivalent of a number.

! -- Arguments are as follows:-
!       value:   the number to be expressed in character form
!       string:  the number expressed in character form
!       nchar:   the maximum number of characters in which to express number

! -- Revision history:-
!       June-November, 1995: version 1.

subroutine i2a(value,string,nchar)

	use inter

	integer, intent(in)             :: value
	character (len=*), intent(out)  :: string
	integer, intent(in), optional   :: nchar
	character (len=12)              :: afmt
	integer                         :: llen

	string=' '
	afmt='(i    )'
	llen=min(30,len(string))
	if(present(nchar)) llen=min(llen,nchar)
	write(afmt(3:6),'(i4)') llen
	write(string(1:llen),afmt,err=100) value
	string=adjustl(string)
	if(string(1:1).eq.'*') go to 100
	return

100     string(1:llen)=repeat('#',llen)
	return

end subroutine i2a


subroutine d2a(value,string,nchar)

	use inter

	double precision, intent(in)    :: value
	character (len=*), intent(out)  :: string
	integer, intent(in), optional   :: nchar
	integer                         :: llen, ifail
	double precision                :: value_check
	character (len=32)              :: word

	string=' '
	llen=min(29,len(string))
	if(present(nchar)) llen=min(llen,nchar)
	call wrtsig(ifail,value,word,llen,1,value_check,0)
	if(ifail.lt.0) then
	  call sub_error('D2A')
	else if(ifail.gt.0) then
	  string(1:llen)=repeat('#',llen)
	else
	  string=adjustl(word)
	end if
	return

end subroutine d2a


subroutine r2a(value,string,nchar)

	use inter

	real,intent(in)                 :: value
	character (len=*), intent(out)  :: string
	integer, intent(in), optional   :: nchar
	integer                         :: llen,ifail
	double precision                :: dvalue,dvalue_check
	character (len=32)              :: word

	string=' '
	llen=min(29,len(string))
	if(present(nchar)) llen=min(llen,nchar)
	dvalue=value
	call wrtsig(ifail,dvalue,word,llen,0,dvalue_check,0)
	if(ifail.lt.0) then
	  call sub_error('R2A')
	else if(ifail.gt.0) then
	  string(1:llen)=repeat('#',llen)
	else
	  string=adjustl(word)
	end if
	return

end subroutine r2a

	
	SUBROUTINE WRTSIG(IFAIL,VAL,WORD,NW,PRECIS,TVAL,NOPNT)
! --
! -- SUBROUTINE WRTSIG WRITES A NUMBER INTO A CONFINED SPACE WITH MAXIMUM
! -- PRECISION
! --

! -- Revision history:-
!       July, 1993: version 1.
!       August 1994: modified for unix version (#ifdef's added)
!       August, 1995: #ifdefs commented out for inclusion in Groundwater
!                     Data Utilities

!       failure criteria:
!           ifail= 1 ...... number too large or small for single precision type
!           ifail= 2 ...... number too large or small for double precision type
!           ifail= 3 ...... field width too small to represent number
!           ifail=-1 ...... internal error type 1
!           ifail=-2 ...... internal error type 2
!           ifail=-3 ...... internal error type 3

	INTEGER PRECIS,LW,POS,INC,D,P,W,J,JJ,K,JEXP,N,JFAIL,NW, &
	EPOS,PP,NOPNT,KEXP,IFLAG,LEXP
	INTEGER IFAIL
	DOUBLE PRECISION VAL,TVAL
	CHARACTER*29 TWORD,TTWORD,FMT*14
	CHARACTER*(*) WORD

	LEXP=0
	IFLAG=0
	WORD=' '
	POS=1
	IF(VAL.LT.0.0D0)POS=0
!#ifdef USE_D_FORMAT
!        WRITE(TWORD,'(1PD23.15D3)') VAL
!#else
	WRITE(TWORD,'(1PE23.15E3)') VAL
!#endif
	READ(TWORD(20:23),'(I4)') JEXP
	EPOS=1
	IF(JEXP.LT.0)EPOS=0

	JFAIL=0
	IFAIL=0
	IF(PRECIS.EQ.0)THEN
	  LW=MIN(15,NW)
	ELSE
	  LW=MIN(23,NW)
	END IF

	N=0
	IF(NOPNT.EQ.1)N=N+1
	IF(POS.EQ.1)N=N+1
	IF(PRECIS.EQ.0)THEN
	  IF(ABS(JEXP).GT.38)THEN
	    IFAIL=1
	    RETURN
	  END IF
	  IF(POS.EQ.1) THEN
	    IF(LW.GE.13) THEN
	      WRITE(WORD,'(1PE13.7)',ERR=80) VAL
	      GO TO 200
	    END IF
	  ELSE
	    IF(LW.GE.14)THEN
	      WRITE(WORD,'(1PE14.7)',ERR=80) VAL
	      GO TO 200
	    END IF
	  END IF
	  IF(LW.GE.14-N) THEN
	    LW=14-N
	    GO TO 80
	  END IF
	ELSE
	  IF(ABS(JEXP).GT.275)THEN
	    IFAIL=2
	    RETURN
	  END IF
	  IF(POS.EQ.1) THEN
	    IF(LW.GE.22) THEN
!#ifdef USE_D_FORMAT
!              WRITE(WORD,'(1PD22.15D3)',ERR=80) VAL
!#else
	      WRITE(WORD,'(1PE22.15E3)',ERR=80) VAL
!#endif
	      GO TO 200
	    END IF
	  ELSE
	    IF(LW.GE.23) THEN
!#ifdef USE_D_FORMAT
!              WRITE(WORD,'(1PD23.15D3)',ERR=80) VAL
!#else
	      WRITE(WORD,'(1PE23.15E3)',ERR=80) VAL
!#endif
	      GO TO 200
	    END IF
	  END IF
	  IF(LW.GE.23-N)THEN
	    LW=23-N
	    GO TO 80
	  END IF
	END IF

	IF(NOPNT.EQ.1)THEN
	  IF((JEXP.EQ.LW-2+POS).OR.(JEXP.EQ.LW-3+POS))THEN
	    WRITE(FMT,15)LW+1
15          FORMAT('(F',I2,'.0)')
	    WRITE(WORD,FMT,ERR=19) VAL
	    IF(INDEX(WORD,'*').NE.0) GO TO 19
	    IF(WORD(1:1).EQ.' ') GO TO 19
	    WORD(LW+1:LW+1)=' '
	    GO TO 200
	  END IF
	END IF
19      D=MIN(LW-2+POS,LW-JEXP-3+POS)
20      IF(D.LT.0) GO TO 80
	WRITE(FMT,30) LW,D
30      FORMAT('(F',I2,'.',I2,')')
	WRITE(WORD,FMT,ERR=80) VAL
	IF(INDEX(WORD,'*').NE.0) THEN
	  D=D-1
	  GO TO 20
	END IF
	K=INDEX(WORD,'.')
	IF(K.EQ.0)THEN
	  IFAIL=-1
	  RETURN
	END IF
	IF((K.EQ.1).OR.((POS.EQ.0).AND.(K.EQ.2)))THEN
	  DO 70 J=1,3
	  IF(K+J.GT.LW) GO TO 75
	  IF(WORD(K+J:K+J).NE.'0') GO TO 200
70        CONTINUE
	  GO TO 80
75        IFAIL=3
	  RETURN
	END IF
	GO TO 200

80      WORD=' '
	IF(NOPNT.EQ.0)THEN
	  D=LW-7
	  IF(POS.EQ.1) D=D+1
	  IF(EPOS.EQ.1) D=D+1
	  IF(ABS(JEXP).LT.100) D=D+1
	  IF(ABS(JEXP).LT.10) D=D+1
	  IF((JEXP.GE.100).AND.(JEXP-(D-1).LT.100))THEN
	    P=1+(JEXP-99)
	    D=D+1
	    LEXP=99
	  ELSE IF((JEXP.GE.10).AND.(JEXP-(D-1).LT.10))THEN
	    P=1+(JEXP-9)
	    D=D+1
	    LEXP=9
	  ELSE IF((JEXP.EQ.-10).OR.(JEXP.EQ.-100)) THEN
	    IFLAG=1
	    D=D+1
	  ELSE
	    P=1
	  END IF
	  INC=0
85        IF(D.LE.0) GO TO 300
	  IF(IFLAG.EQ.0)THEN
	    WRITE(FMT,100,ERR=300) P,D+7,D-1
	  ELSE
	    WRITE(FMT,100,ERR=300) 0,D+8,D
	  END IF
	  WRITE(TWORD,FMT) VAL
	  IF(IFLAG.EQ.1) GO TO 87
	  READ(TWORD(D+4:D+7),'(I4)',ERR=500) KEXP
	  IF(((KEXP.EQ.10).AND.((JEXP.EQ.9).OR.(LEXP.EQ.9))).OR. &
	  ((KEXP.EQ.100).AND.((JEXP.EQ.99).OR.LEXP.EQ.99))) THEN
	    IF(INC.EQ.0)THEN
	      IF(LEXP.EQ.0)THEN
		IF(D-1.EQ.0) THEN
		  D=D-1
		ELSE
		  P=P+1
		END IF
	      ELSE IF(LEXP.EQ.9)THEN
		IF(JEXP-(D-2).LT.10) THEN
		  P=P+1
		ELSE
		  D=D-1
		END IF
	      ELSE IF(LEXP.EQ.99)THEN
		IF(JEXP-(D-2).LT.100)THEN
		  P=P+1
		ELSE
		  D=D-1
		END IF
	      END IF
	      INC=INC+1
	      GO TO 85
	    END IF
	  END IF
!#ifdef USE_D_FORMAT
!87        J=INDEX(TWORD,'D')
!#else
87        J=INDEX(TWORD,'E')
!#endif
	  GO TO 151
	END IF
	INC=0
	P=LW-2
	PP=JEXP-(P-1)
	IF(PP.GE.10)THEN
	  P=P-1
	  IF(PP.GE.100)P=P-1
	ELSE IF(PP.LT.0)THEN
	  P=P-1
	  IF(PP.LE.-10)THEN
	    P=P-1
	    IF(PP.LE.-100)P=P-1
	  END IF
	END IF
	IF(POS.EQ.0)P=P-1
90      CONTINUE
	D=P-1
	W=D+8
	WRITE(FMT,100) P,W,D
	IF(D.LT.0)THEN
	  IF(JFAIL.EQ.1) GO TO 300
	  JFAIL=1
	  P=P+1
	  GO TO 90
	END IF
!#ifdef USE_D_FORMAT
!100     FORMAT('(',I2,'pD',I2,'.',I2,'D3)')
!#else
100     FORMAT('(',I2,'pE',I2,'.',I2,'E3)')
!#endif
	WRITE(TWORD,FMT) VAL
!#ifdef USE_D_FORMAT
!        J=INDEX(TWORD,'D')
!#else
	J=INDEX(TWORD,'E')
!#endif
	IF(TWORD(J-1:J-1).NE.'.')THEN
	  IFAIL=-1
	  RETURN
	END IF
	N=1
	IF(TWORD(J+1:J+1).EQ.'-') N=N+1
	IF(TWORD(J+2:J+2).NE.'0') THEN
	  N=N+2
	  GO TO 120
	END IF
	IF(TWORD(J+3:J+3).NE.'0') N=N+1
120     N=N+1
	IF(J+N-2-POS.LT.LW)THEN
	  IF(INC.EQ.-1) GO TO 150
	  TTWORD=TWORD
	  P=P+1
	  INC=1
	  GO TO 90
	ELSE IF(J+N-2-POS.EQ.LW) THEN
	  GO TO 150
	ELSE
	  IF(INC.EQ.1)THEN
	    TWORD=TTWORD
	    GO TO 150
	  END IF
	  IF(JFAIL.EQ.1) GO TO 300
	  P=P-1
	  INC=-1
	  GO TO 90
	END IF

150     J=INDEX(TWORD,'.')
151     IF(POS.EQ.0)THEN
	  K=1
	ELSE
	 K=2
	END IF
	WORD(1:J-K)=TWORD(K:J-1)
	JJ=J
	J=J-K+1
	IF(PRECIS.EQ.0)THEN
	  WORD(J:J)='E'
	ELSE
	  WORD(J:J)='D'
	END IF
	JJ=JJ+2
	IF(NOPNT.EQ.0) JJ=JJ-1
	IF(TWORD(JJ:JJ).EQ.'-')THEN
	  J=J+1
	  WORD(J:J)='-'
	END IF
	IF(TWORD(JJ+1:JJ+1).NE.'0')THEN
	  J=J+2
	  WORD(J-1:J)=TWORD(JJ+1:JJ+2)
	  GO TO 180
	END IF
	IF(TWORD(JJ+2:JJ+2).NE.'0')THEN
	  J=J+1
	  WORD(J:J)=TWORD(JJ+2:JJ+2)
	END IF
180     J=J+1
	WORD(J:J)=TWORD(JJ+3:JJ+3)
	IF(IFLAG.EQ.1)THEN
	  IF(POS.EQ.1)THEN
	    JJ=1
	  ELSE
	    JJ=2
	  END IF
	  N=len_trim(WORD)
	  DO 190 J=JJ,N-1
190       WORD(J:J)=WORD(J+1:J+1)
	  WORD(N:N)=' '
	END IF

200     IF(len_trim(WORD).GT.LW)THEN
	  IFAIL=-2
	  RETURN
	END IF
	WRITE(FMT,30) LW,0
	READ(WORD,FMT,ERR=400) TVAL
	RETURN
300     IFAIL=3
	RETURN
400     IFAIL=-3
	RETURN
500     IFAIL=-2
	RETURN
	END
!*****************************************************************************
! subprograms for reading and parsing data (mainly from files) ------->
!*****************************************************************************

subroutine linesplit(ifail,num)

! -- subroutine linesplit splits a line into whitespace-delimited words

! -- Arguments are as follows:-
!       ifail:   returned as -1 if line is blank
!                returned as  1 if less than num segments
!       num:     number of words to be extracted

!    Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	integer, intent(out)            :: ifail
	integer, intent(in)             :: num
	integer                         :: nblc,j,i,nw
	character (len=3)               :: aspace

	ifail=0; nw=0; j=0
	aspace=' ,'//achar(9)   
	if(num.gt.NUM_WORD_DIM) call sub_error('LINESPLIT')
	nblc=len_trim(cline)
	if(nblc.eq.0) then
	  ifail=-1
	  return
	end if

5       if(nw.eq.num) return
	do i=j+1,nblc
	  if(index(aspace,cline(i:i)).eq.0) go to 20
	end do
	ifail=1
	return
20      nw=nw+1
	left_word(nw)=i
	do i=left_word(nw)+1,nblc
	  if(index(aspace,cline(i:i)).ne.0) go to 40
	end do
	right_word(nw)=nblc
	if(nw.lt.num) ifail=1
	return
40      right_word(nw)=i-1
	j=right_word(nw)
	go to 5

end subroutine linesplit


integer function char2int(ifail,num)

! -- Function char2int extracts an integer from a word demarcated by subroutine
!    linesplit.

! -- Arguments are as follows:-
!       ifail:    returned as zero unless an error condition arises
!       num:      the number of the word previously extracted by linesplit
!       returns   value of integer read from word

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn

	integer, intent(in)             :: num
	integer, intent(out)            :: ifail
	character (len=8)               :: afmt

	if(num.gt.NUM_WORD_DIM) call sub_error('CHAR2INT')
	if((right_word(num).lt.left_word(num)).or. &
	  (left_word(num).le.0)) call sub_error('CHAR2INT')

	ifail=0
	afmt='(i   )'
	write(afmt(3:5),'(i3)') right_word(num)-left_word(num)+1
	read(cline(left_word(num):right_word(num)),afmt,err=100) char2int
	return

100     ifail=1
	return

end function char2int


real function char2real(ifail,num)

! -- Function char2real extracts a real number from a word demarcated by
!    subroutine linesplit.

! -- Arguments are as follows:-
!       ifail:    returned as zero unless an error condition arises
!       num:      the number of the word previously extracted by linesplit
!       returns   value of real number read from word

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn

	integer, intent(in)             :: num
	integer, intent(out)            :: ifail
	integer                         :: ierr
	character (len=10)              :: afmt

	if(num.gt.NUM_WORD_DIM) call sub_error('CHAR2REAL')
	if((right_word(num).lt.left_word(num)).or. &
	  (left_word(num).le.0)) call sub_error('CHAR2REAL')

	ifail=0
	afmt='(f   .0)'
	write(afmt(3:5),'(i3)') right_word(num)-left_word(num)+1
	read(cline(left_word(num):right_word(num)),afmt, iostat=ierr) char2real
	if(ierr.ne.0) go to 110
	return

110     ifail=1
	return

end function char2real


double precision function char2double(ifail,num)

! -- Function char2double extracts a double precision number from a word
!    demarcated by subroutine linesplit.

! -- Arguments are as follows:-
!       ifail:    returned as zero unless an error condition arises
!       num:      the number of the word previously extracted by linesplit
!       returns   value of double precision number read from word

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn

	integer, intent(in)             :: num
	integer, intent(out)            :: ifail
	integer                         :: ierr
	character (len=10)              :: afmt

	if(num.gt.NUM_WORD_DIM) call sub_error('CHAR2DOUBLE')
	if((right_word(num).lt.left_word(num)).or. &
	  (left_word(num).le.0)) call sub_error('CHAR2DOUBLE')

	ifail=0
	afmt='(f   .0)'
	write(afmt(3:5),'(i3)') right_word(num)-left_word(num)+1
	read(cline(left_word(num):right_word(num)),afmt, iostat=ierr) char2double
	if(ierr.ne.0) go to 110
	return

110     ifail=1
	return

end function char2double
!     Last change:  JD   23 Dec 2000    8:30 pm
subroutine open_output_file(ifail,aprompt,outfile,outunit,file_format)

! -- Subroutine open_output_file opens a file for data output.

! -- Subroutine arguments are as follows:-
!       ifail:    returned as non-zero in case of failure
!       aprompt:  user prompt for filename
!       outfile:  name of output file
!       outunit:  unit number of output file
!       file_format:  supplied as either "formatted" or "unformatted" to open
!                     formatted or unformatted file

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	integer, intent(out)                       :: ifail
	character (len=*)                          :: aprompt,outfile
	integer, intent(out)                       :: outunit
	character (len=*), intent(in), optional    :: file_format
	integer                                    :: ierr,nbb,ifail1
	logical                                    :: lexist,lopened
	character (len=1)                          :: aa,aformat
        character (len=200)                        :: atempf
        character (len=20)                         :: bin_unform

! -- The correct unformatted word is read.

        include 'bin_unform.inc'

! -- The user is prompted for the name of the output file.

        aformat='f'
        if(present(file_format)) then
	  if(file_format.eq.'formatted') then
	    aformat='f'
	  else if(file_format.eq.'unformatted')then
	    aformat='u'
	  else
	    call sub_error('OPEN_OUTPUT_FILE')
	  end if
	end if

	imessage=0
	ifail=0
10      write(6,'(a)',advance='no') aprompt(1:len_trim(aprompt)+1)
	read(5,'(a)')outfile
	if(outfile.eq.' ') go to 10
	outfile=adjustl(outfile)
	if(index(eschar,outfile(1:2)).ne.0) then
	  escset=1
	  return
	end if
        nbb=len_trim(outfile)
        call getfile(ifail1,outfile,atempf,1,nbb)
        if(ifail1.ne.0) go to 10
        outfile=atempf
	inquire(file=outfile,opened=lopened)
	if(lopened)then
	  write(amessage,30) trim(outfile)
30        format(' File ',a,' is already open  - try again.')
	  call write_message(increment=1)
	  go to 10
	end if
	inquire(file=outfile,exist=lexist)
!       if(lexist) then
!40        write(6,50,advance='no')
!50        format(' File already exists: overwrite it?  [y/n] ')
!         read(5,'(a)') aa
!         call casetrans(aa,'lo')
!         if((aa.eq.'e').or.(aa.eq.'n'))then
!           write(6,*)
!           go to 10
!         end if
!         if(aa.ne.'y') go to 40
!       end if

! -- The file is opened.

	outunit=nextunit()
        if(aformat.eq.'f')then
          open(unit=outunit,file=outfile,status='new',iostat=ierr)
          if(ierr.ne.0) then
            open(unit=outunit,file=outfile,status='replace',iostat=ierr)
          end if
        else
          open(unit=outunit,file=outfile,status='new',form=bin_unform,iostat=ierr)
          if(ierr.ne.0) then
            open(unit=outunit,file=outfile,status='replace',form=bin_unform,iostat=ierr)
          end if
        end if
	if(ierr.ne.0) then
	  if(imessage.gt.5) then
	    ifail=1
	    return
	  end if
	  write(amessage,100) trim(outfile)
100       format(' Unable to open file ',a,' - try again.')
	  call write_message(increment=1)
	  go to 10
	end if

	return

end subroutine open_output_file

!*****************************************************************************
! subroutines comprising the generic subroutine CHAR2NUM ------->
!*****************************************************************************


! -- The subroutines comprising char2num convert a string to either an integer,
!    a real number, or a double precision number.

! -- Arguments are as follows:-
!      ifail:   indicates failure if returned as non-zero
!      string:  a character string containing a number
!      num:     an integer (for a2i), real (for a2r), or double precision (for
!               a2d) number extracted from the string.

! -- Revision history:-
!       June-November, 1995: version 1.


subroutine a2i(ifail,string,num)

	integer, intent(out)            :: ifail
	character (len=*), intent(in)   :: string
	integer, intent(out)            :: num
	character (len=10)              :: afmt

	ifail=0
	afmt='(i    )'
	write(afmt(3:6),'(i4)')len(string)
	read(string,afmt,err=10) num
	return

10      ifail=1
	return

end subroutine a2i


subroutine a2r(ifail,string,num)

	integer, intent(out)            :: ifail
	character (len=*), intent(in)   :: string
	real, intent(out)               :: num
	character (len=10)              :: afmt

	ifail=0
	afmt='(f    .0)'
	write(afmt(3:6),'(i4)')len(string)
	read(string,afmt,err=10) num
	return

10      ifail=1
	return

end subroutine a2r


subroutine a2d(ifail,string,num)

	integer, intent(out)            :: ifail
	character (len=*), intent(in)   :: string
	double precision, intent(out)   :: num
	character (len=10)              :: afmt

	ifail=0
	afmt='(f    .0)'
	write(afmt(3:6),'(i4)')len(string)
	read(string,afmt,err=10) num
	return

10      ifail=1
	return

end subroutine a2d

integer function numsecs(h1,m1,s1,h2,m2,s2)

! -- Subroutine NUMSECS calculates the number of seconds between two times.

! -- Arguments are as follows:-
!       h1,m1,s1:   hours, minutes seconds of first time
!       h2,m2,y2:   hours, minutes seconds of second time

! -- Revision history:-
!       June-November 1995: version 1.

	integer, intent(in)             :: h1,m1,s1,h2,m2,s2

	numsecs=(h2-h1)*3600+(m2-m1)*60+s2-s1

end function numsecs
integer function numdays(DR,MR,YR,D,M,Y)

! -- Function numdays calculates the number of days between dates
!    D-M-Y and DR-MR-YR. If the former preceeds the latter the answer is
!    negative.

! -- Arguments are as follows:-
!       dr,mr,yr:     days, months and years of first date
!       d,m,y:        days, months and years of second date
!       numdays returns the number of elapsed days

! -- Revision history:-
!       22 July 1994:  version 1
!       13 September 1995:  modified for Groundwater Data Utilities


	integer, intent(in)     :: dr,mr,yr,d,m,y

	INTEGER FLAG,I,J,DA(12),YE,ME,DE,YL,ML,DL
	logical leap

	DATA DA /31,28,31,30,31,30,31,31,30,31,30,31/

! --    THE SMALLER OF THE TWO DATES IS NOW CHOSEN TO DO THE COUNTING FROM.

	IF(Y.LT.YR)GO TO 10
	IF((Y.EQ.YR).AND.(M.LT.MR)) GO TO 10
	IF((Y.EQ.YR).AND.(M.EQ.MR).AND.(D.LT.DR)) GO TO 10
	FLAG=0
	YE=YR
	ME=MR
	DE=DR
	YL=Y
	ML=M
	DL=D
	GO TO 20
10      FLAG=1
	YE=Y
	ME=M
	DE=D
	YL=YR
	ML=MR
	DL=DR

! --    IN THE ABOVE THE POSTSCRIPT "E" STANDS FOR EARLIER DATE, WHILE
!       "L" STANDS FOR THE LATER DATE.

20      numdays=0
	IF((ME.EQ.ML).AND.(YL.EQ.YE))THEN
	numdays=DL-DE
	IF(FLAG.EQ.1) numdays=-numdays
	RETURN
	END IF

	DO 30 J=ME,12
	IF((ML.EQ.J).AND.(YE.EQ.YL))GOTO 40
	numdays=numdays+DA(J)
	IF((J.EQ.2).AND.(leap(ye)))numdays=numdays+1
30      CONTINUE
	GO TO 50
40      numdays=numdays+DL-DE
	IF(FLAG.EQ.1)numdays=-numdays
	RETURN

50      DO 60 I=YE+1,YL
	DO 70 J=1,12
	IF((YL.EQ.I).AND.(ML.EQ.J))GO TO 80
	numdays=numdays+DA(J)
	IF((J.EQ.2).AND.(leap(i))) numdays=numdays+1
70      CONTINUE
60      CONTINUE
	call sub_error('NUMDAYS')
	RETURN

80      numdays=numdays+DL-DE
	IF(FLAG.EQ.1) numdays=-numdays

	RETURN
end function numdays
subroutine char2time(ifail,atime,hh,mm,ss)

! -- Subroutine CHAR2TIME extracts the time from a string.

! -- Arguments are as follows:-
!       ifail:     indicates failure if returned as non-zero
!       atime:     a string containing the time in ASCII format
!       hh,mm,ss   hours, minutes and seconds extracted from the atime string.

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	integer, intent(out)            :: ifail
	character (len=*), intent(in)   :: atime
	integer, intent(out)            :: hh,mm,ss
	integer                         :: lentime,i,j
	character (len=2)               :: asep
	character (len=20)              :: btime

	ifail=0
	asep=':.'
	if(atime.eq.' ') go to 9000
	btime=adjustl(atime)
	lentime=len_trim(btime)
	if(lentime.lt.5) go to 9000

	do i=1,lentime
	  if(index(asep,btime(i:i)).ne.0) go to 20
	end do
	go to 9000

! -- The first integer is extracted from the string. This represents hours.

20      if(i.eq.1) go to 9000
	call char2num(ifail,btime(1:i-1),hh)
	if(ifail.ne.0) go to 9000
	if((hh.lt.0).or.(hh.gt.23)) go to 9000

	i=i+1
	if(lentime-i.lt.2) go to 9000
	do j=i,lentime
	  if(index(asep,btime(j:j)).ne.0) go to 40
	end do
	go to 9000

! -- The second integer (representing minutes) is extracted from the string.

40      if(j.eq.i) go to 9000
	call char2num(ifail,btime(i:j-1),mm)
	if(ifail.ne.0) go to 9000
	if((mm.lt.0).or.(mm.gt.59)) go to 9000

! -- The third integer (representing seconds) is extracted from the string.

	j=j+1
	if(lentime-j.lt.0) go to 9000
	call char2num(ifail,btime(j:lentime),ss)
	if(ifail.ne.0) go to 9000
	if((ss.lt.0).or.(ss.gt.59)) go to 9000
	ifail=0
	return

9000    ifail=1
	return

end subroutine char2time
subroutine char2date(ifail,adate,dd,mm,yy)

! -- Subroutine CHAR2DATE extracts the date from a string.


! -- Arguments are as follows:-
!      ifail:      returns a non-zero value if an error condition is encountered
!      adate:      the string containing the date
!      dd,mm,yy    the day, month and year read from the date string

! --  Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	integer, intent(out)    :: ifail
	character (len=*), intent(in)   :: adate
	integer, intent(out) :: dd,mm,yy
	integer :: lendate,i,j
	character (len=2)       :: asep
	character (len=20)      :: bdate

	ifail=0
	asep=':/'
	if(adate.eq.' ') go to 9000
	bdate=adjustl(adate)
	lendate=len_trim(bdate)
	if(lendate.lt.8) go to 9000

	do i=1,lendate
	  if(index(asep,bdate(i:i)).ne.0) go to 20
	end do
	go to 9000

! -- The first integer is extracted from the date string. This is either days
!    or months depending on the contents of file settings.fig.

20      if(i.eq.1) go to 9000
	if(datespec.ne.1) then
	   call char2num(ifail,bdate(1:i-1),mm)
	else
	   call char2num(ifail,bdate(1:i-1),dd)
	end if
	if(ifail.ne.0) go to 9000

	i=i+1
	if(lendate-i.lt.5) go to 9000
	do j=i,lendate
	  if(index(asep,bdate(j:j)).ne.0) go to 40
	end do
	go to 9000

! -- The second integer is extracted from the date string. This is either months
!    or days depending on the contents of file settings.fig.

40      if(j.eq.i) go to 9000
	if(datespec.ne.1) then
	  call char2num(ifail,bdate(i:j-1),dd)
	else
	  call char2num(ifail,bdate(i:j-1),mm)
	end if
	if(ifail.ne.0) go to 9000
	if((dd.le.0).or.(dd.gt.31)) go to 9000
	if((mm.le.0).or.(mm.gt.12)) go to 9000
	if(dd.eq.31)then
	  if((mm.eq.2).or.(mm.eq.4).or.(mm.eq.6).or.(mm.eq.9).or.&
	  (mm.eq.11)) go to 9000
	end if
	if((mm.eq.2).and.(dd.eq.30)) go to 9000

! -- The third integer is extracted from the date string. This is years.

	j=j+1
!	if(lendate-j.ne.3) go to 9000
	call char2num(ifail,bdate(j:lendate),yy)
	if(ifail.ne.0) go to 9000
	if(.not.leap(yy))then
	  if((mm.eq.2).and.(dd.eq.29)) go to 9000
	end if
	ifail=0
	return

9000    ifail=1
	return

end subroutine char2date
!*****************************************************************************
! functions comprising the generic function KEY_READ
!*****************************************************************************

! -- Function key_read reads and checks a number from the keyboard.

! -- Arguments are as follows:-
!       value:   value of the number read
!       returns  0 unless an error condition arises

! -- Revision history:-
!       June-November, 1995: version 1.

integer function int_key_read(value)

	use defn

	integer, intent(out)    :: value
	integer                 :: ifail
	character (len=50)      :: atemp

	int_key_read=0
	read(5,'(a)') atemp
	if(atemp.ne.' ') atemp=adjustl(atemp)   ! lf90 bug
	if(atemp.eq.' ') then
	  int_key_read=-1
	else if(index(eschar,atemp(1:2)).ne.0) then
	  escset=1
	else
	  call a2i(ifail,atemp,value)
	  if(ifail.ne.0) int_key_read=1
	end if
	return

end function int_key_read


integer function real_key_read(value)

	use defn

	real, intent(out)       :: value
	integer                 :: ifail
	character (len=50)      :: atemp

	real_key_read=0
	read(5,'(a)') atemp
	if(atemp.ne.' ') atemp=adjustl(atemp)   ! lf90 bug
	if(atemp.eq.' ') then
	  real_key_read=-1
	else if(index(eschar,atemp(1:2)).ne.0) then
	  escset=1
	else
	  call a2r(ifail,atemp,value)
	  if(ifail.ne.0) real_key_read=1
	end if
	return

end function real_key_read


integer function double_key_read(value)

	use defn

	double precision, intent(out)   :: value
	integer                         :: ifail
	character (len=50)              :: atemp

	double_key_read=0
	read(5,'(a)') atemp
	if(atemp.ne.' ') atemp=adjustl(atemp)   ! lf90 bug
	if(atemp.eq.' ') then
	  double_key_read=-1
	else if(index(eschar,atemp(1:2)).ne.0) then
	  escset=1
	else
	  call a2d(ifail,atemp,value)
	  if(ifail.ne.0) double_key_read=1
	end if
	return

end function double_key_read

subroutine casetrans(string,hi_or_lo)

! -- Subroutine casetrans converts a string to upper or lower case.

! -- Arguments are as follows:-
!      string:	  contains the string whose case must be changed
!      hi_or_lo:  must be either 'lo' or 'hi' to indicate
!                 change of case direction.

! -- Revision history:-
!       June-November, 1995: version 1.

	use inter

	character (len=*), intent(inout)        :: string
	character (len=*), intent(in)           :: hi_or_lo
	character                               :: alo, ahi
	integer                                 :: inc,i

	if(hi_or_lo.eq.'lo') then
	  alo='A'; ahi='Z'; inc=iachar('a')-iachar('A')
	else if(hi_or_lo.eq.'hi') then
	  alo='a'; ahi='z'; inc=iachar('A')-iachar('a')
	else
	  call sub_error('CASETRANS')
	endif

	do i=1,len_trim(string)
	  if((string(i:i).ge.alo).and.(string(i:i).le.ahi)) &
	  string(i:i)=achar(iachar(string(i:i))+inc)
	end do

	return

end subroutine casetrans
!     Last change:  JD   11 Apr 2003    8:38 am
subroutine open_input_file(ifail,aprompt,infile,inunit,file_format,form_prompt,fformat)

! -- Subroutine open_input_file opens an input file.

! -- Arguments are as follows:-
!       ifail:        returned as non-zero if failure
!       aprompt:      user prompt for filename
!       infile:       name of input file
!       inunit:       unit number to communicate with file
!       file_format:  supplied as either "formatted" or "unformatted" to open
!                     formatted or unformatted file
!       form_prompt:  supplied as either "yes" or "no" to prompt for file format
!       fformat:      returns file format to main program if form_prompt is supplied

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	integer, intent(out)                       :: ifail
	character (len=*), intent(in)              :: aprompt
	character (len=*), intent(out)             :: infile
	integer, intent(out)                       :: inunit
	character (len=*), intent(in), optional    :: file_format
        character (len=*), intent(in), optional    :: form_prompt
        character (len=*), intent(out),optional    :: fformat
	integer                                    :: ierr,nbb,ifail1
	character (len=1)                          :: aformat,aa
	character (len=10)                         :: aaformat
        character (len=120)                        :: bfile
	logical                                    :: lopened
        character (len=20)                         :: bin_unform

! -- The correct unformatted word is read.

        include 'bin_unform.inc'

! -- The user is prompted for the name of the file to open.

	imessage=0
	ifail=0
10      write(6,'(a)',advance='no') aprompt(1:len_trim(aprompt)+1)
	read(5,'(a)') infile
	if(infile.eq.' ') go to 10
	infile=adjustl(infile)
	if(index(eschar,infile(1:2)).ne.0) then
	  escset=1
	  return
	end if
        nbb=len_trim(infile)
        call getfile(ifail1,infile,bfile,1,nbb)
        if(ifail1.ne.0) go to 10
        infile=bfile


! -- Is the file already open?

	inquire(file=infile,opened=lopened)
	if(lopened)then
	  write(amessage,30) trim(infile)
30        format(' File ',a,' is already open  - try again.')
	  call write_message(increment=1)
	  go to 10
	end if

! -- The file is opened (after the format is established).

	aformat='f'
        if(present(form_prompt))then
          if(form_prompt.eq.'yes')then
            if(.not.(present(fformat))) call sub_error('OPEN_INPUT_FILE')
31          write(6,32,advance='no')
32          format(' Is this a formatted or unformatted file? [f/u]: ')
            read(5,'(a)') aa
            call casetrans(aa,'lo')
            if(aa.eq.'e') go to 10
            if(aa.eq.'f')then
              aformat='f'
              fformat='f'
            else if(aa.eq.'u')then
              aformat='u'
              fformat='u'
            else
              go to 31
            end if
          end if
        else if(present(file_format)) then
	  if(file_format.eq.'formatted') then
	    aformat='f'
	  else if(file_format.eq.'unformatted')then
	    aformat='u'
	  else
	    call sub_error('OPEN_INPUT_FILE')
	  end if
	end if
	inunit=nextunit()
	if(aformat.eq.'f')then
	  open(unit=inunit,file=infile,status='old',iostat=ierr)
	else
	  open(unit=inunit,file=infile,status='old',form=bin_unform,  &
          iostat=ierr)
	end if

! -- If the file could not be opened a test is made as to whether the file
!    is formatted or unformatted when the opposite was expected.

	if(ierr.ne.0)then
	  inquire(file=infile,unformatted=aaformat,iostat=ierr)
	  if(ierr.ne.0)go to 40
	  call casetrans(aaformat,'lo')
	  if(aaformat.eq.' ') go to 40
	  if(aformat.eq.'f')then
	    if(aaformat.eq.'yes') then
	      if(imessage.gt.5) then
		ifail=1
		return
	      end if
	      write(amessage,35) trim(infile)
35            format(' File ',a,' is an unformatted file  - try again.')
	      call write_message(increment=1)
	      go to 10
	    else
	      go to 40
	    end if
	  else
	    if(aaformat.eq.'no') then
	      if(imessage.gt.5) then
		ifail=1
		return
	      end if
	      write(amessage,36) trim(infile)
36            format(' File ',a,' is a formatted file  - try again.')
	      call write_message(increment=1)
	      go to 10
	    else
	      go to 40
	    end if
	  end if
40        if(imessage.gt.5) then
	    ifail=1
	    return
	  end if
	  write(amessage,50) trim(infile)
50        format(' Cannot open file ',a,'  - try again.')
	  call write_message(increment=1)
	  go to 10
	end if

	return

end subroutine open_input_file
!     Last change:  JD   23 Dec 2000    8:32 pm
subroutine open_named_input_file(ifail,aprompt,infile,inunit)

! -- Subroutine open_named_input_file opens an input file for which a default
!    name exists.

! -- Arguments are as follows:-
!       ifail:    returned as non-zero in case of failure
!       aprompt:  user prompt for filename
!       infile:   name of input file (carries default name on entry)
!       inunit:   unit number of input file

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	integer, intent(out)                    :: ifail
	character (len=*), intent(in)           :: aprompt
	character (len=*), intent(inout)        :: infile
	integer, intent(out)                    :: inunit
	integer                                 :: ierr,nbb,ifail1
	logical                                 :: lopened
	character (len=200)                     :: tempfile,atempf

! -- The user is prompted for the name of the file to open.

	imessage=0
	ifail=0
5       if(infile.eq.' ')then
10        write(6,'(a)',advance='no') aprompt(1:len_trim(aprompt)+1)
	  read(5,'(a)') tempfile
	  if(tempfile.eq.' ') go to 10
	  tempfile=adjustl(tempfile)
	  if(index(eschar,tempfile(1:2)).ne.0) then
	    escset=1
	    return
	  end if
          nbb=len_trim(tempfile)
          call getfile(ifail1,tempfile,atempf,1,nbb)
          if(ifail1.ne.0) go to 10
          tempfile=atempf
	else
	  write(6,'(a)',advance='no') aprompt(1:len_trim(aprompt)-1)//&
	  ' ['//trim(infile)//']: '
	  read(5,'(a)') tempfile
	  if(tempfile.eq.' ')then
	    tempfile=infile
	  else
	    tempfile=adjustl(tempfile)
	    if(index(eschar,tempfile(1:2)).ne.0) then
	      escset=1
	      return
	    end if
            nbb=len_trim(tempfile)
            call getfile(ifail1,tempfile,atempf,1,nbb)
            if(ifail1.ne.0) go to 5
            tempfile=atempf
	  end if
	end if

! -- Is the file already open?

	inquire(file=tempfile,opened=lopened)
	if(lopened)then
	  write(amessage,30) trim(tempfile)
30        format(' File ',a,' is already open  - try again.')
	  call write_message(increment=1)
	  go to 5
	end if

! -- The file is opened.

	inunit=nextunit()
	open(unit=inunit,file=tempfile,status='old',iostat=ierr)
	if(ierr.ne.0)then
	  if(imessage.gt.5) then
	    ifail=1
	    return
	  end if
	  write(amessage,50) trim(tempfile)
50        format(' Cannot open file ',a,'  - try again.')
	  call write_message(increment=1)
	  if(infile.eq.tempfile) infile=' '
	  go to 5
	end if
	infile=tempfile

	return

end subroutine open_named_input_file
subroutine factor(gridspec,east,north,fac1,fac2,fac3,fac4,&
	icellno,jcellno)

! -- Subroutine factor calculates interpolation factors for a bore.
!    There are four factors, one for each of the four cell centres
!    surrounding the bore. When interpolation takes place, the heads at the
!    respective cell centres are multiplied by these factors and added
!    together to determine the head at the bore.

! -- Subroutine arguments are as follows:-
!       gridspec:            defined variable holding grid specifications
!       east, north:         east and north coordinates of bore
!       fac1,fac2,fac3,fac4  interpolation factors for bore
!       icellno              identifier for cell in which bore is situated
!       jcellno              identifier for the cell centre to the top left
!                            of the bore

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	type(modelgrid), intent(in)             :: gridspec
	double precision, intent(in)            :: east,north
	real, intent(out)                       :: fac1,fac2,fac3,fac4
	integer, intent(out)                    :: icellno,jcellno

	integer :: i,irow,icol,jrow,jcol,nrow,ncol
	real :: x,y,etemp,ntemp,rtemp1,rtemp2, &
	x1,y1,x2,y2,delx,dely
	real, dimension(:),pointer :: delr,delc

	nrow=gridspec%nrow
	ncol=gridspec%ncol
	delr=>gridspec%delr
	delc=>gridspec%delc

! -- First the bore coordinates are expressed as local grid coordinates.

	etemp=east-gridspec%east_corner
	ntemp=north-gridspec%north_corner
	x=etemp*gridspec%cosang+ntemp*gridspec%sinang
	y=ntemp*gridspec%cosang-etemp*gridspec%sinang
	if((x.lt.0.0).or.(y.gt.0.0))then
	  icellno=-999
	  return
	end if

! -- The location of the bore within the finite-difference grid is next
!    determined.

	rtemp1=0.0
	rtemp2=delr(1)/2.0
	do i=1,ncol+1
	if(x.le.rtemp2)then
	  x1=x-rtemp1
	  x2=rtemp2-x
	  delx=rtemp2-rtemp1
	  jcol=i-1
	  if(i.eq.1)then
	    icol=1
	  else if(i.eq.ncol+1)then
	    icol=ncol
	  else
	    if(x1.le.delr(i-1)/2.0)then
	      icol=i-1
	    else
	      icol=i
	    end if
	  end if
	  go to 100
	end if
	rtemp1=rtemp2
	if(i.lt.ncol)then
	  rtemp2=rtemp2+(delr(i)+delr(i+1))/2.0
	else if(i.eq.ncol) then
	  rtemp2=rtemp2+delr(i)/2.0
	end if
	end do
	icellno=-999
	return

100     rtemp1=0.0
	rtemp2=-delc(1)/2.0
	do i=1,nrow+1
	if(y.ge.rtemp2)then
	  y1=rtemp1-y
	  y2=y-rtemp2
	  dely=rtemp1-rtemp2
	  jrow=i-1
	  if(i.eq.1)then
	    irow=1
	  else if(i.eq.nrow+1)then
	    irow=nrow
	  else if(y1.le.delc(i-1)/2.0)then
	    irow=i-1
	  else
	    irow=i
	  end if
	  go to 200
	end if
	rtemp1=rtemp2
	if(i.lt.nrow)then
	  rtemp2=rtemp2-(delc(i)+delc(i+1))/2.0
	else if(i.eq.nrow) then
	  rtemp2=rtemp2-delc(i)/2.0
	end if
	end do
	icellno=-999
	return

! -- The interpolation factors are calculated.

200     jcellno=(ncol+1)*jrow+jcol+1
	icellno=ncol*(irow-1)+icol
	rtemp1=1.0/delx/dely
	fac1=x2*y2*rtemp1
	fac2=x1*y2*rtemp1
	fac3=x2*y1*rtemp1
	fac4=x1*y1*rtemp1

	return

end subroutine factor
!     Last change:  JD   11 Apr 2003    4:53 am
subroutine read_bore_list_file(ifail,aprompt,coord_check)

! -- Subroutine read_bore_list_file reads a bore listing file, checking
! -- it for errors.

! -- Arguments are as follows:-
!       ifail:       returned as zero unless an error condition is encountered
!       aprompt:     contains text used to prompt for filename
!       coord_check  if supplied as 'yes' a check is made that listed bores
!                    are cited in a previously-read bore coordinates file

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	integer, intent(out)                    :: ifail
	character (len=*), intent(in)           :: aprompt
	character (len=*), intent(in), optional :: coord_check

	integer                         :: iunit,ierr,iline,imsg,nbore,i,idup,nbb,ifail1
	character (len=20)              :: aline
	character (len=200)             :: amsg,atempf
	character (len=10)              :: atemp
	logical                         :: check

	if(associated(bore_list_id)) deallocate (bore_list_id)
	nullify (bore_list_id)
	check=.true.
	if(present(coord_check))then
	  if(coord_check.eq.'no')check=.false.
	end if    

	idup=0
	imessage=0
	ifail=0
10      write(6,'(a)',advance='no') aprompt
	read(5,'(a)') bore_list_file
	bore_list_file=adjustl(bore_list_file)
	if(index(eschar,bore_list_file(1:2)).ne.0) then
	  escset=1
	  return
	end if
	if(bore_list_file.eq.' ') go to 10
        nbb=len_trim(bore_list_file)
        call getfile(ifail1,bore_list_file,atempf,1,nbb)
        if(ifail1.ne.0) go to 10
        bore_list_file=atempf
	iunit=nextunit()
	open(unit=iunit,file=bore_list_file,status='old',iostat=ierr)
	if(ierr.ne.0)then
	  if(imessage.eq.5) go to 9990
	  write(amessage,20) trim(bore_list_file)
20        format('cannot open bore listing file ',a,' - try again.')
	  call write_message(error='yes',increment=1)
	  go to 10
	end if
	write(initial_message,30) trim(bore_list_file)
30      format(' Errors in bore listing file ',a,' ----->')

! -- The bore listing file is first perused to count the number of entries.

	imessage=0
	iline=0
	nbore=0
	do
	  iline=iline+1
	  read(iunit,'(a)',err=9000,end=200) cline
	  call linesplit(ifail,1)
	  if(ifail.lt.0) cycle
!         if(ifail.gt.0) then
!           if(imessage.eq.0) call write_initial_message(leadspace='yes')
!           call num2char(iline,aline)
!           write(amessage,70) trim(aline)
!70          format('   Line ',a,': insufficient entries (minimum of one ', &
!           'entry required).')
!           call write_message(increment=1)
!           cycle
!         end if
	  nbore=nbore+1
	end do

200     if(nbore.eq.0) then
	  if(imessage.eq.0) call write_initial_message(leadspace='yes')
	  write(amessage,210)
210       format('   No bores read from file.')
	  call write_message
	  go to 9990
	end if
	if(imessage.ne.0) go to 9990

! -- The bore listing file is next re-read and data placed into memory.

	rewind(unit=iunit,err=9100)
	allocate(bore_list_id(nbore), stat=ierr)
	if(ierr.ne.0) then
	  write(amessage,250)
250       format(' Cannot allocate sufficient memory to run program.')
	  call write_message(leadspace='yes',endspace='yes')
	  go to 9990
	end if

	iline=0
	num_bore_list=0
	do
	  iline=iline+1
	  read(iunit,'(a)',end=365,err=9000) cline
	  call linesplit(ifail,1)
	  if(ifail.lt.0) cycle
	  num_bore_list=num_bore_list+1
	  call num2char(iline,aline)
	  write(amsg,280) trim(aline)
280       format('  Line ',a,':')
	  imsg=len_trim(amsg)+1
	  if(right_word(1)-left_word(1)+1.gt.10) then
	    if(imessage.eq.0) call write_initial_message(leadspace='yes')
	    write(amessage,310) amsg(1:imsg)
310         format(1x,a,'bore identifier greater than 10 characters in length.')
	    call write_message(increment=1)
	    bore_list_id(num_bore_list)='*'
	  else
	    bore_list_id(num_bore_list)=cline(left_word(1):right_word(1))
	    call casetrans(bore_list_id(num_bore_list),'hi')
	  end if
!!	  if(num_bore_list.ne.1)then
!!	    atemp=bore_list_id(num_bore_list)
!!	    if(atemp.eq.'*') go to 369
!!	    duplication: do i=1,num_bore_list-1
!!	      if(bore_list_id(i).eq.atemp) then
!               if(imessage.eq.0) call write_initial_message(leadspace='yes')
!               write(amessage,360) amsg(1:imsg), trim(atemp)
!360             format(1x,a,'bore ',a,' cited previously.')
!               call write_message(increment=1)
!!		idup=idup+1
!!		go to 367
!!	      end if
!!	    end do duplication
!!	  end if
369       continue
	  atemp=bore_list_id(num_bore_list)
	  if(atemp.eq.'*') go to 367
	  if(.not.check) go to 367
	  do i=1,num_bore_coord
	    if(bore_coord_id(i).eq.atemp) go to 367
	  end do
	  if(imessage.eq.0) call write_initial_message(leadspace='yes')
	  write(amessage,366) amsg(1:imsg),trim(atemp),trim(bore_coord_file)
366       format(1x,a,'no coordinates for bore ',a, &
	  ' found in bore coordinates file ',a,'.')
	  call write_message(increment=1)
367       continue
	end do

365     if(imessage.eq.0)then
	  call num2char(num_bore_list,aline)
	  write(amessage,370) trim(aline),trim(bore_list_file)
370       format('  - ',a,' bores read from bore listing file ',a)
	  call write_message
	  if(idup.ne.0)then
	    write(amessage,380)
380         format('    warning: duplicate listing of at least one bore.')
	    call write_message
	  end if
	  ifail=0
	  go to 9999
	end if
!       write(6,*)
	go to 9990

9000    call num2char(iline,aline)
	write(amessage,9010) trim(aline),trim(bore_list_file)
9010    format('Error reading line ',a,' of bore listing file ',a,'.')
	call write_message(leadspace='yes',endspace='yes')
	go to 9990
9100    write(amessage,9110) trim(bore_list_file)
9110    format(' Cannot rewind bore listing file ',a,'.')
	call write_message(leadspace='yes',endspace='yes')
	go to 9990
9990    ifail=1
9999    close(unit=iunit,iostat=ierr)
	imessage=0
	return

end subroutine read_bore_list_file
!     Last change:  JD   23 Dec 2000    8:28 pm
subroutine read_bore_coord_file(ifail,aprompt)

! -- Subroutine read_bore_coord_file reads a bore coordinates file, checking
! -- it for errors.

! -- Arguments are as follows:-
!       ifail:     returned as zero unless an error condition arises
!       prompt:    contains text used to prompt for filename

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	integer, intent(out)            :: ifail
	character (len=*), intent(in)   :: aprompt
	integer                         :: iunit,ierr,iline,imsg,nbore,i,ilay,nbb,ifail1
	character (len=20)              :: aline
	character (len=200)             :: amsg,atempf
	character (len=10)              :: atemp

	if(associated(bore_coord_id)) deallocate (bore_coord_id)
	if(associated(bore_coord_east)) deallocate (bore_coord_east)
	if(associated(bore_coord_north)) deallocate (bore_coord_north)
	if(associated(bore_coord_layer)) deallocate (bore_coord_layer)
	nullify(bore_coord_id,bore_coord_east,bore_coord_north,bore_coord_layer)

	imessage=0
	ifail=0
10      if(bore_coord_file.eq.' ')then
	  write(6,'(a)',advance='no') aprompt
	else
	  write(6,'(a)',advance='no') aprompt(1:len(aprompt)-2)//' [' &
	  //trim(bore_coord_file)//']: '
	end if
	read(5,'(a)') amsg
	amsg=adjustl(amsg)
	if(index(eschar,amsg(1:2)).ne.0) then
	  escset=1
	  return
	end if
	if(amsg.eq.' ') then
	  if(bore_coord_file.eq.' ') go to 10
	  amsg=bore_coord_file
	else
          nbb=len_trim(amsg)
          call getfile(ifail1,amsg,atempf,1,nbb)
          if(ifail1.ne.0) go to 10
          amsg=atempf
        end if
	iunit=nextunit()
	open(unit=iunit,file=amsg,status='old',iostat=ierr)
	if(ierr.ne.0)then
	  if(imessage.eq.5) go to 9990
	  write(amessage,20) trim(amsg)
20        format('cannot open bore coordinates file ',a,' - try again.')
	  call write_message(error='yes',increment=1)
	  if(amsg.eq.bore_coord_file) bore_coord_file=' '
	  go to 10
	end if
	bore_coord_file=amsg
	write(initial_message,30) trim(bore_coord_file)
30      format(' Errors in bore coordinates file ',a,' ----->')

! -- The bore coordinates file is first perused to count the number of entries.

	imessage=0
	iline=0
	nbore=0
	do
	  if(imessage.gt.40) go to 9990
	  iline=iline+1
	  read(iunit,'(a)',err=9000,end=200) cline
	  call linesplit(ifail,3)
	  if(ifail.lt.0) cycle
	  if(ifail.gt.0) then
	    if(imessage.eq.0) call write_initial_message(leadspace='yes')
	    call num2char(iline,aline)
	    write(amessage,70) trim(aline)
70          format('   Line ',a,': insufficient entries (minimum of three ', &
	    'entries required).')
	    call write_message(increment=1)
	    cycle
	  end if
	  nbore=nbore+1
	end do

200     if(nbore.eq.0) then
	  if(imessage.eq.0) call write_initial_message(leadspace='yes')
	  write(amessage,210)
210       format('   No bore coordinates read from file.')
	  call write_message
	  go to 9990
	end if
	if(imessage.ne.0) go to 9990

! -- The bore coordinates file is next re-read and data placed into memory.

	if(nbore.gt.1000) write(6,220)
220     format(' Working.....')
	rewind(unit=iunit,err=9100)
	allocate(bore_coord_id(nbore),bore_coord_east(nbore), &
	bore_coord_north(nbore),bore_coord_layer(nbore),stat=ierr)
	if(ierr.ne.0) then
	  write(amessage,250)
250       format(' Cannot allocate sufficient memory to run program.')
	  call write_message(leadspace='yes',endspace='yes')
	  go to 9990
	end if

	iline=0
	num_bore_coord=0
	do
	  if(imessage.gt.40) go to 9990
	  iline=iline+1
	  read(iunit,'(a)',end=365,err=9000) cline
	  call linesplit(ifail,4)
	  if(ifail.lt.0) cycle
	  if(ifail.gt.0) then
	    ilay=0
	    call linesplit(ifail,3)
	  else
	    ilay=1
	  end if
	  num_bore_coord=num_bore_coord+1
	  call num2char(iline,aline)
	  write(amsg,280) trim(aline)
280       format('  Line ',a,':')
	  imsg=len_trim(amsg)+1
	  if(right_word(1)-left_word(1)+1.gt.10) then
	    if(imessage.eq.0) call write_initial_message(leadspace='yes')
	    write(amessage,310) amsg(1:imsg)
310         format(1x,a,'bore identifier greater than 10 characters in length.')
	    call write_message(increment=1)
	    bore_coord_id(num_bore_coord)='*'
	  else
	    bore_coord_id(num_bore_coord)=cline(left_word(1):right_word(1))
	    call casetrans(bore_coord_id(num_bore_coord),'hi')
	  end if
	  bore_coord_east(num_bore_coord)=char2double(ifail,2)
	  if(ifail.ne.0)then
	    if(imessage.eq.0) call write_initial_message(leadspace='yes')
	    write(amessage,330) amsg(1:imsg)
330         format(1x,a,'cannot read bore east coordinate.')
	    call write_message(increment=1)
	  end if
	  bore_coord_north(num_bore_coord)=char2double(ifail,3)
	  if(ifail.ne.0)then
	    if(imessage.eq.0) call write_initial_message(leadspace='yes')
	    write(amessage,350) amsg(1:imsg)
350         format(1x,a,'cannot read bore north coordinate.')
	    call write_message(increment=1)
	  end if
	  if(ilay.eq.0) then
	    bore_coord_layer(num_bore_coord)=-999
	  else
	    bore_coord_layer(num_bore_coord)=char2int(ifail,4)
	    if(ifail.ne.0) then
	      if(imessage.eq.0) call write_initial_message (leadspace='yes')
	      write(amessage,380) amsg(1:imsg)
380           format(1x,a,'cannot read bore layer number.')
	      call write_message(increment=1)
	    else
	      if(bore_coord_layer(num_bore_coord).le.0)then
		if(imessage.eq.0) call write_initial_message (leadspace='yes')
		write(amessage,390) amsg(1:imsg)
390             format(1x,a,'illegal bore layer number.')
		call write_message(increment=1)
	      end if
	    end if
	  end if
	  if(num_bore_coord.ne.1)then
	    atemp=bore_coord_id(num_bore_coord)
	    if(atemp.eq.'*') go to 369
	    duplication: do i=1,num_bore_coord-1
	      if(bore_coord_id(i).eq.atemp) then
		if(imessage.eq.0) call write_initial_message(leadspace='yes')
		write(amessage,360) amsg(1:imsg), trim(atemp)
360             format(1x,a,'bore ',a,' cited previously.')
		call write_message(increment=1)
		exit duplication
	      end if
	    end do duplication
	  end if
369       continue
	end do

365     if(imessage.eq.0)then
	  call num2char(num_bore_coord,aline)
	  write(amessage,370) trim(aline),trim(bore_coord_file)
370       format('  - ',a,' bores and coordinates read from bore ',&
	  'coordinates file ',a)
	  call write_message
	  ifail=0
	  go to 9999
	end if
	go to 9990

9000    call num2char(iline,aline)
	write(amessage,9010) trim(aline),trim(bore_coord_file)
9010    format('Error reading line ',a,' of bore coordinates file ',a,'.')
	call write_message(leadspace='yes',endspace='yes')
	go to 9990
9100    write(amessage,9110) trim(bore_coord_file)
9110    format(' Cannot rewind bore coordinates file ',a,'.')
	call write_message(leadspace='yes',endspace='yes')
	go to 9990
9990    ifail=1
9999    close(unit=iunit,iostat=ierr)
	imessage=0
	return

end subroutine read_bore_coord_file
subroutine close_spec_file(gridspec,ok)

! -- Subroutine close_spec_file closes the grid specification file.

! -- Arguments are as follows:-
!       gridspec:     defined type holding grid specification information 
!                     (including unit number)
!       ok:           if supplied as "yes" a message is written to the screen

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	type (modelgrid)                :: gridspec
	logical                         :: lopen
	character (len=*),optional      :: ok

	inquire(unit=gridspec%specunit, opened=lopen)
	if(lopen) close(unit=gridspec%specunit)
	if(present(ok))then
	  if(ok.eq.'yes')then
	    write(amessage,10) trim(gridspec%specfile)
10          format('  - grid specifications read from file ',a)
	    call write_message
	  endif
	endif
	return

end subroutine close_spec_file
subroutine read_spec_data(ifail,gridspec)

! -- Subroutine read_spec_data reads grid specifications (except row and
!    column numbers) from a grid specification file.

! -- Arguments are as follows:-
!       ifail:     returned as zero unless an error condition arises
!       gridspec:  defined type holding grid specifications

! -- Revision history:-
!       May-June, 1995: version 1.

	use defn
	use inter

	type (modelgrid)        :: gridspec
	integer, intent(out)    :: ifail
	character (len=8)       :: anum
	integer                 :: ierr

	ifail=0
10      gridspec%specline=gridspec%specline+1
	read(gridspec%specunit,'(a)',end=150) cline
	call linesplit(ifail,3)
	if(ifail.lt.0) go to 10
	if(ifail.gt.0) then
	  call num2char(gridspec%specline,anum)
	  write(amessage,20) trim(anum),trim(gridspec%specfile)
20        format('insufficient data items on line ',a,' of file ',a,'.')
	  call write_message(error='yes',leadspace='yes')
	  go to 500
	end if
	gridspec%east_corner=char2double(ifail,1)
	if(ifail.ne.0) go to 150
	gridspec%north_corner=char2double(ifail,2)
	if(ifail.ne.0) go to 150
	gridspec%rotation = char2double(ifail,3)
	if(ifail.ne.0) go to 170
	gridspec%cosang=cos(gridspec%rotation*3.14159265/180.0)
	gridspec%sinang=sin(gridspec%rotation*3.14159265/180.0)

	allocate(gridspec%delr(1:gridspec%ncol),&
	gridspec%delc(1:gridspec%nrow),stat=ierr)
	if(ierr.ne.0)then
	  write(6,50)
50        format(/,' *** Cannot allocate sufficient memory to run program ***',/)
	  ifail=1
	  return
	end if

	read(gridspec%specunit,*,err=400,end=400) gridspec%delr
	read(gridspec%specunit,*,err=400,end=400) gridspec%delc
	if(any(gridspec%delr.le.0.0)) go to 450
	if(any(gridspec%delc.le.0.0)) go to 450
	return

150     call num2char(gridspec%specline,anum)
	write(amessage,160) trim(anum),trim(gridspec%specfile)
160     format('cannot read grid corner coordinates from line ',a, &
	' of file ',a,'.')
	call write_message(error='yes',leadspace='yes')
	go to 500
170     call num2char(gridspec%specline,anum)
	write(amessage,180) trim(anum),trim(gridspec%specfile)
180     format('cannot read grid rotation from line ',a,' of file ',a,'.')
	call write_message(error='yes',leadspace='yes')
	go to 500
400     write(amessage,410) trim(gridspec%specfile)
410     format('cannot read DELR and/or DELC vector from grid specification ',&
	'file ',a,'.')
	call write_message(error='yes',leadspace='yes')
	go to 500
450     write(amessage,460) trim(gridspec%specfile)
460     format('grid cell row or column width less than or equal to zero in ',&
	'file ',a)
	call write_message(error='yes',leadspace='yes')
	go to 500

500     ifail=1
	return

end subroutine read_spec_data
subroutine read_spec_dim(ifail,gridspec)

! -- Subroutine read_spec_dim reads the grid row and column numbers from the
!    first line of a grid specification file.

! -- Arguments are as follows:-
!       ifail:     returned as zero unless an error condition arises
!       gridspec:  defined type variable holding grid specifications

!    Revision history:-
!       June-November, 1995: verson 1.

	use defn
	use inter

	type (modelgrid)                :: gridspec
	integer, intent(out)            :: ifail
	character (len=8)               :: anum

	gridspec%specline=0
	ifail=0
10      gridspec%specline=gridspec%specline+1
	read(gridspec%specunit,'(a)',end=100) cline
	call linesplit(ifail,2)
	if(ifail.lt.0) go to 10
	if(ifail.gt.0) then
	  call num2char(gridspec%specline,anum)
	  write(amessage,20) trim(anum),trim(gridspec%specfile)
20        format('insufficient items on line ',a,' of file ',a,'.')
	  call write_message(error='yes',leadspace='yes')
	  go to 200
	end if
	gridspec%nrow=char2int(ifail,1)
	if(ifail.ne.0) go to 130
	gridspec%ncol=char2int(ifail,2)
	if(ifail.ne.0) go to 130
	return

100     write(amessage,110) trim(gridspec%specfile)
110     format('grid dimensions missing from file ',a,'.')
	call write_message(error='yes',leadspace='yes')
	go to 200
130     call num2char(gridspec%specline,anum)
	write(amessage,140) trim(anum),trim(gridspec%specfile)
140     format('cannot read grid dimensions from line ',a,' of file ',a,'.')
	call write_message(error='yes',leadspace='yes')
	go to 200

200     ifail=1
	return

end subroutine read_spec_dim
!     Last change:  JD   26 Dec 2000    1:12 pm
subroutine spec_open(ifail,gridspec)

! -- Subroutine spec_open opens a grid specification file.

! -- Arguments are as follows:-
!       ifail:     returned as zero unless error condition arises.
!       gridspec:  defined variable containing grid specifications.

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	type (modelgrid)        :: gridspec
	integer, intent(out)    :: ifail
	integer                 :: ierr,jstep,nbb
	character (len=120)      :: atemp,aatemp,bbtemp

	ifail=0
	atemp=gridspec%specfile
	imessage=0
5       jstep=0
	do
	  jstep=jstep+1
	  if(jstep.ge.5) then
	    ifail=1
	    return
	  end if
10        if(atemp.ne.' ') then
	    write(6,20,advance='no') trim(atemp)
20          format(' Enter name of grid specification file [',a,']: ')
	  else
	    write(6,30,advance='no')
30          format(' Enter name of grid specification file: ')
	  endif
	  read(5,'(a)',err=10) aatemp
          nbb=len_trim(aatemp)
          if(aatemp.ne.' ')then
            call getfile(ifail,aatemp,bbtemp,1,nbb)
            if(ifail.ne.0) go to 10
          else
            bbtemp=' '
          end if
          gridspec%specfile=bbtemp
	  if(index(eschar,gridspec%specfile(1:2)).ne.0) then
	    escset=1
	    return
	  end if
	  if(gridspec%specfile.ne.' ') exit
	  if(atemp.ne.' ')then
	    gridspec%specfile=atemp
	    exit
	  end if
	end do

	gridspec%specunit=nextunit()
	open(unit=gridspec%specunit,file=gridspec%specfile,status='old',&
	iostat=ierr)
	if(ierr.ne.0) then
	  if(imessage.eq.5) then
	    ifail=1
	    return
	  end if
	  write(amessage,50) trim(gridspec%specfile)
50        format('cannot open file ',a,' - try again.')
	  call write_message(error='yes',increment=1)
	  if(gridspec%specfile.eq.atemp) atemp=' '
	  go to 5
	end if
	ifail=0
	return

end subroutine spec_open
!     Last change:  JD   21 Dec 2000    2:33 pm
subroutine readfig(specfile,coordfile,sampfile,pumpfile,pilotfile)

! -- Subroutine readfig reads a filename file.

! -- Arguments are as follows:-
!       specfile:  name of grid specification file
!       coordfile: name of bore coordinates file
!       sampfile:  name of bore sample file
!       pumpfile:  name of bore pumping file


	use defn
	use inter

	character (len=*),intent(out)           :: specfile
	character (len=*),intent(out),optional  :: coordfile,sampfile,pumpfile,pilotfile
	integer                                 :: iunit,ierr,iequals,nb,ifail
        character (len=100)                     :: filename

	specfile=' '
	if(present(coordfile)) coordfile=' '
	if(present(sampfile)) sampfile=' '
	if(present(pumpfile)) pumpfile=' '
        if(present(pilotfile)) pilotfile=' '

	iunit=nextunit()
	open(unit=iunit,file='files.fig',status='old',err=200)
	do
	  read(iunit,'(a)',err=100,end=200) cline
          cline=adjustl(cline)
	  iequals=index(cline,'=')
	  if(iequals.le.1) cycle
          nb=len_trim(cline)
          if(nb.eq.iequals) return
          call getfile(ifail,cline,filename,iequals+1,nb)
          if(ifail.ne.0) cycle
	  call casetrans(cline(1:iequals),'lo')
	  if(cline(1:10).eq.'grid_speci') then
	    specfile=filename
	  else if(cline(1:10).eq.'bore_coord') then
	    if(present(coordfile)) then
	      coordfile=filename
	    end if
	  else if(cline(1:10).eq.'bore_sampl') then
	    if(present(sampfile)) then
	      sampfile=filename
	    end if
	  else if(cline(1:10).eq.'bore_pumpi') then
	    if(present(pumpfile)) then
	      pumpfile=filename
	    end if
          else if(cline(1:12).eq.'pilot_points')then
            if(present(pilotfile))then
              pilotfile=filename
            end if
	  end if
100       continue
	end do

200     close(unit=iunit,iostat=ierr)
	return

end subroutine readfig

!     Last change:  JD   16 Dec 2000    0:53 am
subroutine read_settings(ifail,idate,iheader)

! -- Subroutine read_settings reads a settings file located in the current
! -- directory.

! -- Arguments are as follows:-
!      ifail:    returned as zero unless settings file cannot be read
!      idate:    returned as zero unless date format is incorrect in settings
!                file
!      iheader:  returned as zero unless header specifier is incorrect


	use defn
	use inter

	integer, intent(out)			:: ifail,idate,iheader
	integer                                 :: iunit,ierr,iequals,i
	character (len=40)                      :: aline

	ifail=0
	idate=0
        iheader=0
	datespec=0
        headerspec=' '

	iunit=nextunit()
	open(unit=iunit,file='settings.fig',status='old',iostat=ierr)
	if(ierr.ne.0) then
	  ifail=1
	  return
	end if

	do
	  read(iunit,'(a)',err=100,end=200) cline
	  call casetrans(cline,'lo')
	  iequals=index(cline,'=')
	  if(iequals.le.1) cycle
	  aline=cline(1:iequals-1)
	  aline=adjustl(aline)
	  if(aline(1:4).eq.'date') then
	    aline=cline(iequals+1:)
	    aline=adjustl(aline)
	    if((aline(1:2).eq.'dd').and.(aline(4:5).eq.'mm')) then
	      datespec=1
	    else if((aline(1:2).eq.'mm').and.(aline(4:5).eq.'dd')) then
	      datespec=2
	    else
	      idate=1
	    end if
	  else if(aline(1:6).eq.'colrow') then
	    aline=cline(iequals+1:)
            do i=1,len_trim(aline)
              if(aline(i:i).eq.'''')aline(i:i)=' '
            end do
	    aline=adjustl(aline)
            call casetrans(aline,'lo')
            if(aline(1:3).eq.'yes')then
              headerspec='yes'
            else if(aline(1:2).eq.'no')then
              headerspec='no '
            else
              iheader=1
            end if
          end if
	end do
	go to 200

100	ifail=2
200     close(unit=iunit,iostat=ierr)
	return

end subroutine read_settings

subroutine read_rest_of_sample_line(ifail,cols,ndays,nsecs,value,iline,sampfile)

! -- Subroutine read_rest_of_sample_line reads the date, time, value and
!    optional fifth column from a line of a bore sample file.

! -- Arguments are as follows:-
!       ifail:     returned as zero unless an error condition is encountered
!       cols:      number of data columns in the line
!       ndays:     number of days from 1/1/1970 until sample date
!       nsecs:     number of seconds from midnight until sample time
!       value:     sample value
!       iline:     current line number of bore sample file
!       sampfile:  name of bore sample file

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	integer, intent(out)            :: ifail
	integer, intent(in)             :: cols
	integer, intent(out)            :: ndays,nsecs
	double precision, intent(out)   :: value
	integer, intent(in)             :: iline
	character (len=*), intent(in)   :: sampfile
	integer                         :: dd,mm,yy,hhh,mmm,sss
	character (len=15)              :: aline
	character (len=2)               :: aa

	ifail=0
	call char2date(ifail,cline(left_word(2):right_word(2)),dd,mm,yy)
	if(ifail.ne.0) then
	  call num2char(iline,aline)
	  write(amessage,150) trim(aline),trim(sampfile)
150       format('illegal date at line ',a,' of bore sample file ',a)
	  call write_message(error='yes',leadspace='yes')
	  go to 9800
	end if
	ndays=numdays(1,1,1970,dd,mm,yy)

	call char2time(ifail,cline(left_word(3):right_word(3)),hhh,mmm,sss)
	if(ifail.ne.0) then
	  call num2char(iline,aline)
	  write(amessage,160) trim(aline),trim(sampfile)
160       format('illegal time at line ',a,' of bore sample file ',a)
	  call write_message(error='yes',leadspace='yes')
	  go to 9800
	end if
	nsecs=numsecs(0,0,0,hhh,mmm,sss)

	value=char2double(ifail,4)
	if(ifail.ne.0)then
	  call num2char(iline,aline)
	  write(amessage,180) trim(aline),trim(sampfile)
180       format('cannot read sample value at line ',a,' of bore sample file ',a)
	  call write_message(error='yes',leadspace='yes')
	  go to 9800
	end if
	if(value.lt.-1.0e37) then
	  call num2char(iline,aline)
	  write(amessage,190) trim(aline),trim(sampfile)
190       format('illegal sample value at line ',a,' of bore sample file ',a, &
	  '; lower limit is -1.0E37.')
	  call write_message(error='yes',leadspace='yes')
	  go to 9800
	end if
	if(cols.eq.5)then
	  aa=cline(left_word(5):right_word(5))
	  call casetrans(aa,'lo')
	  if(aa.eq.'x ') then
	    value=-1.1e38
	  else
	    call num2char(iline,aline)
	    write(amessage,210) trim(aline),trim(sampfile)
210         format('illegal optional fifth item on line ',a,' of bore sample ',&
	    'file ',a,'; item must be "x" if present.')
	    call write_message(error='yes',leadspace='yes')
	    go to 9800
	  end if
	end if
	return

9800    ifail=1
	return

end subroutine read_rest_of_sample_line
subroutine write_message(increment,iunit,error,leadspace,endspace)

! -- Subroutine write_message formats and writes a message.

! -- Arguments are as follows:-
!       increment:  the increment to the message counter
!       iunit:      the unit number to which the message is written
!       error:      if "yes" precede message with "Error"
!       leadspace   if "yes" precede message with blank line
!       endspace    if "yes" follow message by blank line

! -- Revision history:-
!       June-November, 1995: version 1.

	use defn
	use inter

	integer, intent(in), optional           ::increment,iunit
	integer                                 ::jend,i,nblc,junit,leadblank
	integer                                 ::itake,j
	character (len=*), intent(in), optional ::error,leadspace,endspace
	character (len=20) ablank

	ablank=' '
	itake=0
	j=0
	if(present(increment)) imessage=imessage+increment
	if(present(iunit))then
	  junit=iunit
	else
	  junit=6
	end if
	if(present(leadspace))then
	  if(leadspace.eq.'yes') write(junit,*)
	endif
	if(present(error))then
	  if(index(error,'yes').ne.0)then
	    nblc=len_trim(amessage)
	    amessage=adjustr(amessage(1:nblc+8))
	    if(nblc+8.lt.len(amessage)) amessage(nblc+9:)=' '
	    amessage(1:8)=' Error: '
	  end if
	end if

	do i=1,20
	  if(amessage(i:i).ne.' ')exit
20      end do
	leadblank=i-1
	nblc=len_trim(amessage)
5       jend=j+78-itake
	if(jend.ge.nblc) go to 100
	do i=jend,j+1,-1
	if(amessage(i:i).eq.' ') then
	  if(itake.eq.0) then
	     write(junit,'(a)',err=200) amessage(j+1:i)
	     itake=2+leadblank
	  else
	     write(junit,'(a)',err=200) ablank(1:leadblank+2)//amessage(j+1:i)
	  end if
	  j=i
	  go to 5
	end if
	end do
	if(itake.eq.0)then
	  write(junit,'(a)',err=200) amessage(j+1:jend)
	  itake=2+leadblank
	else
	  write(junit,'(a)',err=200) ablank(1:leadblank+2)//amessage(j+1:jend)
	end if
	j=jend
	go to 5
100     jend=nblc
	if(itake.eq.0)then
	write(junit,'(a)',err=200) amessage(j+1:jend)
	  else
	write(junit,'(a)',err=200) ablank(1:leadblank+2)//amessage(j+1:jend)
	  end if
	if(present(endspace))then
	  if(endspace.eq.'yes') write(junit,*)
	end if
	return

200     call exit(100)

end subroutine write_message
logical function leap(year)

! -- Function LEAP returns .true. if a year is a leap year.

! -- Revision history:-
!       June-November, 1995: version 1.

	integer, intent(in)     :: year

        leap = ( mod(year,4).eq.0 .and. mod(year,100).ne.0 ) .or. &
               ( mod(year,400).eq.0 .and. year.ne.0 )

	return
end function leap
subroutine sub_error(subname)

! -- Subroutine sub_error names the subroutine causing a run-time error.

! -- Arguments are as follows:-
!       subname:  name of offending subroutine

! -- Revision history:-
!       June-November, 1995: version 1.

	character (len=*)               ::subname

	write(6,10) trim(subname)
10      format(/,' *** PROGRAMMING ERROR CALLING SUBROUTINE ',a,' ***')
	stop

end subroutine sub_error
integer function nextunit()

! -- Function nextunit determines the lowest unit number available for
! -- opening.

! -- Revision history:-
!       June-November, 1995: version 1.

	logical::lopen

	do nextunit=10,100
	  inquire(unit=nextunit,opened=lopen)
	  if(.not.lopen) return
	end do
	write(6,10)
10      format(' *** No more unit numbers to open files ***')
	stop

end function nextunit
!     Last change:  JD   29 Jan 2001    2:11 pm

subroutine getfile(ifail,cline,filename,ibeg,iend)

! Subroutine getfile extracts a filename from a string.

! -- Arguments are as follows:-
!       ifail: returned as zero if filename successfully read
!       cline: a character string containing the file name
!       filename: the name of the file read from the string
!       ibeg: character position at which to begin search for filename
!       iend: on input  - character position at which to end search for filename
!             on output - character postion at which filename ends


        integer, intent(out)               :: ifail
        integer, intent(in)                :: ibeg
        integer, intent(inout)             :: iend
        character (len=*), intent(in)      :: cline
        character (len=*), intent(out)     :: filename

        integer                            :: i,j,k
        character (len=1)                  :: aa

        ifail=0
        do i=ibeg,iend
          aa=cline(i:i)
          if((aa.ne.' ').and.(aa.ne.',').and.(aa.ne.char(9)))go to 50
        end do
        ifail=1
        return

50      if((aa.eq.'"').or.(aa.eq.''''))then
          do j=i+1,iend
            if(cline(j:j).eq.aa) go to 60
          end do
          ifail=1
          return
60        iend=j
          if(i+1.gt.j-1)then
            ifail=1
            return
          else
            filename=cline(i+1:j-1)
          end if
        else
          do j=i+1,iend
            if((cline(j:j).eq.' ').or.(cline(j:j).eq.',').or.(cline(j:j).eq.char(9)))then
              k=j-1
              go to 100
            end if
          end do
          k=iend
100       filename=cline(i:k)
          if(cline(k:k).eq.'"')then
            ifail=1
            return
          else if(cline(k:k).eq.'''')then
            ifail=1
            return
          end if

          iend=k
        end if
        filename=adjustl(filename)
        return

end subroutine getfile


subroutine write_initial_message(leadspace,endspace)

! -- Subroutine write_initial_message writes the first part of a two-part
!    error message.

! -- Arguments are as follows:-
!       leadspace:  if "yes" write a leading space to message
!       endspace:   if "no" write a trailing space to message

! -- Revision history:-
!       June-November, 1995: version1. 

	use defn
	use inter

	character (len=*), intent(in), optional :: leadspace,endspace
	character (len=500)     :: atemp

	atemp=amessage
	amessage=initial_message
	call write_message(leadspace=leadspace,endspace=endspace)
	amessage=atemp

end subroutine write_initial_message
